/* Evlan Client - Command shell for the Evlan programming language.
 * Copyright (C) 2004-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.*;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;
import java.security.MessageDigest;

public class Dialogs
{
   public static class ConnectInfo
   {
      public final String host;
      public final int port;
      public final boolean useSsl;

      public ConnectInfo(String host, int port, boolean useSsl)
      {
         this.host = host;
         this.port = port;
         this.useSsl = useSsl;
      }
   }

   public static class LoginInfo
   {
      public final String username;
      public final byte[] passhash;

      public LoginInfo(String username, byte[] passhash)
      {
         this.username = username;
         this.passhash = passhash;
      }
   }

   public static ConnectInfo showConnectDialog(Component parent, String host, int port, boolean useSsl)
   {
      JTextField hostField = new JTextField();
      JTextField portField = new JTextField();
      JCheckBox sslField = new JCheckBox("SSL mode", useSsl);

      if(host != null)
         hostField.setText(host);
      if(port != -1)
         portField.setText(String.valueOf(port));

      JComponent component = new JPanel();
      component.setLayout(new GridLayout(3, 2));
      component.add(new JLabel("host:"));
      component.add(hostField);
      component.add(new JLabel("port:"));
      component.add(portField);
      component.add(new JLabel(""));
      component.add(sslField);
      JOptionPane.showMessageDialog(parent, component, "Connect", JOptionPane.PLAIN_MESSAGE);

      port = -1;
      try {port = Integer.parseInt(portField.getText());} catch(Exception e) {}
      return new ConnectInfo(hostField.getText(), port, sslField.isSelected());
   }

   public static LoginInfo showLoginDialog(Component parent, String prefix)
   {
      JTextField usernameField = new JTextField("administrator");
      JTextField passwordField = new JPasswordField();

      JComponent component = new JPanel();
      component.setLayout(new GridLayout(2, 2));
      component.add(new JLabel("username:"));
      component.add(usernameField);
      component.add(new JLabel("password:"));
      component.add(passwordField);
      JOptionPane.showMessageDialog(parent, component, "Log In", JOptionPane.PLAIN_MESSAGE);

      String username = usernameField.getText();
      byte[] passhash = null;
      String password = passwordField.getText();

      try
      {
         passhash = hashPassword(prefix, username, password);
      }
      catch(java.security.NoSuchAlgorithmException e)
      {
         JOptionPane.showMessageDialog(parent, "Java implementation missing SHA-1 hash algorithm.", "Fatal Error", JOptionPane.ERROR_MESSAGE);
         System.exit(1);
      }
      catch(java.io.UnsupportedEncodingException e)
      {
         JOptionPane.showMessageDialog(parent, "Java implementation missing UTF-8 encoding.", "Fatal Error", JOptionPane.ERROR_MESSAGE);
         System.exit(1);
      }

      return new LoginInfo(username, passhash);
   }

   public static byte[] showPasswordDialog(Component parent, String prefix, String username)
   {
      JTextField passwordField = new JPasswordField();
      JTextField confirmField = new JPasswordField();

      JComponent component = new JPanel();
      component.setLayout(new GridLayout(3, 2));
      component.add(new JLabel("username:"));
      component.add(new JLabel(username));
      component.add(new JLabel("password:"));
      component.add(passwordField);
      component.add(new JLabel("confirm:"));
      component.add(confirmField);

      String password, confirmed;

      JOptionPane.showMessageDialog(parent, component, "Change Password", JOptionPane.PLAIN_MESSAGE);
      password = passwordField.getText();
      confirmed = confirmField.getText();

      JComponent box = Box.createVerticalBox();
      box.add(new JLabel("Passwords didn't match; try again."));
      box.add(component);

      while(!password.equals(confirmed))
      {
         passwordField.setText("");
         confirmField.setText("");
         JOptionPane.showMessageDialog(parent, box, "Change Password", JOptionPane.PLAIN_MESSAGE);
         password = passwordField.getText();
         confirmed = confirmField.getText();
      }

      byte[] passhash = null;

      try
      {
         passhash = hashPassword(prefix, username, password);
      }
      catch(java.security.NoSuchAlgorithmException e)
      {
         JOptionPane.showMessageDialog(parent, "Java implementation missing SHA-1 hash algorithm.", "Fatal Error", JOptionPane.ERROR_MESSAGE);
         System.exit(1);
      }
      catch(java.io.UnsupportedEncodingException e)
      {
         JOptionPane.showMessageDialog(parent, "Java implementation missing UTF-8 encoding.", "Fatal Error", JOptionPane.ERROR_MESSAGE);
         System.exit(1);
      }

      return passhash;
   }


   static String stripFormatting(String name)
   {
      char[] buf = new char[name.length()];

      int pos = 0;
      char c;
      for(int i = 0; i < buf.length; i++)
      {
         c = name.charAt(i);
         if(Character.isLetterOrDigit(c))
            buf[pos++] = c;
      }
      return new String(buf, 0, pos);
   }

   static byte[] hashPassword(String prefix, String username, String password)
      throws java.security.NoSuchAlgorithmException, java.io.UnsupportedEncodingException
   {
      MessageDigest hasher = MessageDigest.getInstance("SHA-1");
      return hasher.digest((prefix + " " + stripFormatting(username) + " " + password.length() + " " + password).getBytes("UTF-8"));
   }
}
