/* Evlan Client - Command shell for the Evlan programming language.
 * Copyright (C) 2004-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.*;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;
import java.io.*;
import java.net.Socket;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.SSLException;
import java.util.*;
import java.security.MessageDigest;

public class Gui extends JComponent
                 implements KeyListener, ChangeListener, Runnable
{
   boolean mIsApplet;

   String mHost;
   Object mMutex;
   Vector mCommands;
   boolean mInHistory;
   int mHistoryPosition;
   int mAcknowledgeCounter;
   Set mKeywords;
   Map mDelays;

   TransferManager mConnection;
   Thread mListenThread;
   boolean mIsStopping;

   JFileChooser mFileChooser;
   JEditorPane mLogArea;
   JScrollPane mLogScroll;
   JTextArea mEditArea;
   JScrollPane mEditScroll;
   int mOldHeight;
   boolean mScrollAtEnd;

   // Gui ============================================================
   public Gui(boolean isApplet)
   {
      mIsApplet = isApplet;
      mHost = "";

      mMutex = new Object();
      mCommands = new Vector();
      mInHistory = false;
      mHistoryPosition = 0;
      mAcknowledgeCounter = 0;
      mDelays = new Hashtable();
      mKeywords = new HashSet();
      mKeywords.add("and");
      mKeywords.add("array");
      mKeywords.add("catch");
      mKeywords.add("do");
      mKeywords.add("else");
      mKeywords.add("false");
      mKeywords.add("finally");
      mKeywords.add("if");
      mKeywords.add("import");
      mKeywords.add("not");
      mKeywords.add("object");
      mKeywords.add("of");
      mKeywords.add("or");
      mKeywords.add("return");
      mKeywords.add("then");
      mKeywords.add("throw");
      mKeywords.add("true");
      mKeywords.add("try");
      mKeywords.add("where");

      mLogArea = new JEditorPane("text/html", "<html><body><p id=\"end\"></p></body></html>");
      StyleSheet style = ((HTMLDocument)mLogArea.getDocument()).getStyleSheet();
      //this is sickeningly high-level...
      style.addRule("body {font-family: monospace; font-size: 10px;}");
      style.addRule("p {margin: 0px;}");
      style.addRule("pre {margin: 0px;}");
      style.addRule(".document p {margin-bottom: 6px;}");
      style.addRule(".document pre {margin-bottom: 6px;}");
      style.addRule("div {margin-bottom: 5px; border-style: solid; border-width: 1px;}");
      style.addRule("table {margin-bottom: 5px;}");
      style.addRule("td {padding: 0px; padding-left: 2px; padding-right: 2px; border-style: solid; border-width: 1px;}");

      style.addRule(".linenumber {background-color: #FFFFFF; border-color: #808080;}");

      style.addRule(".echo     {background-color: #E0E0E0; border-color: #808080;}");
      style.addRule(".result   {background-color: #FFFFFF; border-color: #FFFFFF;}");
      style.addRule(".print    {background-color: #E0E0FF; border-color: #8080C0;}");
      style.addRule(".warning  {background-color: #FFFFE0; border-color: #C0C080;}");
      style.addRule(".error    {background-color: #FFE0E0; border-color: #C08080;}");
      style.addRule(".document {background-color: #E0FFFF; border-color: #80C0C0;}");

      style.addRule(".number     {color: #0000FF;}");
      style.addRule(".keyword    {color: #000080; font-weight: bold;}");
      style.addRule(".identifier {}");
      style.addRule(".quote      {color: #FF0000;}");
      style.addRule(".comment    {color: #808080; font-style: italic;}");
      style.addRule(".symbol     {font-weight: bold;}");

      mLogArea.setEditable(false);
      mLogScroll = new JScrollPane(mLogArea);
      mLogScroll.getViewport().addChangeListener(this);
      mScrollAtEnd = true;
      mOldHeight = mLogScroll.getViewport().getViewSize().height;

      mEditArea = new JTextArea("", 3, 0);
      mEditArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
      mEditScroll = new JScrollPane(mEditArea);
      mEditArea.addKeyListener(this);

      setLayout(new BorderLayout());
      add(mLogScroll, BorderLayout.CENTER);
      add(mEditScroll, BorderLayout.SOUTH);

      if(!mIsApplet)
         mFileChooser = new JFileChooser();
   }

   public void start(String host, int port, boolean useSsl)
   {
      if(!mIsApplet)
      {
         Dialogs.ConnectInfo connectInfo = Dialogs.showConnectDialog(this, host, port, useSsl);
         host = connectInfo.host;
         port = connectInfo.port;
         useSsl = connectInfo.useSsl;
      }

      mHost = host + ":" + port;

      try
      {
         Socket socket;
         if(useSsl)
         {
            socket = SSLSocketFactory.getDefault().createSocket(host, port);
            ((SSLSocket)socket).startHandshake();
         }
         else
            socket = new Socket(host, port);

         mConnection = new TransferManager(new CommandStream(socket));

         mListenThread = new Thread(this);
         mListenThread.start();
      }
      catch(SSLException e)
      {
         addText("error", "Failed to negotiate SSL session: " + e.getMessage());
      }
      catch(IOException e)
      {
         addText("error", "Failed to connect to server.");
      }
   }

   public void stop()
   {
      synchronized(mMutex)
      {
         mIsStopping = true;
      }

      if(mConnection != null)
      {
         try {mConnection.close();} catch(Exception e) {}
         //doesn't always return for some reason...
//         try {mListenThread.join();} catch(Exception e) {}
      }
   }

   // Runnable =======================================================
   public void run()
   {
      try
      {
         int currentCommand = -1;

         while(true)
         {
            final ServerCommand command = mConnection.read();

            if(command == null)
            {
               addText("error", "Server disconnected.");
               break;
            }

            if(command.type == "ACKNOWLEDGED")
            {
               currentCommand = mAcknowledgeCounter;
               String commandText;
               synchronized(mCommands) {commandText = (String)mCommands.get(currentCommand);}
               addCode("echo", commandText, currentCommand);
               ++mAcknowledgeCounter;
            }
            else if(command.type == "COMPLETE")
               currentCommand = -1;
            else if(command.type == "RESULT")
            {
               if(command.text.equals(""))
                  addLogText("result", "<p><i>(done)</i></p>", currentCommand);
               else
                  addCode("result", command.text, currentCommand);
            }
            else if(command.type == "SET")
               addCode("result", command.name + " = " + command.text, currentCommand);
            else if(command.type == "PRINT")
               addPreText("print", command.text, currentCommand);
            else if(command.type == "WARNING")
               addPreText("warning", command.text, currentCommand);
            else if(command.type == "ERROR")
               addPreText("error", command.text, currentCommand);
            else if(command.type == "DOCUMENT")
               addDocument(command.text, currentCommand);
            else if(command.type == "BROKEN")
               addPreText("error", command.text, currentCommand);
            else if(command.type == "DELAYED")
            {
               mDelays.put(new Integer(command.id), new Integer(currentCommand));
               currentCommand = -1;
            }
            else if(command.type == "CONTINUING")
            {
               Integer id = new Integer(command.id);
               Integer commandId = (Integer)mDelays.get(id);
               if(commandId == null)
                  currentCommand = -1;
               else
                  currentCommand = commandId.intValue();
               mDelays.remove(id);
            }
            else if(command.type == "UPLOAD")
            {
               if(mIsApplet)
               {
                  addText("error", "Upload denied:  Can't access filesystem from applet.", currentCommand);
                  try {mConnection.write("/data " + command.id + " 0");} catch(Exception e) {}
               }
               else
               {
                  Runnable syncUpdater = new Runnable()
                  {
                     public void run()
                     {
                        try
                        {
                           if(mFileChooser.showOpenDialog(Gui.this) == JFileChooser.APPROVE_OPTION)
                           {
                              File file = mFileChooser.getSelectedFile();
                              mConnection.upload(command.id, new FileInputStream(file));
                              return;
                           }
                        } catch(Exception e) {}

                        try {mConnection.write("/data " + command.id + " 0");} catch(Exception e) {}
                     }
                  };

                  try
                  {
                     SwingUtilities.invokeAndWait(syncUpdater);
                  } catch(Exception e) {}
               }
            }
            else if(command.type == "DOWNLOAD")
            {
               if(mIsApplet)
                  addText("error", "Download denied:  Can't access filesystem from applet.", currentCommand);
               else
               {
                  Runnable syncUpdater = new Runnable()
                  {
                     public void run()
                     {
                        try
                        {
                           if(mFileChooser.showOpenDialog(Gui.this) == JFileChooser.APPROVE_OPTION)
                           {
                              File file = mFileChooser.getSelectedFile();
                              file.delete();
                              mConnection.download(command.id, new FileOutputStream(file));
                              return;
                           }
                        } catch(Exception e) {}
                     }
                  };

                  try
                  {
                     SwingUtilities.invokeAndWait(syncUpdater);
                  } catch(Exception e) {}
               }
            }
            else if(command.type == "LOGIN")
            {
               final String passwordPrefix = command.text;

               Runnable syncUpdater = new Runnable()
               {
                  public void run()
                  {
                     try
                     {
                        Dialogs.LoginInfo loginInfo = Dialogs.showLoginDialog(Gui.this, passwordPrefix);
                        String command = "/login " + loginInfo.username;
                        synchronized(mCommands) {mCommands.add(command);}
                        try {mConnection.writeCommandWithData(command, loginInfo.passhash);}
                        catch(IOException e) {addText("error", "Lost connection to server.");}
                     } catch(Exception e) {}
                  }
               };

               try
               {
                  SwingUtilities.invokeAndWait(syncUpdater);
               } catch(Exception e) {}
            }
         }
      }
      catch(IOException e)
      {
         boolean isUnexpected = false;
         synchronized(mMutex)
         {
            isUnexpected = !mIsStopping;
         }

         if(isUnexpected)
            addText("error", "Lost connection to server.");
      }
   }

   // KeyListener ====================================================
   public void keyTyped(KeyEvent event)
   {
      if(event.getKeyChar() == '\n')
         event.consume();
      else
      {
         mInHistory = false;
         mHistoryPosition = mCommands.size();
      }
   }
   public void keyPressed(KeyEvent event)
   {
      if(event.getKeyCode() == KeyEvent.VK_UP
      && (event.getModifiers() & KeyEvent.CTRL_MASK) != 0)
      {
         event.consume();
         if(mHistoryPosition == mCommands.size() || mInHistory)
            --mHistoryPosition;
         if(mHistoryPosition < 0)
            mHistoryPosition = 0;
         else
         {
            mEditArea.setText((String)mCommands.get(mHistoryPosition));
            mInHistory = true;
         }
      }
      else if(event.getKeyCode() == KeyEvent.VK_DOWN
           && (event.getModifiers() & KeyEvent.CTRL_MASK) != 0)
      {
         event.consume();
         ++mHistoryPosition;
         if(mHistoryPosition > mCommands.size())
            mHistoryPosition = mCommands.size();
         else if(mHistoryPosition < mCommands.size())
         {
            mInHistory = true;
            mEditArea.setText((String)mCommands.get(mHistoryPosition));
         }
         else
            mEditArea.setText("");
      }
      else if(event.getKeyCode() == KeyEvent.VK_ENTER)
      {
         event.consume();
         if((event.getModifiers() & KeyEvent.SHIFT_MASK) == 0)
         {
            String originalText = mEditArea.getText();
            String text = originalText;
            if(text.length() > 0)
            {
               if(text.startsWith("/list") && text.length() > 5)
               {
                  if(text.charAt(5) == ' ')
                     text = text.substring(0, 5) + " 1" + text.substring(5);
                  else if(Character.isDigit(text.charAt(5)))
                     text = text.substring(0, 5) + " " + text.substring(5);
               }

               if(mConnection == null)
                  addText("error", "You are not connected.");
               else if(text.startsWith("/data ") || text.equals("/data"))
                  addText("error", "Manual use of /data command not allowed.");
               else if(text.startsWith("/login ") || text.equals("/login"))
                  addText("error", "Manual use of /login command not allowed.");
               else if(text.equals("/cls"))
                  mLogArea.setText("");
//               else if(text.equals("/login"))
//                  addText("error", "Need username for /login.");
//               else if(text.startsWith("/login "))
//               {
//                  byte[] passhash = Dialogs.showLoginDialog(Gui.this, mHost, text.substring(7, text.length()));
//                  synchronized(mCommands) {mCommands.add(originalText);}
//                  try {mConnection.writeCommandWithData(text, passhash);}
//                  catch(IOException e) {addText("error", "Lost connection to server.");}
//               }
//               else if(text.startsWith("/password ") || text.equals("/password")
//                    || text.startsWith("/adduser ") || text.equals("/adduser"))
//               {
//                  String username = mUsername;
//
//                  int spacePos = text.indexOf(' ');
//                  if(spacePos != -1)
//                  {
//                     int spacePos2 = text.indexOf(' ', spacePos + 1);
//                     if(spacePos2 == -1)
//                        spacePos2 = text.length();
//                     username = text.substring(spacePos+1, spacePos2);
//                  }
//
//                  byte[] passhash = Dialogs.showPasswordDialog(Gui.this, mHost, username);
//                  synchronized(mCommands) {mCommands.add(originalText);}
//                  try {mConnection.writeCommandWithData(text, passhash);}
//                  catch(IOException e) {addText("error", "Lost connection to server.");}
//               }
               else
               {
                  if(mHistoryPosition == mCommands.size())
                     ++mHistoryPosition;
                  synchronized(mCommands) {mCommands.add(originalText);}
                  try
                  {
                     mConnection.write(text);
                  }
                  catch(IOException e)
                  {
                     addText("error", "Lost connection to server.");
                  }
               }
               mEditArea.setText("");
               mInHistory = false;
            }
         }
         else
         {
            mEditArea.replaceSelection("\n");
            mHistoryPosition = mCommands.size();
            mInHistory = false;
         }
      }
   }
   public void keyReleased(KeyEvent event)
   {
      if(event.getKeyCode() == KeyEvent.VK_ENTER)
         event.consume();
   }

   // ChangeListener =================================================
   public void stateChanged(ChangeEvent e)
   {
      JViewport viewport = mLogScroll.getViewport();

      int newHeight = viewport.getViewSize().height;
      if(newHeight != mOldHeight)
      {
         mOldHeight = newHeight;
         if(mScrollAtEnd)
            viewport.setViewPosition(new Point(0, newHeight - viewport.getExtentSize().height));
      }
      else
      {
         mScrollAtEnd = viewport.getViewPosition().y ==
            (newHeight - viewport.getExtentSize().height);
      }
   }

   // internal =======================================================
   private void addLogText(String style, String text, int commandNumber)
   {
      if(commandNumber == mAcknowledgeCounter - 1)
      {
         addLogText("<div class=\"" + style + "\">" + text + "</div>");
      }
      else
      {
         String id;
         if(commandNumber == -1)
            id = "?";
         else
            id = String.valueOf(commandNumber);

         addLogText("<table width=\"100%\"><tr><td class=\"linenumber\">" + id
                  + "</td><td width=\"100%\" class=\"" + style
                  + "\">" + text + "</td></tr></table>");
      }
   }

   private void addText(String style, String text) {addText(style, text, -1);}
   private void addText(String style, String text, int commandNumber)
   {
      addLogText(style, "<p>" + fixText(text) + "</p>", commandNumber);
   }

   private void addPreText(String style, String text) {addPreText(style, text, -1);}
   private void addPreText(String style, String text, int commandNumber)
   {
      addLogText(style, "<pre>" + fixText(text) + "</pre>", commandNumber);
   }

   private void addCode(String style, String text) {addCode(style, text, -1);}
   private void addCode(String style, String text, int commandNumber)
   {
      addLogText(style, "<pre>" + highlight(text) + "</pre>", commandNumber);
   }

   private void addDocument(String text) {addDocument(text, -1);}
   private void addDocument(String text, int commandNumber)
   {
      addLogText("document", fixDocumentation(text), commandNumber);
   }

   private void addLogText(final String text)
   {
      if(SwingUtilities.isEventDispatchThread())
      {
         HTMLDocument doc = (HTMLDocument)mLogArea.getDocument();
         HTMLEditorKit kit = (HTMLEditorKit)mLogArea.getEditorKit();
         try
         {
            Element element = doc.getElement("end");
            doc.insertBeforeStart(element, text);
         } catch(Exception e) {}
      }
      else
      {
         Runnable syncUpdater = new Runnable()
            {public void run() {addLogText(text);}};

         try
         {
            SwingUtilities.invokeAndWait(syncUpdater);
         } catch(Exception e) {}
      }
   }

   private String highlight(String text)
   {
      StringBuffer result = new StringBuffer();
      int pos = 0;

      while(pos < text.length())
      {
         char c = text.charAt(pos);
         if(Character.isWhitespace(c))
         {
            int start = pos;
            do ++pos; while(pos < text.length() && Character.isWhitespace(text.charAt(pos)));
            result.append(fixText(text.substring(start, pos)));
         }
         else if(Character.isLetterOrDigit(c) || c == '_')
         {
            int start = pos;
            do ++pos;
            while(pos < text.length() && (Character.isLetterOrDigit(text.charAt(pos)) || text.charAt(pos) == '_'));
            String item = text.substring(start, pos);
            if(Character.isDigit(c))
               result.append("<span class=\"number\">");
            else if(mKeywords.contains(item))
               result.append("<span class=\"keyword\">");
            else
               result.append("<span class=\"identifier\">");
            result.append(item);
            result.append("</span>");
         }
         else if(c == '\"' || c == '\'')
         {
            int start = pos;
            do ++pos;
            while(pos < text.length() && text.charAt(pos) != '\n'
               && (text.charAt(pos) != c || text.charAt(pos-1) == '\\'));
            if(pos < text.length() && text.charAt(pos) == c)
               ++pos;
            result.append("<span class=\"quote\">");
            result.append(fixText(text.substring(start, pos)));
            result.append("</span>");
         }
         else if(c == '#')
         {
            int start = pos;
            do ++pos; while(pos < text.length() && text.charAt(pos) != '\n');
            result.append("<span class=\"comment\">");
            result.append(fixText(text.substring(start, pos)));
            result.append("</span>");
         }
         else
         {
            int start = pos;
            do
            {
               ++pos;
               if(pos == text.length())
                  break;
               c = text.charAt(pos);
            } while(!Character.isWhitespace(c)
                 && !Character.isLetterOrDigit(c)
                 && c != '_' && c != '\'' && c != '\"' && c != '#');
            result.append("<span class=\"symbol\">");
            result.append(fixText(text.substring(start, pos)));
            result.append("</span>");
         }
      }

      return result.toString();
   }

   static private String fixDocumentation(String text)
   {
      boolean inParagraph = false;
      boolean inPre = false;
      StringBuffer buffer = new StringBuffer();

      int pos = 0;
      while(pos < text.length())
      {
         if(text.charAt(pos) == ' ')
         {
            if(inParagraph)
               buffer.append("</p>");

            if(inPre)
               buffer.append("\n");
            else
               buffer.append("<pre class=\"document\">");

            inParagraph = false;
            inPre = true;
            int end = text.indexOf('\n', pos);
            if(end == -1)
               end = text.length();
            buffer.append(fixText(text.substring(pos, end)));
            pos = end + 1;
         }
         else if(text.charAt(pos) == '\n')
         {
            if(inParagraph)
               buffer.append("</p>");
            else if(inPre)
               buffer.append("\n");
            inParagraph = false;
            ++pos;
         }
         else
         {
            if(inParagraph)
               buffer.append(" ");
            else
            {
               if(inPre)
                  buffer.append("</pre>");
               buffer.append("<p class=\"document\">");
            }

            inParagraph = true;
            inPre = false;
            int end = text.indexOf('\n', pos);
            if(end == -1)
               end = text.length();
            buffer.append(fixText(text.substring(pos, end)));
            pos = end + 1;
         }
      }
      if(inParagraph)
         buffer.append("</p>");
      else if(inPre)
         buffer.append("</pre>");

      return buffer.toString();
   }

   static private String fixText(String text)
   {
      StringBuffer result = new StringBuffer();

      int pos = 0;
      for(int i = 0; i < text.length(); i++)
      {
         char c = text.charAt(i);
         switch(c)
         {
         case '<':
            result.append(text.substring(pos, i));
            result.append("&lt;");
            pos = i+1;
            break;
         case '>':
            result.append(text.substring(pos, i));
            result.append("&gt;");
            pos = i+1;
            break;
         case '&':
            result.append(text.substring(pos, i));
            result.append("&amp;");
            pos = i+1;
            break;

         default:
            break;
         }
      }

      result.append(text.substring(pos));
      return result.toString();
   }
}
