/* Evlan Client - Command shell for the Evlan programming language.
 * Copyright (C) 2004-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

public class ServerCommand
{
   public final String type;
   public final int id;
   public final String name;
   public final String text;

   public final int size;

   public ServerCommand(String commandText)
   {
      String command = getWord(commandText, 0);
      String syntax = "unknown";

      if(command.equals("ACKNOWLEDGED"))
         syntax = "name";
      else if(command.equals("COMPLETE"))
         syntax = "empty";
      else if(command.equals("DELAYED"))
         syntax = "id";
      else if(command.equals("CONTINUING"))
         syntax = "id";
      else if(command.equals("RESULT"))
         syntax = "text";
      else if(command.equals("SET"))
         syntax = "nametext";
      else if(command.equals("PRINT"))
         syntax = "text";
      else if(command.equals("WARNING"))
         syntax = "text";
      else if(command.equals("ERROR"))
         syntax = "text";
      else if(command.equals("DOCUMENT"))
         syntax = "text";
      else if(command.equals("DOWNLOAD"))
         syntax = "id";
      else if(command.equals("UPLOAD"))
         syntax = "id";
      else if(command.equals("RECEIVED"))
         syntax = "id";
      else if(command.equals("BROKEN"))
         syntax = "idtext";
      else if(command.equals("DATA"))
         syntax = "data";
      else if(command.equals("LOGIN"))
         syntax = "text";
      else
         command = "UNKNOWN";

      type = command.intern();

      if(syntax == "empty")
      {
         //nothing
         id = 0;
         name = null;
         text = null;
         size = 0;
      }
      else if(syntax == "name")
      {
         name = getWord(commandText, command.length() + 1);

         id = 0;
         text = null;
         size = 0;
      }
      else if(syntax == "id")
      {
         int _id = 0;
         try
         {
            _id = Integer.parseInt(getWord(commandText, command.length() + 1));
         } catch(Exception e) {}
         id = _id;

         name = null;
         text = null;
         size = 0;
      }
      else if(syntax == "text")
      {
         text = getRest(commandText, command.length() + 1);

         id = 0;
         name = null;
         size = 0;
      }
      else if(syntax == "nametext")
      {
         name = getWord(commandText, command.length() + 1);
         text = getRest(commandText, name.length() + command.length() + 2);

         id = 0;
         size = 0;
      }
      else if(syntax == "idtext")
      {
         String idText = getWord(commandText, command.length() + 1);
         text = getRest(commandText, idText.length() + command.length() + 2);

         int _id = 0;
         try
         {
            _id = Integer.parseInt(idText);
         } catch(Exception e) {}
         id = _id;

         name = null;
         size = 0;
      }
      else if(syntax == "data")
      {
         String idText = getWord(commandText, command.length() + 1);
         String sizeText = getWord(commandText, idText.length() + command.length() + 2);

         int _id = 0, _size = 0;
         try
         {
            _id = Integer.parseInt(idText);
         } catch(Exception e) {}
         try
         {
            _size = Integer.parseInt(sizeText);
         } catch(Exception e) {}

         id = _id;
         size = _size;

         name = null;
         text = null;
      }
      else
      {
         id = 0;
         name = null;
         text = null;
         size = 0;
      }
   }

   private String getRest(String text, int start)
   {
      if(start >= text.length())
         return "";
      return text.substring(start);
   }

   private String getWord(String text, int start)
   {
      if(start >= text.length())
         return "";

      int end = text.indexOf(' ', start);
      if(end == -1)
         return text.substring(start);
      else
         return text.substring(start, end);
   }
}
