#! /bin/bash

if [ $# -ne 1 ]; then
   cat << __EOF__
USAGE: mkmodule.sh NAME
Creates a new module in the source tree rooted at "share/evlan/modules".
Produces a .evlan file.
   NAME          The module name and path.
                 Example:
                    evlan.org/common/String
mkmodule.sh requires that the environment variable \$INITIALS be set to the
user's initials, such that the user may be credited with the ceration of the
module.
__EOF__
   exit 1
fi

if [ "$INITIALS" == "" ]; then
   echo "You have not set your initials.  Set the \$INITIALS environment variable"
   echo "to your initials.  Example:"
   echo "   export INITIALS=KV"
   exit 1
fi

CODEPATH=$1
COMPONENTS=`echo "$CODEPATH" | sed -e "s/\// /g"`

DATE=`date +%Y/%m/%d`

NAMESPACES=""
NAME=""
for DIR in $COMPONENTS; do
   NAMESPACES="$NAMESPACES $NAME"
   NAME=$DIR
done

PACKAGEDIR=share/evlan/modules
HEADER=$PACKAGEDIR/header.txt
for NAMESPACE in $NAMESPACES; do
   PACKAGEDIR=$PACKAGEDIR/$NAMESPACE
   if [ -e $PACKAGEDIR/header.txt ]; then
      HEADER=$PACKAGEDIR/header.txt
   fi
done

mkdir -p $PACKAGEDIR

if [ -e $PACKAGEDIR/$NAME.evlan ]; then
   echo "Module already exists!"
   exit 1
fi

exec 3>$PACKAGEDIR/$NAME.evlan

######################################################################

if [ -e $HEADER ]; then
   cat >&3 < $HEADER
fi

cat >&3 << __EOF__
# ====================================================================
# Revision History:
# $DATE ($INITIALS)
#  + First Version
# ====================================================================

$NAME where
   $NAME = object of
__EOF__
