#! /bin/bash

if [ $# -ne 1 ]; then
   cat << __EOF__
USAGE: mkpackage.sh NAME
Creates a new package in the source tree rooted at "src".  Produces the kake
package file.
   NAME          The fully-qualified module name (including scope).
                 Example:
                    framework::common
mkpackage.sh requires that the environment variable \$INITIALS be set to the
user's initials, such that the user may be credited with the ceration of the
package.
__EOF__
   exit 1
fi

if [ "$INITIALS" == "" ]; then
   echo "You have not set your initials.  Set the \$INITIALS environment variable"
   echo "to your initials.  Example:"
   echo "   export INITIALS=KV"
   exit 1
fi

CODEPATH=$1
NAMESPACES=`echo "$CODEPATH" | sed -e "s/::/ /g"`
UNDERSCORES=`echo "$CODEPATH" | sed -e "s/::/_/g"`
PACKAGEDIR=src/`echo "$CODEPATH" | sed -e "s/::/\//g"`

DATE=`date +%Y/%m/%d`

HEADERDIR=src
HEADER=src/header.txt
for NAMESPACE in $NAMESPACES; do
   HEADERDIR=$HEADERDIR/$NAMESPACE
   if [ -e $HEADERDIR/header.txt ]; then
      HEADER=$HEADERDIR/header.txt
   fi
done

mkdir -p $PACKAGEDIR

if [ -e $PACKAGEDIR/package.kake ]; then
   echo "Package already exists!"
   exit 1
fi

exec 3>$PACKAGEDIR/package.kake

######################################################################

if [ -e $HEADER ]; then
   cat >&3 < $HEADER
fi

cat >&3 << __EOF__

// ===================================================================

/* Revision History:
 * $DATE ($INITIALS)
 *  + First Version
 */

// ===================================================================

__EOF__
