/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2005/03/06 (KV)
 *  + First Version (copied from Win32 version)
 */

// ===================================================================

#include "error.h"
#include <evlan/vm/memory.h>
#include <evlan/vm/arrays.h>
#include <evlan/vm/native.h>
#include <evlan/vm/shortcuts.h>

#include <errno.h>
#include <stdio.h>

namespace evlan {
namespace api {
namespace posix {
using namespace evlan::vm;
using namespace evlan::vm::shortcuts;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::api::error
// TODO (KV): Review evlan::api::error

namespace error
{
   EVLAN_NATIVE_IDENTIFIER(gErrorId, error);
   EVLAN_NATIVE_IDENTIFIER(gOsErrorId, osError);

   EVLAN_NATIVE_IDENTIFIER(gAccessDeniedId, accessDenied);
   EVLAN_NATIVE_IDENTIFIER(gNotFoundId, notFound);

   void makeErrorString(ErrorCode errorCode)
   {
      arrays::makeString(strerror(errorCode));
   }

   int getErrorInfo(ErrorCode errorCode)
   {
      switch(errorCode)
      {
      case ENOENT:
         arrays::makeString($("File not found.  (Your favorite error!)"));
         return gNotFoundId;
      case EPERM:
      case EACCES:
         arrays::makeString($("Access denied.  (Ouch.)"));
         return gAccessDeniedId;
      case EINTR:
         Debug::log(Debug::ERROR) << "EINTR error encountered!";
         arrays::makeString($("POSIX is teh suck.  (EINTR)"));
         return gOsErrorId;
      default:
         makeErrorString(errorCode);
         return gOsErrorId;
      }
   }

   Value throwOsError(ErrorCode errorCode, int generalId, Value& param)
   {
      int id = getErrorInfo(errorCode);
      return simThrow(id, generalId, param);
   }

   Value throwOsError(ErrorCode errorCode, int generalId)
   {
      getErrorInfo(errorCode);
      return simThrow(generalId);
   }

   Value throwOsError(ErrorCode errorCode)
   {
      int id = getErrorInfo(errorCode);
      return simThrow(id);
   }
}

// ===================================================================

} //namespace posix
} //namespace api
} //namespace evlan
