/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/05/06 (KV)
 *  + First Version
 */

// ===================================================================

#include "error.h"
#include <evlan/vm/memory.h>
#include <evlan/vm/arrays.h>
#include <evlan/vm/native.h>
#include <evlan/vm/shortcuts.h>

#include <windows.h>

namespace evlan {
namespace api {
namespace win32 {
using namespace evlan::vm;
using namespace evlan::vm::shortcuts;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::api::error
// TODO (KV): Review evlan::api::error

namespace error
{
   EVLAN_NATIVE_IDENTIFIER(gErrorId, error);
   EVLAN_NATIVE_IDENTIFIER(gOsErrorId, osError);

   EVLAN_NATIVE_IDENTIFIER(gAccessDeniedId, accessDenied);
   EVLAN_NATIVE_IDENTIFIER(gNotFoundId, notFound);

   void makeErrorString(DWORD errorCode)
   {
      char* description;
      if(FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER |
         FORMAT_MESSAGE_FROM_SYSTEM |
         FORMAT_MESSAGE_IGNORE_INSERTS,
         NULL,
         errorCode,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR)&description,
         0,
         NULL))
      {
         int length = strlen(description);
         int pos = 0;

         //filter stupid \r\n shit
         for(int i = 0; i < length; i++)
         {
            if(description[i] == '\r'
            && description[i+1] == '\n')
               {} //ignore \r before \n
            else if(pos < i)
               description[pos++] = description[i];
            else
               ++pos;
         }

         if(description[pos - 1] == '\n')
            --pos;
         description[pos] = '\0';

         arrays::makeString(description);

         LocalFree(description);
      }
      else
         arrays::makeString($("Win32 error #") + String((uint)errorCode) + $(" occurred.  Since numerical errors are completely worthless, I asked Windows for a description of this error.  However, this request caused another error.  Wonderful."));
   }

   int getErrorInfo(DWORD errorCode)
   {
      switch(errorCode)
      {
      case ERROR_FILE_NOT_FOUND:
         arrays::makeString($("File not found.  (Your favorite error!)"));
         return gNotFoundId;
      case ERROR_PATH_NOT_FOUND:
         arrays::makeString($("Path not found.  (This is subtly different from \"File not found.\".)"));
         return gNotFoundId;
      case ERROR_ACCESS_DENIED:
         arrays::makeString($("Access denied.  (Ouch.)"));
         return gAccessDeniedId;
      default:
         makeErrorString(errorCode);
         return gOsErrorId;
      }
   }

   Value throwOsError(unsigned long errorCode, int generalId, Value& param)
   {
      int id = getErrorInfo(errorCode);
      return simThrow(id, generalId, param);
   }

   Value throwOsError(unsigned long errorCode, int generalId)
   {
      getErrorInfo(errorCode);
      return simThrow(generalId);
   }

   Value throwOsError(unsigned long errorCode)
   {
      int id = getErrorInfo(errorCode);
      return simThrow(id);
   }
}

// ===================================================================

} //namespace win32
} //namespace api
} //namespace evlan
