/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/01/22 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef evlan_compiler_Compiler_h__INCLUDED__
#define evlan_compiler_Compiler_h__INCLUDED__

#include "ErrorCollector.h"
#include "Token.h"
#include "Expression.h"
#include "Bytecode.h"

namespace evlan {
namespace compiler {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document evlan::compiler::Compiler
// TODO (KV): Test evlan::compiler::Compiler

interface Compiler;
typedef SmartPointer<Compiler> CompilerPtr;

interface Compiler: implements Object
{
   virtual ~Compiler();

   enum Dialect
   {
      BASIC     //use this
      //there used to be others, and could be in the future, maybe
   };

   enum ProcessingLevel
   {
      NONE,     /* this will produce incorrect code, but is useful for
                 * debugging the compiler */
      CLEANUP,  //this will produce correct code
      OPTIMIZE  //this will produce very nice correct code
   };

   virtual Array<const Token> tokenize(String source) = 0;

   virtual Expression parse(String source) = 0;
   virtual Expression parse(Array<const Token> tokens) = 0;

   virtual BytecodePtr compile(String source) = 0;
   virtual BytecodePtr compile(Array<const Token> tokens) = 0;
   virtual BytecodePtr compile(Expression parseTree) = 0;

   FW_DECLARE_EXCEPTION(HadErrors, Exception);
};

// ===================================================================

} //namespace compiler
} //namespace evlan

#endif //included
