/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef evlan_compiler_Expression_h__INCLUDED
#define evlan_compiler_Expression_h__INCLUDED

// includes ==========================================================
#include <framework/util/Pattern.h>
#ifdef INCLUDE_WRITER_CODE
#include <framework/TextWriter.h>
#endif

// begin namespace ===================================================
namespace evlan {
namespace compiler {
using namespace ::framework::common;

// class definition ==================================================
typedef String Identifier;

struct Expression
{
   struct MemberDefinition;

   // literals =======================================================
   FW_PATTERN_START(Literal)
      FW_PATTERN_VALUE_1(boolean, bool)

      FW_PATTERN_VALUE_1(integer, int32)
      FW_PATTERN_VALUE_1(integer64, int64)
      FW_PATTERN_VALUE_1(real, double)

      FW_PATTERN_VALUE_1(data8,  uint8)
      FW_PATTERN_VALUE_1(data16, uint16)
      FW_PATTERN_VALUE_1(data32, uint32)
      FW_PATTERN_VALUE_1(data64, uint64)

      FW_PATTERN_VALUE_1(character, uint32)
      FW_PATTERN_VALUE_1(atom, String)
      FW_PATTERN_VALUE_1(string8 , Array<const char8 >)
      FW_PATTERN_VALUE_1(string16, Array<const char16>)
      FW_PATTERN_VALUE_1(string32, Array<const char32>)
   FW_PATTERN_END

   // expresions =====================================================
   FW_PATTERN_START(Value)
      FW_PATTERN_VALUE_1(import, String)

      FW_PATTERN_VALUE_1(literal, Literal)
      FW_PATTERN_VALUE_2(variable, int, int)  //index to environment stack

      FW_PATTERN_VALUE_2(memberAccess, Expression, Identifier)
      FW_PATTERN_VALUE_1(arrayConstructor, Array<const Expression>)
      FW_PATTERN_VALUE_3(conditional, Expression, Expression, Expression)
      FW_PATTERN_VALUE_2(whereBlock, Array<const Expression>, Expression)

      FW_PATTERN_VALUE_3(function, Array<const String>, Expression, String)
      FW_PATTERN_VALUE_2(application, Expression, Array<Expression>)

      FW_PATTERN_VALUE_2(objectDefinition, Array<const MemberDefinition>, String)
   FW_PATTERN_END

   // actual data ====================================================
   int line;
   int column;
   Value value;

   inline Expression(int l, int c, Value v)
      : line(l), column(c), value(v) {}
   inline Expression() {}
};

struct Expression::MemberDefinition
{
   Identifier name;
   Expression value;
};

#ifdef INCLUDE_WRITER_CODE
void writeExpression(framework::TextWriterPtr out, Expression expression);
#endif

/* A template for you!
   FW_MATCH_START(expression.value)
   FW_MATCH_CASE_1(Expression::Value::import, name)
   FW_MATCH_CASE_1(Expression::Value::literal, literal)
      FW_MATCH_START(literal)
      FW_MATCH_CASE_1(Expression::Literal::boolean, flag)
      FW_MATCH_CASE_1(Expression::Literal::integer, i)
      FW_MATCH_CASE_1(Expression::Literal::integer64, i)
      FW_MATCH_CASE_1(Expression::Literal::real, x)
      FW_MATCH_CASE_1(Expression::Literal::data8,  d)
      FW_MATCH_CASE_1(Expression::Literal::data16, d)
      FW_MATCH_CASE_1(Expression::Literal::data32, d)
      FW_MATCH_CASE_1(Expression::Literal::data64, d)
      FW_MATCH_CASE_1(Expression::Literal::character, c)
      FW_MATCH_CASE_1(Expression::Literal::string, chars)
      FW_MATCH_END
   FW_MATCH_CASE_2(Expression::Value::variable, level, item)
   FW_MATCH_CASE_2(Expression::Value::memberAccess, item, member)
   FW_MATCH_CASE_1(Expression::Value::arrayConstructor, elements)
   FW_MATCH_CASE_3(Expression::Value::conditional, input, thenClause, elseClause)
   FW_MATCH_CASE_2(Expression::Value::whereBlock, members, result)
   FW_MATCH_CASE_2(Expression::Value::function, parameters, definition)
   FW_MATCH_CASE_2(Expression::Value::application, function, parameters)
   FW_MATCH_CASE_1(Expression::Value::objectDefinition, members)
   FW_MATCH_END
*/

} //namespace compiler
} //namespace evlan

#endif //included
