/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef evlan_compiler_Token_h__INCLUDED
#define evlan_compiler_Token_h__INCLUDED

// includes ==========================================================
#include <framework/util/Pattern.h>
#include <framework/util/Atom.h>
#ifdef INCLUDE_WRITER_CODE
#include <framework/TextWriter.h>
#endif

// begin namespace ===================================================
namespace evlan {
namespace compiler {
using namespace ::framework::common;

// class definition ==================================================
struct Token
{
   FW_PATTERN_START(Value)
      FW_PATTERN_VALUE_1(whitespace, int)

      FW_PATTERN_VALUE_0(endOfLine)
      FW_PATTERN_VALUE_0(endOfStatement)
      FW_PATTERN_VALUE_1(startOfBlock, int)
      FW_PATTERN_VALUE_0(endOfBlock)
      FW_PATTERN_VALUE_0(endOfSource)

      FW_PATTERN_VALUE_1(identifier, String)
      FW_PATTERN_VALUE_1(keyword, ::framework::util::Atom)

      FW_PATTERN_VALUE_1(integer, int32)
      FW_PATTERN_VALUE_1(integer64, int64)

      FW_PATTERN_VALUE_1(real, double)

      FW_PATTERN_VALUE_1(data8, uint8)
      FW_PATTERN_VALUE_1(data16, uint16)
      FW_PATTERN_VALUE_1(data32, uint32)
      FW_PATTERN_VALUE_1(data64, uint64)

      FW_PATTERN_VALUE_1(character, char32)
      FW_PATTERN_VALUE_1(atom, String)
      FW_PATTERN_VALUE_1(string8 , Array<const char8 >)
      FW_PATTERN_VALUE_1(string16, Array<const char16>)
      FW_PATTERN_VALUE_1(string32, Array<const char32>)

      FW_PATTERN_VALUE_1(comment, String)
   FW_PATTERN_END

   int line;
   int column;
   Value value;

   inline Token(int line_, int column_, Value value_)
      : line(line_), column(column_), value(value_) {}
   inline Token() {}
};

#ifdef INCLUDE_WRITER_CODE
void writeToken(framework::TextWriterPtr out, Token token);
void writeToken(framework::TextWriterPtr out, Token::Value token);
#endif

} //namespace compiler
} //namespace evlan

#endif //included
