/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/01/27 (KV)
 *  + First Version
 */

// ===================================================================

#include "cleanupMemberAccess.h"
#include <stack>

namespace evlan {
namespace compiler {
namespace backend {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::compiler::backend::cleanupMemberAccess
// TODO (KV): Review evlan::compiler::backend::cleanupMemberAccess

void cleanupMemberAccess(MutableBytecodePtr bytecode,
                         ErrorCollectorPtr errorCollector)
{
   Array<int> newMap(bytecode->instructions.getSize());
   std::stack<int> todo;

   for(int i = bytecode->instructions.getSize() - 1; i >= 0; i--)
   {
      newMap[i] = -1;
      todo.push(i);
   }

   while(!todo.empty())
   {
      if(newMap[todo.top()] >= 0)
         todo.pop();
      else
      {
         newMap[todo.top()] = -2;

         FW_MATCH_START(bytecode->instructions[todo.top()].type)
         FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadMember, memberName)
            int newInput = newMap[bytecode->instructions[todo.top()].inputs[0]];
            if(newInput == -1)
               todo.push(bytecode->instructions[todo.top()].inputs[0]);
            else if(newInput == -2)
            {
               errorCollector->addError(
                  bytecode->instructions[todo.top()].line,
                  bytecode->instructions[todo.top()].column,
                  $("Cyclic member access.  Dork."));
               newMap[todo.top()] = todo.top();
               todo.pop();
            }
            else
            {
               FW_MATCH_START(bytecode->instructions[newInput].type)
               FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::makeObject, memberTable)
                  bool foundMember = false;
                  for(int i = 0; i < bytecode->classes[memberTable].memberTable.getSize(); i++)
                  {
                     if(bytecode->classes[memberTable].memberTable[i] == memberName)
                     {
                        foundMember = true;
                        int target = bytecode->instructions[newInput].inputs[i];
                        if(newMap[target] == -1)
                           todo.push(target);
                        else if(newMap[target] == -2)
                        {
                           errorCollector->addError(
                              bytecode->instructions[todo.top()].line,
                              bytecode->instructions[todo.top()].column,
                              $("Cyclic member access.  Dork."));
                           newMap[todo.top()] = target;
                           todo.pop();
                        }
                        else
                        {
                           newMap[todo.top()] = newMap[target];
                           todo.pop();
                        }
                        break;
                     }
                  }
                  if(!foundMember)
                  {
                     errorCollector->addError(
                        bytecode->instructions[todo.top()].line,
                        bytecode->instructions[todo.top()].column,
                        $("Object does not contain a member called \"") +
                        bytecode->memberNameTable[memberName] + $("\"."));
                     newMap[todo.top()] = todo.top();
                     todo.pop();
                  }

               FW_MATCH_DEFAULT
                  newMap[todo.top()] = todo.top();
                  todo.pop();
                  //nothing
               FW_MATCH_END
            }
         FW_MATCH_DEFAULT
            newMap[todo.top()] = todo.top();
            todo.pop();
         FW_MATCH_END
      }
   }

   bytecode->rootInstruction = newMap[bytecode->rootInstruction];

   for(int i = 0; i < bytecode->instructions.getSize(); i++)
   {
      for(int j = 0; j < bytecode->instructions[i].inputs.getSize(); j++)
         bytecode->instructions[i].inputs[j] = newMap[bytecode->instructions[i].inputs[j]];
   }
}

// ===================================================================

} //namespace backend
} //namespace compiler
} //namespace evlan
