/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/01/27 (KV)
 *  + First Version
 */

// ===================================================================

#include "cleanupUnusedInstructions.h"
#include <queue>

namespace evlan {
namespace compiler {
namespace backend {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::compiler::backend::cleanupUnusedInstructions
// TODO (KV): Review evlan::compiler::backend::cleanupUnusedInstructions

void cleanupUnusedInstructions(MutableBytecodePtr bytecode)
{
   int count = bytecode->instructions.getSize();
   Array<bool> marked(count);

   for(int i = 0; i < count; i++)
      marked[i] = false;

   std::queue<int> todo;

   todo.push(bytecode->rootInstruction);

   while(!todo.empty())
   {
      if(!marked[todo.front()])
      {
         for(int i = 0; i < bytecode->instructions[todo.front()].inputs.getSize(); i++)
            todo.push(bytecode->instructions[todo.front()].inputs[i]);
         if(bytecode->instructions[todo.front()].function != -1)
            todo.push(bytecode->instructions[todo.front()].function);
         if(bytecode->instructions[todo.front()].branch != -1)
            todo.push(bytecode->instructions[todo.front()].branch / 2);
         marked[todo.front()] = true;
      }
      todo.pop();
   }

   int newCount = 0;
   Array<int> newMap(count);

   for(int i = 0; i < count; i++)
   {
      if(marked[i])
         newMap[i] = newCount++;
      else
         newMap[i] = -1;
   }

   Array<MutableBytecode::Instruction> newInstructions(newCount);

   bytecode->rootInstruction = newMap[bytecode->rootInstruction];

   for(int i = 0; i < count; i++)
   {
      if(marked[i])
      {
         newInstructions[newMap[i]] = bytecode->instructions[i];
         Array<int> newInputs(bytecode->instructions[i].inputs.getSize());
         for(int j = 0; j < newInputs.getSize(); j++)
            newInputs[j] = newMap[bytecode->instructions[i].inputs[j]];
         newInstructions[newMap[i]].inputs = newInputs;

         if(bytecode->instructions[i].function != -1)
         {
            newInstructions[newMap[i]].function =
               newMap[bytecode->instructions[i].function];
         }
         if(bytecode->instructions[i].branch != -1)
         {
            newInstructions[newMap[i]].branch =
               newMap[bytecode->instructions[i].branch/2] +
                      bytecode->instructions[i].branch%2;
         }
      }
   }

   bytecode->instructions = newInstructions;
}

// ===================================================================

} //namespace backend
} //namespace compiler
} //namespace evlan
