/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/01/27 (KV)
 *  + First Version
 */

// ===================================================================

#include "optimizeArrays.h"
#include <framework/util/Vector.h>
#include <framework/util/TableBuilder.h>

namespace evlan {
namespace compiler {
namespace backend {

//Fuck the C++ standard for disallowing this, and shame on GCC for
//allowing it in previous versions then spontaneously breaking it.
//using ::framework::util;
namespace util {using namespace ::framework::util;}

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::compiler::backend::optimizeArrays
// TODO (KV): Review evlan::compiler::backend::optimizeArrays

void optimizeArrays(MutableBytecodePtr bytecode)
{
   util::TableBuilder<Array<const uint32>, Array<const uint32>::SequenceOrdering> newDataTable;

   //assume that there are no repeats in already existing arrays...
   for(int i = 0; i < bytecode->dataTable.getSize(); i++)
   {
      if(newDataTable.lookup(bytecode->dataTable[i]) != i)
         FW_THROW(spnew FatalError($("Assertion failed:  There should be no repeats in the bytecode's binary data table.")));
   }

   for(int i = 0; i < bytecode->instructions.getSize(); i++)
   {
      FW_MATCH_START(bytecode->instructions[i].type)
      FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::makeArray, size)
         int type = -1;
         util::Vector<uint32> data;
         for(int j = 0; j < size; j++)
         {
            int element = bytecode->instructions[i].inputs[j];
#define CHECKTYPE(TYPE)             \
   if(type == -1)                   \
      type = (TYPE);                \
   else if(type != (TYPE))          \
      type = -2;                    \
   if(type >= 0)

            FW_MATCH_START(bytecode->instructions[element].type)
            FW_MATCH_CASE_0(MutableBytecode::Instruction::Type::loadTrue)
               CHECKTYPE(Bytecode::BOOLEAN)
               {
                  if(j % 32 == 0)
                     data.pushBack(0x00000000);
                  data[j / 32] |= (uint32)1 << (31 - j % 32);
               }
            FW_MATCH_CASE_0(MutableBytecode::Instruction::Type::loadFalse)
               CHECKTYPE(Bytecode::BOOLEAN)
               {
                  if(j % 32 == 0)
                     data.pushBack(0x00000000);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadInteger, k)
               CHECKTYPE(Bytecode::INTEGER32)
                  data.pushBack(k);
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadInteger64, k)
               CHECKTYPE(Bytecode::INTEGER64)
               {
                  data.pushBack(((uint64)k & 0xFFFFFFFF00000000ll) >> 32);
                  data.pushBack((uint64)k & 0x00000000FFFFFFFFll);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadReal, x)
               CHECKTYPE(Bytecode::REAL)
               {
                  union {uint64 k; double y;} u;
                  u.y = x;
                  data.pushBack((u.k & 0xFFFFFFFF00000000ll) >> 32);
                  data.pushBack(u.k & 0x00000000FFFFFFFFll);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadData1, d)
               CHECKTYPE(Bytecode::DATA + 0)
               {
                  if(j % 32 == 0)
                     data.pushBack(0x00000000);
                  data[j / 32] |= (uint32)(d & 0x01) << (31 - j % 32);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadData2, d)
               CHECKTYPE(Bytecode::DATA + 1)
               {
                  if(j % 16 == 0)
                     data.pushBack(0x00000000);
                  data[j / 16] |= (uint32)(d & 0x03) << ((15 - j % 16) * 2);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadData4, d)
               CHECKTYPE(Bytecode::DATA + 2)
               {
                  if(j % 8 == 0)
                     data.pushBack(0x00000000);
                  data[j / 8] |= (uint32)(d & 0x0F) << ((7 - j % 8) * 4);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadData8, d)
               CHECKTYPE(Bytecode::DATA + 3)
               {
                  if(j % 4 == 0)
                     data.pushBack(0x00000000);
                  data[j / 4] |= (uint32)d << ((3 - j % 4) * 8);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadData16, d)
               CHECKTYPE(Bytecode::DATA + 4)
               {
                  if(j % 2 == 0)
                     data.pushBack(0x00000000);
                  data[j / 2] |= (uint32)d << ((1 - j % 2) * 16);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadData32, d)
               CHECKTYPE(Bytecode::DATA + 5)
                  data.pushBack(d);
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadData64, d)
               CHECKTYPE(Bytecode::DATA + 6)
               {
                  data.pushBack((d & 0xFFFFFFFF00000000ll) >> 32);
                  data.pushBack(d & 0x00000000FFFFFFFFll);
               }
            FW_MATCH_CASE_2(MutableBytecode::Instruction::Type::loadData, size, index)
               CHECKTYPE(Bytecode::DATA + size)
               {
                  int wordSize = 1 << (size - 5);
                  for(int k = 0; k < wordSize; k++)
                     data.pushBack(bytecode->dataTable[index][k]);
               }
            FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::loadCharacter, c)
               CHECKTYPE(Bytecode::CHARACTER32)
                  data.pushBack(c);
            FW_MATCH_DEFAULT
               type = -2;
            FW_MATCH_END
#undef CHECKTYPE
            if(type == -2)
               break;
         }

         if(type >= 0)
         {
            Array<const uint32> dataArray = data.getArray().toConst();
            if(type == Bytecode::CHARACTER32
            || type == Bytecode::INTEGER32)
            {
               //see if we can make this character8 or character16 instead
               int maxBits = 8;
               if(type == Bytecode::CHARACTER32)
               {
                  for(int i = 0; i < dataArray.getSize(); i++)
                  {
                     if(dataArray[i] >= 0x00010000)
                     {
                        maxBits = 32;
                        break;
                     }
                     else if(dataArray[i] >= 0x00000100)
                        maxBits = 16;
                  }
               }
               else
               {
                  for(int i = 0; i < dataArray.getSize(); i++)
                  {
                     if((int32)dataArray[i] >= 32768
                     || (int32)dataArray[i] < -32768)
                     {
                        maxBits = 32;
                        break;
                     }
                     else if((int32)dataArray[i] >= 128
                          || (int32)dataArray[i] < -128)
                        maxBits = 16;
                  }
               }

               if(maxBits == 8)
               {
                  type -= 2;
#ifdef PLATFORM_BIG_ENDIAN
                  for(int i = 0; i < dataArray.getSize(); i++)
                     ((uint8*)&dataArray[0])[i] = dataArray[i];
#else
                  for(int i = 0; i < dataArray.getSize(); i++)
                     ((uint8*)&dataArray[0])[i^3] = dataArray[i];
#endif
               }
               else if(maxBits == 16)
               {
                  type -= 1;
#ifdef PLATFORM_BIG_ENDIAN
                  for(int i = 0; i < dataArray.getSize(); i++)
                     ((uint16*)&dataArray[0])[i] = dataArray[i];
#else
                  for(int i = 0; i < dataArray.getSize(); i++)
                     ((uint16*)&dataArray[0])[i^1] = dataArray[i];
#endif
               }
            }

            bytecode->instructions[i].type =
               MutableBytecode::Instruction::Type::loadArray(
                  type, bytecode->instructions[i].inputs.getSize(),
                  newDataTable.lookup(dataArray));
            bytecode->instructions[i].inputs = null;
         }
      FW_MATCH_DEFAULT
         //nothing
      FW_MATCH_END
   }

   bytecode->dataTable = newDataTable.makeTable();
}

// ===================================================================

} //namespace backend
} //namespace compiler
} //namespace evlan
