/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/01/27 (KV)
 *  + First Version
 */

// ===================================================================

#include "optimizeMemberTables.h"
#include <algorithm>
#include <framework/util/TableBuilder.h>

namespace evlan {
namespace compiler {
namespace backend {

//Fuck the C++ standard for disallowing this, and shame on GCC for
//allowing it in previous versions then spontaneously breaking it.
//using ::framework::util;
namespace util {using namespace ::framework::util;}

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::compiler::backend::optimizeMemberTables
// TODO (KV): Review evlan::compiler::backend::optimizeMemberTables

namespace
{
   template <typename Type>
   struct MappedOrdering
   {
      Type* input;

      inline bool operator()(int a, int b)
      {
         return input[a] < input[b];
      }
   };

   /* returns an array mapping the unsorted locations to their new,
    * sorted locations */
   template <typename Type>
   Array<int> mappedSort(Array<Type> input)
   {
      if(input.getSize() == 0)
         return null;

      Array<int> newToOldMap(input.getSize());
      for(int i = 0; i < newToOldMap.getSize(); i++)
         newToOldMap[i] = i;
      MappedOrdering<Type> ordering;
      ordering.input = &input[0];
      std::sort(&newToOldMap[0], &newToOldMap[0] + newToOldMap.getSize(), ordering);
      Array<int> oldToNewMap(newToOldMap.getSize());
      for(int i = 0; i < newToOldMap.getSize(); i++)
         oldToNewMap[newToOldMap[i]] = i;
      return oldToNewMap;
   }
}

void optimizeMemberTables(MutableBytecodePtr bytecode)
{
   Array<Array<int> > memberTableMaps(bytecode->classes.getSize());
   Array<int> newClassIndices(bytecode->classes.getSize());
   util::TableBuilder<Bytecode::ObjectClass> newClasses;

   for(int i = 0; i < bytecode->classes.getSize(); i++)
   {
      memberTableMaps[i] = mappedSort(bytecode->classes[i].memberTable);
      Array<int> newTable(bytecode->classes[i].memberTable.getSize());
      for(int j = 0; j < bytecode->classes[i].memberTable.getSize(); j++)
         newTable[memberTableMaps[i][j]] = bytecode->classes[i].memberTable[j];
      newClassIndices[i] = newClasses.lookup(Bytecode::ObjectClass(
         newTable.toConst(), bytecode->classes[i].documentation));
   }

   bytecode->classes = newClasses.makeTable();

   for(int i = 0; i < bytecode->instructions.getSize(); i++)
   {
      FW_MATCH_START(bytecode->instructions[i].type)
      FW_MATCH_CASE_1(MutableBytecode::Instruction::Type::makeObject, index)
         bytecode->instructions[i].type =
            MutableBytecode::Instruction::Type::makeObject(newClassIndices[index]);
         Array<int> newInputs(bytecode->instructions[i].inputs.getSize());
         for(int j = 0; j < bytecode->instructions[i].inputs.getSize(); j++)
            newInputs[memberTableMaps[index][j]] = bytecode->instructions[i].inputs[j];
         bytecode->instructions[i].inputs = newInputs;
      FW_MATCH_DEFAULT
         //nothing
      FW_MATCH_END
   }
}

// ===================================================================

} //namespace backend
} //namespace compiler
} //namespace evlan
