/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
#include "BasicDialect.h"
#include "BasicKeywordSet.h"
#include "BasicParser.h"

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace dialects {
namespace basic {
using namespace syntax;
using namespace grammar;

// implementation definition =================================================
implementation BasicDialect: implements Dialect
{
   BasicDialect();
   ~BasicDialect();

   // Dialect --------------------------------------------------------
   KeywordSetPtr getKeywordSet();
   ParserPtr makeParser();

   // internal -------------------------------------------------------
   BasicKeywordSetPtr mKeywordSet;
};

BasicDialect::BasicDialect()
   : mKeywordSet(spnew BasicKeywordSet)
{
}

BasicDialect::~BasicDialect()
{
}

// Dialect -----------------------------------------------------------
KeywordSetPtr BasicDialect::getKeywordSet()
{
   return mKeywordSet;
}

ParserPtr BasicDialect::makeParser()
{
   return makeBasicParser(mKeywordSet);
}

// factory function ==================================================
DialectPtr makeBasicDialect()
{
   return spnew BasicDialect;
}

} //namespace basic
} //namespace dialects
} //namespace compiler
} //namespace evlan
