/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
#include "BasicKeywordSet.h"
#include <map>

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace dialects {
namespace basic {

struct BasicKeywordSet::Internal
{
   std::map<String, util::Atom> mNameToIdMap;
   std::map<util::Atom, String> mIdToNameMap;
};

// class members =====================================================
BasicKeywordSet::BasicKeywordSet()
   : mTrueName   ($("true")),
     mFalseName  ($("false")),
     mIfName     ($("if")),
     mThenName   ($("then")),
     mElseName   ($("else")),
     mWhereName  ($("where")),
     mOfName     ($("of")),
     mObjectName ($("object")),
     mArrayName  ($("array")),
     mImportName ($("import")),
     mDoName     ($("do")),
     mReturnName ($("return")),
     mThrowName  ($("throw")),
     mTryName    ($("try")),
     mCatchName  ($("catch")),
     mFinallyName($("finally")),

     mOpDotName          ($(".")),
     mOpCommaName        ($(",")),
     mOpAbstractName     ($("=>")),
     mOpOpenParenName    ($("(")),
     mOpCloseParenName   ($(")")),
     mOpOpenBraceName    ($("{")),
     mOpCloseBraceName   ($("}")),
     mOpOpenBracketName  ($("[")),
     mOpCloseBracketName ($("]")),
     mOpAssignName       ($("=")),
     mOpStoreName        ($(":=")),
     mOpScopeName        ($("\\")),

     mOpConsName         ($("::")),
     mOpPlusName         ($("+")),
     mOpMinusName        ($("-")),
     mOpTimesName        ($("*")),
     mOpDivideName       ($("/")),
     mOpModulusName      ($("%")),
     mOpPowerName        ($("^")),
     mOpEqualName        ($("==")),
     mOpLessName         ($("<")),
     mOpGreaterName      ($(">")),
     mOpLessEqualName    ($("<=")),
     mOpGreaterEqualName ($(">=")),
     mOpNotEqualName     ($("!=")),
     mOpAndName          ($("and")),
     mOpOrName           ($("or")),
     mOpNotName          ($("not")),

     mTrueId   (util::Atom::newAtom(mTrueName)),
     mFalseId  (util::Atom::newAtom(mFalseName)),
     mIfId     (util::Atom::newAtom(mIfName)),
     mThenId   (util::Atom::newAtom(mThenName)),
     mElseId   (util::Atom::newAtom(mElseName)),
     mWhereId  (util::Atom::newAtom(mWhereName)),
     mOfId     (util::Atom::newAtom(mOfName)),
     mObjectId (util::Atom::newAtom(mObjectName)),
     mArrayId  (util::Atom::newAtom(mArrayName)),
     mImportId (util::Atom::newAtom(mImportName)),
     mDoId     (util::Atom::newAtom(mDoName)),
     mReturnId (util::Atom::newAtom(mReturnName)),
     mThrowId  (util::Atom::newAtom(mThrowName)),
     mTryId    (util::Atom::newAtom(mTryName)),
     mCatchId  (util::Atom::newAtom(mCatchName)),
     mFinallyId(util::Atom::newAtom(mFinallyName)),

     mOpDotId          (util::Atom::newAtom(mOpDotName)),
     mOpCommaId        (util::Atom::newAtom(mOpCommaName)),
     mOpAbstractId     (util::Atom::newAtom(mOpAbstractName)),
     mOpOpenParenId    (util::Atom::newAtom(mOpOpenParenName)),
     mOpCloseParenId   (util::Atom::newAtom(mOpCloseParenName)),
     mOpOpenBraceId    (util::Atom::newAtom(mOpOpenBraceName)),
     mOpCloseBraceId   (util::Atom::newAtom(mOpCloseBraceName)),
     mOpOpenBracketId  (util::Atom::newAtom(mOpOpenBracketName)),
     mOpCloseBracketId (util::Atom::newAtom(mOpCloseBracketName)),
     mOpAssignId       (util::Atom::newAtom(mOpAssignName)),
     mOpStoreId        (util::Atom::newAtom(mOpStoreName)),
     mOpScopeId        (util::Atom::newAtom(mOpScopeName)),

     mOpConsId         (util::Atom::newAtom(mOpConsName)),
     mOpPlusId         (util::Atom::newAtom(mOpPlusName)),
     mOpMinusId        (util::Atom::newAtom(mOpMinusName)),
     mOpTimesId        (util::Atom::newAtom(mOpTimesName)),
     mOpDivideId       (util::Atom::newAtom(mOpDivideName)),
     mOpModulusId     (util::Atom::newAtom(mOpModulusName)),
     mOpPowerId        (util::Atom::newAtom(mOpPowerName)),
     mOpEqualId        (util::Atom::newAtom(mOpEqualName)),
     mOpLessId         (util::Atom::newAtom(mOpLessName)),
     mOpGreaterId      (util::Atom::newAtom(mOpGreaterName)),
     mOpLessEqualId    (util::Atom::newAtom(mOpLessEqualName)),
     mOpGreaterEqualId (util::Atom::newAtom(mOpGreaterEqualName)),
     mOpNotEqualId     (util::Atom::newAtom(mOpNotEqualName)),
     mOpAndId          (util::Atom::newAtom(mOpAndName)),
     mOpOrId           (util::Atom::newAtom(mOpOrName)),
     mOpNotId          (util::Atom::newAtom(mOpNotName)),

     mInternal(new Internal)
{
   mInternal->mNameToIdMap[mTrueName   ] = mTrueId;
   mInternal->mNameToIdMap[mFalseName  ] = mFalseId;
   mInternal->mNameToIdMap[mIfName     ] = mIfId;
   mInternal->mNameToIdMap[mThenName   ] = mThenId;
   mInternal->mNameToIdMap[mElseName   ] = mElseId;
   mInternal->mNameToIdMap[mWhereName  ] = mWhereId;
   mInternal->mNameToIdMap[mOfName     ] = mOfId;
   mInternal->mNameToIdMap[mObjectName ] = mObjectId;
   mInternal->mNameToIdMap[mArrayName  ] = mArrayId;
   mInternal->mNameToIdMap[mImportName ] = mImportId;
   mInternal->mNameToIdMap[mDoName     ] = mDoId;
   mInternal->mNameToIdMap[mReturnName ] = mReturnId;
   mInternal->mNameToIdMap[mThrowName  ] = mThrowId;
   mInternal->mNameToIdMap[mTryName    ] = mTryId;
   mInternal->mNameToIdMap[mCatchName  ] = mCatchId;
   mInternal->mNameToIdMap[mFinallyName] = mFinallyId;

   mInternal->mNameToIdMap[mOpDotName          ] = mOpDotId;
   mInternal->mNameToIdMap[mOpCommaName        ] = mOpCommaId;
   mInternal->mNameToIdMap[mOpAbstractName     ] = mOpAbstractId;
   mInternal->mNameToIdMap[mOpOpenParenName    ] = mOpOpenParenId;
   mInternal->mNameToIdMap[mOpCloseParenName   ] = mOpCloseParenId;
   mInternal->mNameToIdMap[mOpOpenBraceName    ] = mOpOpenBraceId;
   mInternal->mNameToIdMap[mOpCloseBraceName   ] = mOpCloseBraceId;
   mInternal->mNameToIdMap[mOpOpenBracketName  ] = mOpOpenBracketId;
   mInternal->mNameToIdMap[mOpCloseBracketName ] = mOpCloseBracketId;
   mInternal->mNameToIdMap[mOpAssignName       ] = mOpAssignId;
   mInternal->mNameToIdMap[mOpStoreName        ] = mOpStoreId;
   mInternal->mNameToIdMap[mOpScopeName        ] = mOpScopeId;

   mInternal->mNameToIdMap[mOpConsName         ] = mOpConsId;
   mInternal->mNameToIdMap[mOpPlusName         ] = mOpPlusId;
   mInternal->mNameToIdMap[mOpMinusName        ] = mOpMinusId;
   mInternal->mNameToIdMap[mOpTimesName        ] = mOpTimesId;
   mInternal->mNameToIdMap[mOpDivideName       ] = mOpDivideId;
   mInternal->mNameToIdMap[mOpModulusName      ] = mOpModulusId;
   mInternal->mNameToIdMap[mOpPowerName        ] = mOpPowerId;
   mInternal->mNameToIdMap[mOpEqualName        ] = mOpEqualId;
   mInternal->mNameToIdMap[mOpLessName         ] = mOpLessId;
   mInternal->mNameToIdMap[mOpGreaterName      ] = mOpGreaterId;
   mInternal->mNameToIdMap[mOpLessEqualName    ] = mOpLessEqualId;
   mInternal->mNameToIdMap[mOpGreaterEqualName ] = mOpGreaterEqualId;
   mInternal->mNameToIdMap[mOpNotEqualName     ] = mOpNotEqualId;
   mInternal->mNameToIdMap[mOpAndName          ] = mOpAndId;
   mInternal->mNameToIdMap[mOpOrName           ] = mOpOrId;
   mInternal->mNameToIdMap[mOpNotName          ] = mOpNotId;

   mInternal->mIdToNameMap[mTrueId   ] = mTrueName;
   mInternal->mIdToNameMap[mFalseId  ] = mFalseName;
   mInternal->mIdToNameMap[mIfId     ] = mIfName;
   mInternal->mIdToNameMap[mThenId   ] = mThenName;
   mInternal->mIdToNameMap[mElseId   ] = mElseName;
   mInternal->mIdToNameMap[mWhereId  ] = mWhereName;
   mInternal->mIdToNameMap[mOfId     ] = mOfName;
   mInternal->mIdToNameMap[mObjectId ] = mObjectName;
   mInternal->mIdToNameMap[mArrayId  ] = mArrayName;
   mInternal->mIdToNameMap[mImportId ] = mImportName;
   mInternal->mIdToNameMap[mDoId     ] = mDoName;
   mInternal->mIdToNameMap[mReturnId ] = mReturnName;
   mInternal->mIdToNameMap[mThrowId  ] = mThrowName;
   mInternal->mIdToNameMap[mTryId    ] = mTryName;
   mInternal->mIdToNameMap[mCatchId  ] = mCatchName;
   mInternal->mIdToNameMap[mFinallyId] = mFinallyName;

   mInternal->mIdToNameMap[mOpDotId          ] = mOpDotName;
   mInternal->mIdToNameMap[mOpCommaId        ] = mOpCommaName;
   mInternal->mIdToNameMap[mOpAbstractId     ] = mOpAbstractName;
   mInternal->mIdToNameMap[mOpOpenParenId    ] = mOpOpenParenName;
   mInternal->mIdToNameMap[mOpCloseParenId   ] = mOpCloseParenName;
   mInternal->mIdToNameMap[mOpOpenBraceId    ] = mOpOpenBraceName;
   mInternal->mIdToNameMap[mOpCloseBraceId   ] = mOpCloseBraceName;
   mInternal->mIdToNameMap[mOpOpenBracketId  ] = mOpOpenBracketName;
   mInternal->mIdToNameMap[mOpCloseBracketId ] = mOpCloseBracketName;
   mInternal->mIdToNameMap[mOpAssignId       ] = mOpAssignName;
   mInternal->mIdToNameMap[mOpStoreId        ] = mOpStoreName;
   mInternal->mIdToNameMap[mOpScopeId        ] = mOpScopeName;

   mInternal->mIdToNameMap[mOpConsId         ] = mOpConsName;
   mInternal->mIdToNameMap[mOpPlusId         ] = mOpPlusName;
   mInternal->mIdToNameMap[mOpMinusId        ] = mOpMinusName;
   mInternal->mIdToNameMap[mOpTimesId        ] = mOpTimesName;
   mInternal->mIdToNameMap[mOpDivideId       ] = mOpDivideName;
   mInternal->mIdToNameMap[mOpModulusId      ] = mOpModulusName;
   mInternal->mIdToNameMap[mOpPowerId        ] = mOpPowerName;
   mInternal->mIdToNameMap[mOpEqualId        ] = mOpEqualName;
   mInternal->mIdToNameMap[mOpLessId         ] = mOpLessName;
   mInternal->mIdToNameMap[mOpGreaterId      ] = mOpGreaterName;
   mInternal->mIdToNameMap[mOpLessEqualId    ] = mOpLessEqualName;
   mInternal->mIdToNameMap[mOpGreaterEqualId ] = mOpGreaterEqualName;
   mInternal->mIdToNameMap[mOpNotEqualId     ] = mOpNotEqualName;
   mInternal->mIdToNameMap[mOpAndId          ] = mOpAndName;
   mInternal->mIdToNameMap[mOpOrId           ] = mOpOrName;
   mInternal->mIdToNameMap[mOpNotId          ] = mOpNotName;
}

BasicKeywordSet::~BasicKeywordSet()
{
   delete mInternal;
}

// KeywordSet --------------------------------------------------------
util::Atom BasicKeywordSet::getKeywordId(String name)
{
   return mInternal->mNameToIdMap[name];
}

String BasicKeywordSet::getKeywordName(util::Atom id)
{
   return mInternal->mIdToNameMap[id];
}

bool BasicKeywordSet::isBlockKeyword(util::Atom id)
{
   return id == mWhereId || id == mOfId || id == mDoId;// || id == mCatchId;
}

} //namespace basic
} //namespace dialects
} //namespace compiler
} //namespace evlan
