/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef evlan_compiler_dialects_basic_BasicKeywordSet_h__INCLUDED
#define evlan_compiler_dialects_basic_BasicKeywordSet_h__INCLUDED

// includes ==========================================================
#include <framework/common/all.h>
#include <evlan/compiler/syntax/KeywordSet.h>

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace dialects {
namespace basic {
using namespace ::framework::common;

//Fuck the C++ standard for disallowing this, and shame on GCC for
//allowing it in previous versions then spontaneously breaking it.
//using ::framework::util;
namespace util {using namespace ::framework::util;}

// class definition ==================================================
class BasicKeywordSet;

typedef SmartPointer<BasicKeywordSet> BasicKeywordSetPtr;

class BasicKeywordSet: implements syntax::KeywordSet
{
public:
   BasicKeywordSet();
   ~BasicKeywordSet();

   const String mTrueName;
   const String mFalseName;
   const String mIfName;
   const String mThenName;
   const String mElseName;
   const String mWhereName;
   const String mOfName;
   const String mObjectName;
   const String mArrayName;
   const String mImportName;
   const String mDoName;
   const String mReturnName;
   const String mThrowName;
   const String mTryName;
   const String mCatchName;
   const String mFinallyName;

   const String mOpDotName;
   const String mOpCommaName;
   const String mOpAbstractName;
   const String mOpOpenParenName;
   const String mOpCloseParenName;
   const String mOpOpenBraceName;
   const String mOpCloseBraceName;
   const String mOpOpenBracketName;
   const String mOpCloseBracketName;
   const String mOpAssignName;
   const String mOpStoreName;
   const String mOpScopeName;

   const String mOpConsName;
   const String mOpPlusName;
   const String mOpMinusName;
   const String mOpTimesName;
   const String mOpDivideName;
   const String mOpModulusName;
   const String mOpPowerName;
   const String mOpEqualName;
   const String mOpLessName;
   const String mOpGreaterName;
   const String mOpLessEqualName;
   const String mOpGreaterEqualName;
   const String mOpNotEqualName;
   const String mOpAndName;
   const String mOpOrName;
   const String mOpNotName;

   const util::Atom mTrueId;
   const util::Atom mFalseId;
   const util::Atom mIfId;
   const util::Atom mThenId;
   const util::Atom mElseId;
   const util::Atom mWhereId;
   const util::Atom mOfId;
   const util::Atom mObjectId;
   const util::Atom mArrayId;
   const util::Atom mImportId;
   const util::Atom mDoId;
   const util::Atom mReturnId;
   const util::Atom mThrowId;
   const util::Atom mTryId;
   const util::Atom mCatchId;
   const util::Atom mFinallyId;

   const util::Atom mOpDotId;
   const util::Atom mOpCommaId;
   const util::Atom mOpAbstractId;
   const util::Atom mOpOpenParenId;
   const util::Atom mOpCloseParenId;
   const util::Atom mOpOpenBraceId;
   const util::Atom mOpCloseBraceId;
   const util::Atom mOpOpenBracketId;
   const util::Atom mOpCloseBracketId;
   const util::Atom mOpAssignId;
   const util::Atom mOpStoreId;
   const util::Atom mOpScopeId;

   const util::Atom mOpConsId;
   const util::Atom mOpPlusId;
   const util::Atom mOpMinusId;
   const util::Atom mOpTimesId;
   const util::Atom mOpDivideId;
   const util::Atom mOpModulusId;
   const util::Atom mOpPowerId;
   const util::Atom mOpEqualId;
   const util::Atom mOpLessId;
   const util::Atom mOpGreaterId;
   const util::Atom mOpLessEqualId;
   const util::Atom mOpGreaterEqualId;
   const util::Atom mOpNotEqualId;
   const util::Atom mOpAndId;
   const util::Atom mOpOrId;
   const util::Atom mOpNotId;

   // KeywordSet --------------------------------------------------
   util::Atom getKeywordId(String name);
   String getKeywordName(util::Atom id);
   bool isBlockKeyword(util::Atom id);

private:
   struct Internal;
   Internal* mInternal;  //insulate use of STL map<>

   //disable copying
   BasicKeywordSet(const BasicKeywordSet&);
   BasicKeywordSet& operator=(const BasicKeywordSet&);
};

} //namespace basic
} //namespace dialects
} //namespace compiler
} //namespace evlan

#endif //included
