/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef evlan_compiler_ParseObjects_h__INCLUDED
#define evlan_compiler_ParseObjects_h__INCLUDED

// includes ==========================================================
#include <framework/common/all.h>
#include <evlan/compiler/grammar/ParseTree.h>

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace dialects {
namespace basic {
using namespace ::framework::common;

// definition ========================================================
namespace ParseObjects
{
   using grammar::ParseTreePtr;

   // helpers --------------------------------------------------------
   struct WhereItem
   {
      int line;
      int column;
      String documentation;
      String name;
      ParseTreePtr value;
   };
   struct MemberDefinition
   {
      int line;
      int column;
      String documentation;
      String name;
      ParseTreePtr value;
   };
   struct DoStatement
   {
      int line;
      int column;
      String variable;
      ParseTreePtr body;
      bool isProcedure;
   };

   // factory functions ----------------------------------------------
   ParseTreePtr makeImport(int line, int column, String name);
   ParseTreePtr makeImport(int line, int column, Array<const char8 > name);
   ParseTreePtr makeImport(int line, int column, Array<const char16> name);
   ParseTreePtr makeImport(int line, int column, Array<const char32> name);

   ParseTreePtr makeLiteral(int line, int column,
                            Expression::Literal value);
   ParseTreePtr makeVariable(int line, int column, String name, int depth = 0);

   ParseTreePtr makeMemberAccess(int line, int column,
                                 ParseTreePtr object, String member);
   ParseTreePtr makeArray(int line, int column,
                          Array<const ParseTreePtr> elements);
   ParseTreePtr makeConditional(int line, int column, ParseTreePtr input,
                                ParseTreePtr thenClause, ParseTreePtr elseClause);
   ParseTreePtr makeWhereBlock(int line, int column, ParseTreePtr output,
                               Array<const WhereItem> items);

   ParseTreePtr makeFunction(int line, int column,
                             Array<const String> inputNames, ParseTreePtr body);
   ParseTreePtr makeApplication(int line, int column,
                                ParseTreePtr function,
                                Array<ParseTreePtr> parameter);
   ParseTreePtr makeApplication(int line, int column,
                                ParseTreePtr function,
                                ParseTreePtr parameter);

   ParseTreePtr makeObject(int line, int column,
                           Array<const MemberDefinition> members);

   ParseTreePtr makeDo(int line, int column,
                       Array<const DoStatement> statements,
                       ParseTreePtr returnValue);
};

} //namespace dialects
} //namespace basic
} //namespace compiler
} //namespace evlan

#endif //included
