/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
#include "Environment.h"
#include <map>

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace grammar {

// class members =====================================================
struct Environment::Internal
{
   std::map<String, Array<Index> > identifierMap;
   int count;

   void add(String identifier, Index index);
};

Environment::Environment()
   : mInternal(spnew Internal)
{
   mInternal->count = 0;
}

Environment::~Environment()
{
}

Environment::Index Environment::getIndex(String identifier, int depth) const
{
   std::map<String, Array<Index> >::iterator i = mInternal->identifierMap.find(identifier);
   if(i == mInternal->identifierMap.end() || depth >= i->second.getSize())
      return Index(-1, -1);
   else
      return i->second[depth];
}

Environment Environment::push(String identifier) const
{
   //copy current map
   InternalPtr newInternal = spnew Internal(*mInternal);
   //update
   newInternal->add(identifier, Index(newInternal->count, 0));
   ++newInternal->count;
   //return
   return Environment(newInternal);
}

Environment Environment::push(Array<const String> identifiers) const
{
   //copy current map
   InternalPtr newInternal = spnew Internal(*mInternal);
   //update
   for(int i = 0; i < identifiers.getSize(); i++)
      newInternal->add(identifiers[i], Index(newInternal->count, i));
   ++newInternal->count;
   //return
   return Environment(newInternal);
}

void Environment::Internal::add(String identifier, Index index)
{
   //FIXME (KV):  This is inefficient, but won't be used all that much.
   std::map<String, Array<Index> >::iterator i = identifierMap.find(identifier);
   if(i == identifierMap.end())
   {
      Array<Index> newArray(1);
      newArray[0] = index;
      identifierMap[identifier] = newArray;
   }
   else
   {
      Array<Index> newArray(i->second.getSize() + 1);
      memcpy(&newArray[1], &i->second[0], sizeof(Index) * i->second.getSize());
      newArray[0] = index;
      i->second = newArray;
   }
}

} //namespace grammar
} //namespace compiler
} //namespace evlan
