/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef evlan_compiler_grammar_Environment_h__INCLUDED
#define evlan_compiler_grammar_Environment_h__INCLUDED

// includes ==========================================================
#include <framework/common/all.h>

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace grammar {
using namespace ::framework::common;

// class definition ==================================================
class Environment;

class Environment
{
public:
   //"Index" represents an index to a location in the environment stack.
   //Example:
   //   x => x4 where
   //      x2 = x * x
   //      x4 = x2 * x2
   //From anywhere inside the where block, the following bindings are visible:
   //   variable  (level, item)
   //   x         (0, 0)
   //   x2        (1, 0)
   //   x4        (1, 1)
   struct Index
   {
      //all items in the same object or where block are at the same "level"
      int level;
      int item;

      inline Index() {}
      inline Index(int l, int i)
         : level(l), item(i) {}
   };

   Environment();   //make empty environment
   ~Environment();
   Index getIndex(String identifier, int depth = 0) const;
   Environment push(String identifier) const;
   Environment push(Array<const String> identifiers) const;

private:
   struct Internal;
   typedef SmartPointer<Internal> InternalPtr;
   InternalPtr mInternal;
   inline Environment(InternalPtr internal): mInternal(internal) {}
};

} //namespace grammar
} //namespace compiler
} //namespace evlan

#endif //included
