/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef evlan_compiler_grammar_ParseTree_h__INCLUDED
#define evlan_compiler_grammar_ParseTree_h__INCLUDED

// includes ==========================================================
#ifdef INCLUDE_WRITER_CODE
#include <framework/TextWriter.h>
#endif
#include <evlan/compiler/ErrorCollector.h>
#include <evlan/compiler/Expression.h>
#include "Environment.h"

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace grammar {
using namespace ::framework::common;

// class definition ==================================================
interface ParseTree;
typedef SmartPointer<ParseTree> ParseTreePtr;

interface ParseTree: implements Object
{
   virtual ~ParseTree();

   virtual Expression toExpression(Environment context,
                                   ErrorCollectorPtr errorCollector) = 0;
   virtual void setDocumentation(String documentation) = 0;
#ifdef INCLUDE_WRITER_CODE
   virtual void print(String indent, framework::TextWriterPtr out) = 0;
#endif
};

//use this when there's some error preventing you from construction a parse
// tree.
ParseTreePtr makeEmptyParseTree(int line, int column);

} //namespace grammar
} //namespace compiler
} //namespace evlan

#endif //included
