/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
#include "TokenFilter.h"
#include <framework/util/Vector.h>
#include <stack>

// begin namespace ===================================================
namespace evlan {
namespace compiler {
namespace syntax {

//Fuck the C++ standard for disallowing this, and shame on GCC for
//allowing it in previous versions then spontaneously breaking it.
//using ::framework::util;
namespace util {using namespace ::framework::util;}

Array<const Token> filterTokens(Array<const Token> source)
{
   util::Vector<Token> tokens;

   enum
   {
      START_STATEMENT,
      AFTER_INDENT,
      AFTER_DOCUMENT_LINE,
      IN_CODE
   } state = IN_CODE;

   std::stack<int> indentLevels;
   indentLevels.push(-1);
   util::Vector<String> docLines;
   int line = 0, column = 0;

   for(int i = 0; i < source.getSize(); i++)
   {
      FW_MATCH_START(source[i].value)

      FW_MATCH_CASE_1(Token::Value::whitespace, amount)
         if(state == START_STATEMENT && amount == indentLevels.top())
            state = AFTER_INDENT;

      FW_MATCH_CASE_0(Token::Value::endOfLine)
         if(state == START_STATEMENT || state == AFTER_INDENT)
         {
            docLines.clear();
            state = START_STATEMENT;
         }
         else if(state == AFTER_DOCUMENT_LINE)
            state = START_STATEMENT;

      FW_MATCH_CASE_1(Token::Value::comment, text)
         if(state == AFTER_INDENT
         || (state == START_STATEMENT && indentLevels.top() == 0))
         {
            if(docLines.getSize() == 0)
            {
               line = source[i].line;
               column = source[i].column;
            }
            docLines.pushBack(text);
            state = AFTER_DOCUMENT_LINE;
         }
         else if(state != IN_CODE)
            docLines.clear();

      FW_MATCH_CASE_0(Token::Value::endOfStatement)
         state = START_STATEMENT;
         tokens.pushBack(source[i]);

      FW_MATCH_CASE_1(Token::Value::startOfBlock, indentLevel)
         state = START_STATEMENT;
         indentLevels.push(indentLevel);
         tokens.pushBack(source[i]);

      FW_MATCH_CASE_0(Token::Value::endOfBlock)
         state = IN_CODE;
         indentLevels.pop();
         tokens.pushBack(source[i]);

      FW_MATCH_DEFAULT
         if(state != IN_CODE)
         {
            if(docLines.getSize() > 0)
            {
               //combine documentation lines
               int length = 0;
               for(int i = 0; i < docLines.getSize(); i++)
                  length += docLines[i].getLength() + 1;
               Array<char> buffer(length);
               length = 0;
               for(int i = 0; i < docLines.getSize(); i++)
               {
                  docLines[i].copyToBuffer(&buffer[length], 0, docLines[i].getLength(), false);
                  length += docLines[i].getLength();
                  buffer[length++] = '\n';
               }
               docLines.clear();
               String doc = String(buffer.toConst());

               tokens.pushBack(Token(line, column, Token::Value::comment(doc)));
            }

            state = IN_CODE;
         }
         tokens.pushBack(source[i]);

      FW_MATCH_END
   }

   return tokens.getArray().toConst();
}

} //namespace syntax
} //namespace compiler
} //namespace evlan
