/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/04/01 (KV)
 *  + First Version
 */

// ===================================================================

#include "arrays.h"
#include "memory.h"

namespace evlan {
namespace vm {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::vm::arrays
// TODO (KV): Review evlan::vm::arrays

namespace arrays
{
   void getElement(int index)
   {
      if(core::gStackTop->isArray())
      {
         if(index < 0
         || index >= core::gStackTop->array.size)
            *core::gStackTop = core::Value::makeError(core::Value::OUT_OF_BOUNDS, null);
         else
            *core::gStackTop = core::gStackTop->array.elements[index];
      }
      else if(core::gStackTop->isPackedArray())
      {
         if(index < 0
         || index >= core::gStackTop->packedArray.size)
            *core::gStackTop = core::Value::makeError(core::Value::OUT_OF_BOUNDS, null);
         else
         {
            index += core::gStackTop->packedArray.offset;

            int elementSize = core::gStackTop->packedArray.elementSize;

            switch(core::gStackTop->packedArray.packType)
            {
            case core::Value::PACKED_BOOLEAN:
               if(((byte*)core::gStackTop->packedArray.elements)[index/8] & (1 << (7-index%8)))
                  *core::gStackTop = core::Value::makeBoolean(true);
               else
                  *core::gStackTop = core::Value::makeBoolean(false);
               break;
            case core::Value::PACKED_SCALAR:
               *core::gStackTop = core::Value::makeScalar(((double*)core::gStackTop->packedArray.elements)[index]);
               break;

            default:
               if(elementSize < 6)
               {
                  uint32 result;

                  switch(elementSize)
                  {
                  case 0:
                     result = (((byte*)core::gStackTop->packedArray.elements)[index/8] >> (7-index%8)) & 1;
                     break;
                  case 1:
                     result = (((byte*)core::gStackTop->packedArray.elements)[index/4] >> (6-index%4*2)) & 3;
                     break;
                  case 2:
                     result = (((byte*)core::gStackTop->packedArray.elements)[index/2] >> (4-index%2*4)) & 15;
                     break;
                  case 3:
                     result = ((byte*)core::gStackTop->packedArray.elements)[index];
                     break;
                  case 4:
                     result = ((uint16*)core::gStackTop->packedArray.elements)[index];
                     break;
                  default:
                  case 5:
                     result = ((uint32*)core::gStackTop->packedArray.elements)[index];
                     break;
                  }

                  if(core::gStackTop->packedArray.packType == core::Value::PACKED_INTEGER)
                  {
                     if(elementSize < 5)
                     {
                        int bitsize = (1 << elementSize);
                        if(result & (1 << (bitsize - 1)))
                           result |= ~((1 << bitsize) - 1); //sign-extend
                     }
                     *core::gStackTop = core::Value::makeScalar((double)(int32)result);
                  }
                  else if(core::gStackTop->packedArray.packType == core::Value::PACKED_CHARACTER)
                     *core::gStackTop = core::Value::makeBuiltin(result, core::Value::CHARACTER);
                  else if(core::gStackTop->packedArray.packType == core::Value::PACKED_ATOM)
                     *core::gStackTop = core::Value::makeBuiltin(result, core::Value::ATOM);
                  else
                     *core::gStackTop = core::Value::makeBuiltin(result, (core::Value::BuiltinType)(core::Value::DATA1 + elementSize));
               }
               else
               {
                  *core::gStackTop = core::Value::makeBuiltin(
                     ((uint64*)core::gStackTop->packedArray.elements)[index],
                     core::Value::DATA64);
               }
               break;
            }
         }
      }
      else if(!core::gStackTop->isError())
         *core::gStackTop = core::Value::makeError(core::Value::TYPE_ERROR, null);
   }

   void getElement()
   {
      if(core::gStackTop->isScalar())
      {
         int intval = (int)core::gStackTop->builtin.value.scalar;
         if(core::gStackTop->builtin.value.scalar == (double)intval)
         {
            ++core::gStackTop;
            getElement(intval);
            return;
         }
      }
      else if(core::gStackTop->isError())
      {
         core::gStackTop[1] = core::gStackTop[0];
         ++core::gStackTop;
         return;
      }

      ++core::gStackTop;
      *core::gStackTop = core::Value::makeError(core::Value::TYPE_ERROR, null);
   }

   void tryPackArray()
   {
      if(!core::gStackTop->isArray())
         return;

      int size = core::gStackTop->array.size;
      if(size <= 0)
         return;

      core::Value* target = core::gStackTop->array.elements;

      if(target[0].type != core::Value::BUILTIN_OBJECT)
         return;
      int type = target[0].builtin.builtinType;
      uint32 maximum = 0;
      bool hasNonInteger = false;

      for(int i = 0; i < size; i++)
      {
         if(target[i].type != core::Value::BUILTIN_OBJECT
         || target[i].builtin.builtinType != type)
            return;
         if(type == core::Value::SCALAR && !hasNonInteger)
         {
            int intValue = (int)target[i].builtin.value.scalar;
            if(target[i].builtin.value.scalar == (double)intValue)
            {
               if(intValue < 0)
                  intValue = -intValue;
               if(maximum < (uint)intValue)
                  maximum = intValue;
            }
            else
               hasNonInteger = true;
         }
         else if(type == core::Value::CHARACTER
              || type == core::Value::ATOM)
         {
            uint32 intValue = target[i].builtin.value.data32;
            if(maximum < intValue)
               maximum = intValue;
         }
      }

      int packType;
      int elementSize;

      switch(type)
      {
      case core::Value::BOOLEAN:
         packType = core::Value::PACKED_BOOLEAN;
         elementSize = 0;
         break;
      case core::Value::SCALAR:
         if(hasNonInteger)
         {
            packType = core::Value::PACKED_SCALAR;
            elementSize = 6;
         }
         else
         {
            packType = core::Value::PACKED_INTEGER;
            //Don't pack smaller 8-bit, since such sizes aren't well-optimized.
//            if(maximum < (1 << 1))
//               elementSize = 1;
//            else if(maximum < (1 << 3))
//               elementSize = 2;
//            else
            if(maximum < (1 << 7))
               elementSize = 3;
            else if(maximum < (1 << 15))
               elementSize = 4;
            else //maximum < (1 << 31)
               elementSize = 5;
         }
         break;
      case core::Value::CHARACTER:
         packType = core::Value::PACKED_CHARACTER;
         if(maximum < (1 << 8))
            elementSize = 3;
         else if(maximum < (1 << 16))
            elementSize = 4;
         else //maximum < (1 << 32)
            elementSize = 5;
         break;
      case core::Value::ATOM:
         packType = core::Value::PACKED_ATOM;
         if(maximum < (1 << 8))
            elementSize = 3;
         else if(maximum < (1 << 16))
            elementSize = 4;
         else //maximum < (1 << 32)
            elementSize = 5;
         break;
      default:
         packType = core::Value::PACKED_DATA;
         elementSize = type - core::Value::DATA1;
         break;
      }

      core::gStackTop->type = core::Value::PACKED_ARRAY;
      core::gStackTop->packedArray.packType = packType;
      core::gStackTop->packedArray.elementSize = elementSize;
      core::gStackTop->packedArray.elements = (word*)target;
      core::gStackTop->packedArray.offset = 0;
      core::gStackTop->packedArray.size = size;

      int wordSize;
      if(elementSize > core::LOG2_WORD_SIZE)
         wordSize = size << (elementSize - core::LOG2_WORD_SIZE);
      else if(elementSize == core::LOG2_WORD_SIZE)
         wordSize = size;
      else
      {
         int diff = (core::LOG2_WORD_SIZE - elementSize);
         wordSize = size >> diff;
         if((size & ((1 << diff) - 1)) != 0)
            ++wordSize;
      }

#define COPY_SMALL(BITS, VARIABLE, TYPE)                              \
      for(int i = 0; i < size; i++)                                   \
      {                                                               \
         if(i%(8/BITS) == 0)                                          \
            ((byte*)target)[i/(8/BITS)] = 0;                          \
         ((byte*)target)[i/(8/BITS)] |=                               \
            ((TYPE)target[i].builtin.value.VARIABLE &                 \
            ((1 << BITS) - 1)) << ((8-BITS)-(i%(8/BITS))*BITS);       \
      }
#define COPY_LARGE(TYPE, VARIABLE)                                    \
      for(int i = 0; i < size; i++)                                   \
         ((TYPE*)target)[i] = (TYPE)target[i].builtin.value.VARIABLE;

      switch(packType)
      {
      case core::Value::PACKED_BOOLEAN:
         for(int i = 0; i < size; i++)
         {
            if(i%8 == 0)
               ((byte*)target)[i/8] = 0;
            if(target[i].builtin.value.boolean)
               ((byte*)target)[i/8] |= 1 << (7-(i%8));
         }
         break;

      case core::Value::PACKED_SCALAR:
         for(int i = 0; i < size; i++)
            ((double*)target)[i] = target[i].builtin.value.scalar;
         break;

      case core::Value::PACKED_INTEGER:
         switch(elementSize)
         {
         case 0:
            COPY_SMALL(1, scalar, int8);
            break;
         case 1:
            COPY_SMALL(2, scalar, int8);
            break;
         case 2:
            COPY_SMALL(4, scalar, int8);
            break;
         case 3:
            COPY_LARGE(int8, scalar);
            break;
         case 4:
            COPY_LARGE(int16, scalar);
            break;
         case 5:
            COPY_LARGE(int32, scalar);
            break;
         }
         break;

      case core::Value::PACKED_CHARACTER:
      case core::Value::PACKED_ATOM:
      case core::Value::PACKED_DATA:
         switch(elementSize)
         {
         case 0:
            COPY_SMALL(1, data32, uint8);
            break;
         case 1:
            COPY_SMALL(2, data32, uint8);
            break;
         case 2:
            COPY_SMALL(4, data32, uint8);
            break;
         case 3:
            COPY_LARGE(int8, data32);
            break;
         case 4:
            COPY_LARGE(int16, data32);
            break;
         case 5:
            COPY_LARGE(int32, data32);
            break;
         case 6:
            COPY_LARGE(int64, data64);
            break;
         }
         break;
      }
#undef COPY_SMALL
#undef COPY_LARGE

      word* oldEnd = (word*)(target + size);
      word* newEnd = (word*)target + wordSize;

      memory::returnWords(newEnd, oldEnd - newEnd);
   }

   void repackData(void* target, const uint32* source, int size, int elementSize)
   {
      switch(elementSize)
      {
      case 0:
      case 1:
      case 2:
      case 3:
         size *= 4;
#ifdef PLATFORM_BIG_ENDIAN
         memcpy(target, source, size);
#else
         for(int i = 0; i < size; i++)
            ((byte*)target)[i] = ((const byte*)source)[i^3];
#endif
         break;
      case 4:
         size *= 2;
#ifdef PLATFORM_BIG_ENDIAN
         memcpy(target, source, size * 2);
#else
         for(int i = 0; i < size; i++)
            ((uint16*)target)[i] = ((const uint16*)source)[i^1];
#endif
         break;
      case 5:
         memcpy(target, source, size * 4);
         break;
      case 6:
#ifdef PLATFORM_BIG_ENDIAN
         memcpy(target, source, size * 4);
#else
         for(int i = 0; i < size; i++)
            ((uint32*)target)[i] = source[i^1];
#endif
         break;
      }
   }


   void makeString(char* string)
   {
      int length = strlen(string);
      int wordSize = length / sizeof(word);
      if(length % sizeof(word) > 0)
         ++wordSize;
      word* target = null;
      if(wordSize > 0)
      {
         target = memory::allocateWords(wordSize);
         memcpy(target, string, length);
      }

      --core::gStackTop;
      core::gStackTop->type = core::Value::PACKED_ARRAY;
      core::gStackTop->packedArray.packType    = core::Value::PACKED_CHARACTER;
      core::gStackTop->packedArray.elementSize = 3;
      core::gStackTop->packedArray.offset      = 0;
      core::gStackTop->packedArray.size        = length;
      core::gStackTop->packedArray.elements    = target;
   }

   void makeString(char* string, int length)
   {
      int wordSize = length / sizeof(word);
      if(length % sizeof(word) > 0)
         ++wordSize;
      word* target = null;
      if(wordSize > 0)
      {
         target = memory::allocateWords(wordSize);
         memcpy(target, string, length);
      }

      --core::gStackTop;
      core::gStackTop->type = core::Value::PACKED_ARRAY;
      core::gStackTop->packedArray.packType    = core::Value::PACKED_CHARACTER;
      core::gStackTop->packedArray.elementSize = 3;
      core::gStackTop->packedArray.offset      = 0;
      core::gStackTop->packedArray.size        = length;
      core::gStackTop->packedArray.elements    = target;
   }

   void makeString(String string)
   {
      int length = string.getLength();
      int wordSize = length / sizeof(word);
      if(length % sizeof(word) > 0)
         ++wordSize;
      word* target = null;
      if(wordSize > 0)
      {
         target = memory::allocateWords(wordSize);
         memcpy(target, &string.getBuffer()[0], length);
      }

      --core::gStackTop;
      core::gStackTop->type = core::Value::PACKED_ARRAY;
      core::gStackTop->packedArray.packType    = core::Value::PACKED_CHARACTER;
      core::gStackTop->packedArray.elementSize = 3;
      core::gStackTop->packedArray.offset      = 0;
      core::gStackTop->packedArray.size        = length;
      core::gStackTop->packedArray.elements    = target;
   }

   void copyString(char* buffer, core::Value value, bool nullTerminate)
   {
      int length = getStringLength(value);

      if(length > 0)
      {
         int offset = value.packedArray.offset;
         switch(value.packedArray.elementSize)
         {
         default:
         case 3:
            memcpy(buffer, (byte*)value.packedArray.elements + offset, length);
            break;
         case 4:
            for(int i = 0; i < length; i++)
               buffer[i] = ((uint16*)value.packedArray.elements)[i + offset];
            break;
         case 5:
            for(int i = 0; i < length; i++)
               buffer[i] = ((uint32*)value.packedArray.elements)[i + offset];
            break;
         }
      }
      if(nullTerminate)
         buffer[length] = '\0';
   }

   String getString(core::Value value)
   {
      int length = getStringLength(value);
      String result;

      if(length > 0)
      {
         Array<char> buffer(length);

         int offset = value.packedArray.offset;
         switch(value.packedArray.elementSize)
         {
         default:
         case 3:
            memcpy(&buffer[0], (byte*)value.packedArray.elements + offset, length);
            break;
         case 4:
            for(int i = 0; i < length; i++)
               buffer[i] = ((uint16*)value.packedArray.elements)[i + offset];
            break;
         case 5:
            for(int i = 0; i < length; i++)
               buffer[i] = ((uint32*)value.packedArray.elements)[i + offset];
            break;
         }
         result = String(buffer.toConst());
      }

      return result;
   }

   char* getCString(core::Value value)
   {
      int length = getStringLength(value);
      char* buffer = new char[length + 1];

      if(length > 0)
      {
         int offset = value.packedArray.offset;
         switch(value.packedArray.elementSize)
         {
         default:
         case 3:
            memcpy(buffer, (byte*)value.packedArray.elements + offset, length);
            break;
         case 4:
            for(int i = 0; i < length; i++)
               buffer[i] = ((uint16*)value.packedArray.elements)[i + offset];
            break;
         case 5:
            for(int i = 0; i < length; i++)
               buffer[i] = ((uint32*)value.packedArray.elements)[i + offset];
            break;
         }
      }
      buffer[length] = '\0';

      return buffer;
   }
}

// ===================================================================

} //namespace vm
} //namespace evlan
