/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/02/10 (KV)
 *  + First Version
 */

// ===================================================================

#include "core.h"

namespace evlan {
namespace vm {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::vm::Core
// TODO (KV): Review evlan::vm::Core

namespace core
{
   Value* gStackTop EVLAN_VM_CORE_EXPORT = null;
   Value* gStackEnd EVLAN_VM_CORE_EXPORT = null;

   Task* gTaskQueue EVLAN_VM_CORE_EXPORT = null;
   int gTaskQueueSize EVLAN_VM_CORE_EXPORT = 0;

#ifdef INCLUDE_WRITER_CODE
   void printBuiltinType(TextWriterPtr out, int id)
   {
      switch(id)
      {
      case Value::BOOLEAN:
         out->write($("boolean"));
         break;
      case Value::SCALAR:
         out->write($("scalar"));
         break;
      case Value::CHARACTER:
         out->write($("character"));
         break;
      case Value::ATOM:
         out->write($("atom"));
         break;
      default:
         {
            int lg2size = id - Value::DATA1;
            if(lg2size <= 30)
            {
               out->write($("data"));
               out->write(String(1 << lg2size));
            }
            else
               out->write($("??"));
         }
         break;
      }
   }

   void printBuiltin(TextWriterPtr out, const Value& value)
   {
      printBuiltinType(out, value.builtin.builtinType);
      switch(value.builtin.builtinType)
      {
      case Value::BOOLEAN:
         out->write(value.builtin.value.boolean?$("(true)"):$("(false)"));
         break;
      case Value::SCALAR:
         out->write($("("));
         out->write(String(value.builtin.value.scalar));
         out->write($(")"));
         break;
      case Value::CHARACTER:
         out->write($("("));
         switch((char)value.builtin.value.data32)
         {
         case '\a':
            out->write($("\'\\a\'"));
            break;
         case '\b':
            out->write($("\'\\b\'"));
            break;
         case '\f':
            out->write($("\'\\f\'"));
            break;
         case '\n':
            out->write($("\'\\n\'"));
            break;
         case '\r':
            out->write($("\'\\r\'"));
            break;
         case '\t':
            out->write($("\'\\t\'"));
            break;
         case '\v':
            out->write($("\'\\v\'"));
            break;
         case '\'':
            out->write($("\'\\\'\'"));
            break;
         case '\"':
            out->write($("\'\\\"\'"));
            break;
         case '\\':
            out->write($("\'\\\\\'"));
            break;
         default:
            if(value.builtin.value.data32 < ' ')
            {
               out->write($("0x"));
               out->write(String(value.builtin.value.data32, 2, 16));
            }
            else if(value.builtin.value.data32 < 0x00000100)
            {
               out->write($("\'"));
               out->write((char)value.builtin.value.data32, 1);
               out->write($("\'"));
            }
            else
            {
               out->write($("0x"));
               out->write(String(value.builtin.value.data32, 8, 16));
            }
            break;
         }
         out->write($(")"));
         break;
      case Value::ATOM:
         out->write($("("));
         out->write(String(value.builtin.value.data32));
         out->write($(")"));
         break;
      case Value::DATA1:
         out->write($("(0b"));
         out->write((value.builtin.value.data32 & 0x00000001)?$("0"):$("1"));
         out->write($(")"));
         break;
      case Value::DATA2:
         out->write($("(0b"));
         out->write((value.builtin.value.data32 & 0x00000002)?$("0"):$("1"));
         out->write((value.builtin.value.data32 & 0x00000001)?$("0"):$("1"));
         out->write($(")"));
         break;
      case Value::DATA4:
         out->write($("(0x"));
         out->write(String(value.builtin.value.data32 & 0x0000000F, 1, 16));
         out->write($(")"));
         break;
      case Value::DATA8:
         out->write($("(0x"));
         out->write(String(value.builtin.value.data32 & 0x000000FF, 2, 16));
         out->write($(")"));
         break;
      case Value::DATA16:
         out->write($("(0x"));
         out->write(String(value.builtin.value.data32 & 0x0000FFFF, 4, 16));
         out->write($(")"));
         break;
      case Value::DATA32:
         out->write($("(0x"));
         out->write(String(value.builtin.value.data32, 8, 16));
         out->write($(")"));
         break;
      case Value::DATA64:
         out->write($("(0x"));
         out->write(String(value.builtin.value.data64, 16, 16));
         out->write($(")"));
         break;
      }
   }

   void writeValue(TextWriterPtr out, const Value& value)
   {
      switch(value.type)
      {
      case Value::EMPTY:
         out->write($("empty"));
         break;
      case Value::BUILTIN_OBJECT:
         out->write($("builtin object: "));
         printBuiltin(out, value);
         break;
      case Value::NATIVE_CONTAINER:
         out->write($("native container"));
         break;
      case Value::NATIVE_FUNCTION:
         out->write($("native function: tag = 0x"));
         out->write(String((word)value.function.tag, sizeof(word)*2, 16));
         break;
      case Value::RUNTIME_ERROR:
         out->write($("runtime error: "));
         switch(value.runtimeError.errorCode)
         {
         case Value::NOT_A_FUNCTION:
            out->write($("not a function"));
            break;
         case Value::ARITY_MISMATCH:
            out->write($("arity mismatch"));
            break;
         case Value::NOT_AN_OBJECT:
            out->write($("not an object"));
            break;
         case Value::NO_SUCH_MEMBER:
            out->write($("no such member"));
            break;
         case Value::CONDITION_NOT_BOOLEAN:
            out->write($("condition not boolean"));
            break;
         case Value::TYPE_ERROR:
            out->write($("type error"));
            break;
         case Value::OUT_OF_BOUNDS:
            out->write($("out of bounds"));
            break;
         default:
            out->write($("unknown error"));
            break;
         }
         out->write($(", location = 0x"));
         out->write(String((word)value.runtimeError.location, sizeof(word)*2, 16));
         break;
      case Value::FUNCTION:
         out->write($("function: tag = 0x"));
         out->write(String((word)value.function.tag, sizeof(word)*2, 16));
         out->write($(", environment = 0x"));
         out->write(String((word)value.function.environment, sizeof(word)*2, 16));
         break;
      case Value::OBJECT:
         out->write($("object: tag = 0x"));
         out->write(String((word)value.object.tag, sizeof(word)*2, 16));
         out->write($(", members = 0x"));
         out->write(String((word)value.object.members, sizeof(word)*2, 16));
         break;
      case Value::ARRAY:
         out->write($("array: size = "));
         out->write(String((word)value.array.size));
         out->write($(", elements = 0x"));
         out->write(String((word)value.array.elements, sizeof(word)*2, 16));
         break;
      case Value::PACKED_ARRAY:
         out->write($("packed array: type = "));
         switch(value.packedArray.packType)
         {
         case core::Value::PACKED_BOOLEAN:
            out->write($("boolean"));
            break;
         case core::Value::PACKED_SCALAR:
            out->write($("scalar"));
            break;
         case core::Value::PACKED_INTEGER:
            out->write($("integer"));
            break;
         case core::Value::PACKED_CHARACTER:
            out->write($("character"));
            break;
         case core::Value::PACKED_DATA:
            out->write($("data"));
            break;
         default:
            out->write($("invalid"));
            break;
         }

         out->write($(", elementSize = "));
         out->write(String((int)value.packedArray.elementSize));
         out->write($(", offset = "));
         out->write(String((int)value.packedArray.offset));
         out->write($(", size = "));
         out->write(String(value.packedArray.size));
         out->write($(", elements = 0x"));
         out->write(String((word)value.packedArray.elements, sizeof(word)*2, 16));
         break;
      case Value::LARGE_DATA:
         out->write($("large data: size = "));
         out->write(String((int)value.largeData.size));
         out->write($(", value = 0x"));
         out->write(String((word)value.largeData.value, sizeof(word)*2, 16));
         break;

      case Value::TASK:
         out->write($("task: type = "));
         switch(value.task.taskType)
         {
            default:
            case Value::STOP:
               out->write($("stop"));
               break;
            case Value::CALL:
               out->write($("call"));
               break;
            case Value::RETURN:
               out->write($("return"));
               break;
            case Value::TRY:
               out->write($("try"));
               break;
            case Value::THROW:
               out->write($("throw"));
               break;
            case Value::FINALLY:
               out->write($("finally"));
               break;
         }
         out->write($(", parameters = 0x"));
         out->write(String((word)value.task.parameters, sizeof(word)*2, 16));
         break;

      case Value::CONTEXT:
         out->write($("context: type = "));
         switch(value.context.contextType)
         {
            default:
            case Value::STOP:
               out->write($("stop"));
               break;
            case Value::CALL:
               out->write($("call"));
               break;
            case Value::RETURN:
               out->write($("return"));
               break;
            case Value::TRY:
               out->write($("try"));
               break;
            case Value::THROW:
               out->write($("throw"));
               if(value.context.throwOnReturn)
                  out->write($("OnReturn"));
               break;
            case Value::FINALLY:
               out->write($("finally"));
               break;
         }
         out->write($(", parameter = 0x"));
         out->write(String((word)value.context.parameter, sizeof(word)*2, 16));
         out->write($(", parent = 0x"));
         out->write(String((word)value.context.parent, sizeof(word)*2, 16));
         break;

      case Value::MODULE:
         out->write($("module: module = 0x"));
         out->write(String((word)value.module.module, sizeof(word)*2, 16));
         break;

      case Value::PROCESS:
         out->write($("process: process = 0x"));
         out->write(String((word)value.process.process, sizeof(word)*2, 16));
         break;

      default:
         out->write($("invalid value"));
         break;
      }
   }

   void writeFunction(TextWriterPtr out, const FunctionTag* function)
   {
      out->write($("function "));
      out->write(String((word)function, sizeof(word)*2, 16));
      out->write($(":\n   module:   "));
      out->write(String((word)function->module, sizeof(word)*2, 16));
      out->write($("\n   arity:    "));
      out->write(String(function->arity));
      out->write($("\n   env size: "));
      out->write(String(function->environmentSize));
      out->write($("\n   instructions:\n"));
      for(int j = 0; j < function->instructionCount; j++)
      {
         out->write($("      "));
         switch(function->instructions[j].opcode)
         {
         case NOOP:
            out->write($("noop            "));
            break;
         case RETURN:
            out->write($("return          "));
            break;
         case GOTO:
            out->write($("goto            "));
            break;
         case TEST_CONDITION:
            out->write($("test            "));
            break;
         case PUSH_EMPTY:
            out->write($("push empty      "));
            break;
         case POP:
            out->write($("pop             "));
            break;
         case LOAD:
            out->write($("load stack      "));
            break;
         case LOAD_ENVIRONMENT:
            out->write($("load env        "));
            break;
         case LOAD_CONSTANT:
            out->write($("load const      "));
            break;
         case STORE:
            out->write($("store           "));
            break;
         case STORE_POP:
            out->write($("store pop       "));
            break;
         case GET_MEMBER:
            out->write($("member          "));
            break;
         case APPLY:
            out->write($("apply           "));
            break;
         case APPLY_RETURN:
            out->write($("apply tail      "));
            break;
         case MAKE_FUNCTION:
            out->write($("make function   "));
            break;
         case MAKE_OBJECT:
            out->write($("make object     "));
            break;
         case MAKE_ARRAY:
            out->write($("make array      "));
            break;
         case ALLOCATE_MULTI:
            out->write($("multi alloc     "));
            break;
         case MAKE_FUNCTION_MULTI:
            out->write($("multi function  "));
            break;
         case MAKE_OBJECT_MULTI:
            out->write($("multi object    "));
            break;
         case MAKE_ARRAY_MULTI:
            out->write($("multi array     "));
            break;
         case STORE_MULTI:
            out->write($("multi store     "));
            break;
         case STORE_MULTI_POP:
            out->write($("multi pop       "));
            break;

         case IMMEDIATE_INTEGER:
            out->write($("im int          "));
            break;
         case IMMEDIATE_NEGATIVE:
            out->write($("im neg          "));
            break;
         case IMMEDIATE_BOOLEAN:
            out->write($("im bool         "));
            break;
         case IMMEDIATE_CHARACTER:
            out->write($("im char         "));
            break;
         case IMMEDIATE_ATOM:
            out->write($("im atom         "));
            break;
         case IMMEDIATE_INTEGER32:
            out->write($("im int32        "));
            break;
         case IMMEDIATE_INTEGER64:
            out->write($("im int64        "));
            break;
         case IMMEDIATE_REAL32:
            out->write($("im real32       "));
            break;
         case IMMEDIATE_REAL64:
            out->write($("im real64       "));
            break;
         case IMMEDIATE_CHARACTER32:
            out->write($("im char32       "));
            break;
         case IMMEDIATE_ATOM32:
            out->write($("im atom32       "));
            break;
         case IMMEDIATE_DATA1:
            out->write($("im data1        "));
            break;
         case IMMEDIATE_DATA2:
            out->write($("im data2        "));
            break;
         case IMMEDIATE_DATA4:
            out->write($("im data4        "));
            break;
         case IMMEDIATE_DATA8:
            out->write($("im data8        "));
            break;
         case IMMEDIATE_DATA16:
            out->write($("im data16       "));
            break;
         case IMMEDIATE_DATA32:
            out->write($("im data32       "));
            break;
         case IMMEDIATE_DATA64:
            out->write($("im data64       "));
            break;

         case NOT:
            out->write($("not             "));
            break;
         case NEGATE:
            out->write($("negate          "));
            break;
         case SIZE:
            out->write($("size            "));
            break;
         case AND:
            out->write($("and             "));
            break;
         case OR:
            out->write($("or              "));
            break;
         case ADD:
            out->write($("add             "));
            break;
         case SUBTRACT:
            out->write($("subtract        "));
            break;
         case MULTIPLY:
            out->write($("multiply        "));
            break;
         case DIVIDE:
            out->write($("divide          "));
            break;
         case MODULUS:
            out->write($("modulus        "));
            break;
         case EQUALS:
            out->write($("equals          "));
            break;
         case IS_LESS_THAN:
            out->write($("isLessThan      "));
            break;
         case IS_GREATER_THAN:
            out->write($("isGreaterThan   "));
            break;
         case GET_ELEMENT:
            out->write($("getElement      "));
            break;

         default:
            out->write($("INVALID         "));
            break;
         }
         out->write(String(function->instructions[j].parameter, 8));

         if(function->sourceLocations != null)
         {
            out->write($("   "));
            out->write(String(function->sourceLocations[j].line + 1, 6));
            out->write($(":"));
            out->write(String(function->sourceLocations[j].column + 1, 3));
         }

         out->write($("\n"));
      }
   }

   void writeModule(TextWriterPtr out, const Module* module)
   {
      out->write($("module "));
      out->write(String((word)module, sizeof(word)*2, 16));
      out->write($(":\n   functions: "));
      out->write(String(module->functionTagCount));
      out->write($("\n   classes:   "));
      out->write(String(module->objectTagCount));
      out->write($("\n   constants: "));
      out->write(String(module->constantCount));
      out->write($("\n\n"));

      for(int i = 0; i < module->functionTagCount; i++)
      {
         FunctionTag* tag = module->functionTags[i];

         out->write($("   function "));
         out->write(String(i));
         out->write($(":\n      arity:    "));
         out->write(String(tag->arity));
         out->write($("\n      env size: "));
         out->write(String(tag->environmentSize));
         out->write($("\n      instructions:\n"));

         for(int j = 0; j < tag->instructionCount; j++)
         {
            out->write($("         "));
            switch(tag->instructions[j].opcode)
            {
            case NOOP:
               out->write($("noop            "));
               break;
            case RETURN:
               out->write($("return          "));
               break;
            case GOTO:
               out->write($("goto            "));
               break;
            case TEST_CONDITION:
               out->write($("test            "));
               break;
            case PUSH_EMPTY:
               out->write($("push empty      "));
               break;
            case POP:
               out->write($("pop             "));
               break;
            case LOAD:
               out->write($("load stack      "));
               break;
            case LOAD_ENVIRONMENT:
               out->write($("load env        "));
               break;
            case LOAD_CONSTANT:
               out->write($("load const      "));
               break;
            case STORE:
               out->write($("store           "));
               break;
            case STORE_POP:
               out->write($("store pop       "));
               break;
            case GET_MEMBER:
               out->write($("member          "));
               break;
            case APPLY:
               out->write($("apply           "));
               break;
            case APPLY_RETURN:
               out->write($("apply tail      "));
               break;
            case MAKE_FUNCTION:
               out->write($("make function   "));
               break;
            case MAKE_OBJECT:
               out->write($("make object     "));
               break;
            case MAKE_ARRAY:
               out->write($("make array      "));
               break;
            case ALLOCATE_MULTI:
               out->write($("multi alloc     "));
               break;
            case MAKE_FUNCTION_MULTI:
               out->write($("multi function  "));
               break;
            case MAKE_OBJECT_MULTI:
               out->write($("multi object    "));
               break;
            case MAKE_ARRAY_MULTI:
               out->write($("multi array     "));
               break;
            case STORE_MULTI:
               out->write($("multi store     "));
               break;
            case STORE_MULTI_POP:
               out->write($("multi pop       "));
               break;

            case IMMEDIATE_INTEGER:
               out->write($("im int          "));
               break;
            case IMMEDIATE_NEGATIVE:
               out->write($("im neg          "));
               break;
            case IMMEDIATE_BOOLEAN:
               out->write($("im bool         "));
               break;
            case IMMEDIATE_CHARACTER:
               out->write($("im char         "));
               break;
            case IMMEDIATE_ATOM:
               out->write($("im atom         "));
               break;
            case IMMEDIATE_INTEGER32:
               out->write($("im int32        "));
               break;
            case IMMEDIATE_INTEGER64:
               out->write($("im int64        "));
               break;
            case IMMEDIATE_REAL32:
               out->write($("im real32       "));
               break;
            case IMMEDIATE_REAL64:
               out->write($("im real64       "));
               break;
            case IMMEDIATE_CHARACTER32:
               out->write($("im char32       "));
               break;
            case IMMEDIATE_ATOM32:
               out->write($("im atom32       "));
               break;
            case IMMEDIATE_DATA1:
               out->write($("im data1        "));
               break;
            case IMMEDIATE_DATA2:
               out->write($("im data2        "));
               break;
            case IMMEDIATE_DATA4:
               out->write($("im data4        "));
               break;
            case IMMEDIATE_DATA8:
               out->write($("im data8        "));
               break;
            case IMMEDIATE_DATA16:
               out->write($("im data16       "));
               break;
            case IMMEDIATE_DATA32:
               out->write($("im data32       "));
               break;
            case IMMEDIATE_DATA64:
               out->write($("im data64       "));
               break;

            case NOT:
               out->write($("not             "));
               break;
            case NEGATE:
               out->write($("negate          "));
               break;
            case SIZE:
               out->write($("size            "));
               break;
            case AND:
               out->write($("and             "));
               break;
            case OR:
               out->write($("or              "));
               break;
            case ADD:
               out->write($("add             "));
               break;
            case SUBTRACT:
               out->write($("subtract        "));
               break;
            case MULTIPLY:
               out->write($("multiply        "));
               break;
            case DIVIDE:
               out->write($("divide          "));
               break;
            case MODULUS:
               out->write($("modulus        "));
               break;
            case EQUALS:
               out->write($("equals          "));
               break;
            case IS_LESS_THAN:
               out->write($("isLessThan      "));
               break;
            case IS_GREATER_THAN:
               out->write($("isGreaterThan   "));
               break;
            case GET_ELEMENT:
               out->write($("getElement      "));
               break;

            default:
               out->write($("INVALID         "));
               break;
            }
            out->write(String(tag->instructions[j].parameter, 8));

            if(tag->sourceLocations != null)
            {
               out->write($("   "));
               out->write(String(tag->sourceLocations[j].line + 1, 6));
               out->write($(":"));
               out->write(String(tag->sourceLocations[j].column + 1, 3));
            }
            out->write($("\n"));
         }
      }

      out->write($("\n"));

      for(int i = 0; i < module->objectTagCount; i++)
      {
         ObjectTag* tag = module->objectTags[i];

         out->write($("   class "));
         out->write(String(i));
         out->write($(":\n         name         offset\n"));

         for(int j = 0; j < tag->memberCount; j++)
         {
            out->write($("      "));
            String s = String(tag->members[j].nameId);
            out->write(' ', 10 - s.getLength());
            out->write(s);
            out->write($("      "));
            s = String(tag->members[j].offset);
            out->write(' ', 10 - s.getLength());
            out->write(s);
            out->write($("\n"));
         }
      }

      out->write($("\n   constants:\n"));

      for(int i = 0; i < module->constantCount; i++)
      {
         out->write($("      "));
         String s = String(i);
         out->write(' ', 5 - s.getLength());
         out->write(s);
         out->write($(": "));
         writeValue(out, module->constants[i]);
         out->write($("\n"));
      }

      out->write($("\n   extra data:"));
#ifdef PLATFORM_64BIT
      for(int i = 0; i < module->extraDataSize*2; i++)
#else
      for(int i = 0; i < module->extraDataSize; i++)
#endif
      {
         if(i % 8 == 0)
         {
            out->write($("\n      0x"));
            out->write(String((word)(module->extraData + i), sizeof(word)*2, 16));
            out->write($(":"));
         }
         out->write($(" "));
         out->write(String(((uint32*)module->extraData)[i], 8, 16));
      }
      out->write($("\n"));
   }
#endif //INCLUDE_WRITER_CODE
}

// ===================================================================

} //namespace vm
} //namespace evlan
