/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/09/01 (KV)
 *  + First Version
 */

// ===================================================================

#include "init.h"
#include "core.h"
#include "native.h"
#include "memory.h"
#include "operations.h"
#include "messages.h"
#include "loader.h"
#include "persistence.h"

namespace evlan {
namespace vm {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::vm::init
// TODO (KV): Review evlan::vm::init

namespace init
{
   void init(framework::SystemPtr system,
             String stateDirectory,
             String moduleDirectory,
             compiler::ErrorCollectorPtr errorCollector)
   {
      memory::init();
      operations::init();
      native::init();
      loader::init(moduleDirectory, errorCollector);
      messages::init();
      persistence::init(stateDirectory);
   }

   void restore(framework::SystemPtr system,
                String stateDirectory, String moduleDirectory,
                compiler::ErrorCollectorPtr errorCollector,
                String stateName, int frame)
   {
      //restore state
      loader::init(moduleDirectory, errorCollector);
      persistence::init(stateDirectory);
      persistence::restore(system, stateName, frame);
   }

   void shutdown()
   {
      persistence::shutdown();
      messages::shutdown();
      loader::shutdown();
      operations::shutdown();
      native::shutdown();
      memory::shutdown();
   }
}

// ===================================================================

} //namespace vm
} //namespace evlan
