/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/03/12 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef evlan_vm_loader_h__INCLUDED__
#define evlan_vm_loader_h__INCLUDED__

#include "core.h"
#include <evlan/compiler/ErrorCollector.h>
#include <evlan/compiler/Bytecode.h>

namespace evlan {
namespace vm {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document evlan::vm::loader
// TODO (KV): Test evlan::vm::loader

namespace loader
{
   void init(String moduleDirectory,
             compiler::ErrorCollectorPtr errorCollector);
   void shutdown();

   struct ModuleInfo
   {
      String name;
      bool fromFile;
      core::Module* module;
   };
   typedef SmartPointer<ModuleInfo> ModuleInfoPtr;

   /* Loads the module with the given name and places its root on the
    * top of the stack.  The directory path is searched to find the
    * module's implementation.  The loader searches each directory for
    * files with the same name except with various extensions appended
    * to them.  It first searches for platform-native shared libraries
    * (".dll" or ".so"), then Evlan bytecode files, then Evlan source
    * files, loading the first one that it finds.  (Note:  As of this
    * writing, native libraries and bytecode are not implemented; only
    * raw source code files are loaded.)
    *
    * If the module has already been loaded and no changes to the
    * source file are detected, loadModule() will simply return the
    * previous instance rather than reloading it.  If changes are
    * detected, on the other hand, loadModule() will unload the old
    * module (possibly turning it into a zombie) and reload it from
    * the new implementation file.
    *
    * relativeTo indicates a path to which the module name is
    * relative.  If the module name does not start with a '/', it is
    * interpreted relative to this, and may start with any number of
    * "../"'s.  If relativeTo is null, the name must start with '/'.
    * If relativeTo must start with a '/' if it is not null. */
   ModuleInfoPtr loadModule(String name, String relativeTo = null);
   //Loads a module directly from the given source or bytecode.
   ModuleInfoPtr loadModuleFromSource(String name, String source);
   ModuleInfoPtr loadModuleFromBytecode(String name, compiler::BytecodePtr bytecode);

   //Hook for the persistent state loader.
   void initModule(String name, bool fromFile, core::Module* module);

   //Get info about resident modules.
   Array<const ModuleInfoPtr> listResidentModules();
   ModuleInfoPtr getModuleInfo(core::Module* module);

   // exceptions -----------------------------------------------------
   FW_DECLARE_EXCEPTION(LoadFailed, Exception);
}

// ===================================================================

} //namespace vm
} //namespace evlan

#endif //included
