/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/03/18 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef evlan_vm_native_h__INCLUDED__
#define evlan_vm_native_h__INCLUDED__

#include "core.h"
#include <framework/util/Vector.h>
#include <framework/System.h>

namespace evlan {
namespace vm {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document evlan::vm::native
// TODO (KV): Test evlan::vm::native

namespace native
{
   void init();
   void shutdown();

   Array<const String> getList();
   Array<const int> getIdList();

   typedef ObjectPtr (*ProcessConstructor)(core::Process* process, framework::SystemPtr system);

   core::NativeFunctionTag* getFunction(int id);
   ProcessConstructor getProcess(int id);

   void makeSystemProcess(framework::SystemPtr system);

   class NativeIdentifier
   {
   public:
      NativeIdentifier(String identifier, uint32* variable);
      ~NativeIdentifier();

   private:
      NativeIdentifier* mNext;
      NativeIdentifier** mPrevious;

      String mIdentifier;
      uint32* mVariable;

      friend void init();
   };

   class NativeFunction
   {
   public:
      NativeFunction(String identifier, int arity,
                     void (*nativeApply)(),
                     Array<const String> parameterNames);
      ~NativeFunction();

   private:
      NativeFunction* mNext;
      NativeFunction** mPrevious;

      String mIdentifier;
      int mArity;
      void (*mNativeApply)();
      Array<const String> mParameterNames;

      friend void init();
   };

   class NativeProcess
   {
   public:
      NativeProcess(String identifier, ProcessConstructor make);
      ~NativeProcess();

   private:
      NativeProcess* mNext;
      NativeProcess** mPrevious;

      String mIdentifier;
      ProcessConstructor mMake;

      friend void init();
   };

   #define EVLAN_NATIVE_GET_PROCESS_INFO(NAME, PROCESS, TYPE)                    \
      TYPE NAME = PROCESS.process.process->nativeInfo.dynamicCast<TYPE>();       \
      if(NAME == null)                                                           \
         return core::Value::makeError(core::Value::TYPE_ERROR);

   #define EVLAN_NATIVE_PROCESS(NAME, CLASSNAME)                                   \
      ::framework::ObjectPtr CLASSNAME##_MAKE(::evlan::vm::core::Process* process, \
                                              framework::SystemPtr system)         \
         {return spnew CLASSNAME(process, system);}                                \
      ::evlan::vm::native::NativeProcess CLASSNAME##_TAG(                          \
         $(#NAME), CLASSNAME##_MAKE);

   #define EVLAN_NATIVE_IDENTIFIER(VARIABLE, NAME)                                 \
      uint32 VARIABLE;                                                             \
      ::evlan::vm::native::NativeIdentifier VARIABLE##__IMPL($(#NAME), &VARIABLE);

   #define EVLAN_NATIVE_FUNCTION_0(NAME)                                       \
      ::evlan::vm::core::Value NAME##_IMPL();                                  \
      void NAME##_WRAPPER()                                                    \
      {                                                                        \
         ::evlan::vm::core::Value* result = ::evlan::vm::core::gStackTop;      \
         *result = NAME##_IMPL();                                              \
         ::evlan::vm::core::gStackTop = result;                                \
      }                                                                        \
      ::evlan::vm::native::NativeFunction NAME($(#NAME), 0, NAME##_WRAPPER,    \
                                               null);                          \
      ::evlan::vm::core::Value NAME##_IMPL()

   #define EVLAN_NATIVE_FUNCTION_1(NAME, PARAM1)                               \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1);                                    \
      void NAME##_WRAPPER()                                                    \
      {                                                                        \
         ::evlan::vm::core::Value* result = ::evlan::vm::core::gStackTop + 1;  \
         *result =                                                             \
            NAME##_IMPL(::evlan::vm::core::gStackTop[0]);                      \
         ::evlan::vm::core::gStackTop = result;                                \
      }                                                                        \
      const ::framework::common::String NAME##_PARAMETERS[] = {$(#PARAM1)};    \
      ::evlan::vm::native::NativeFunction NAME($(#NAME), 1, NAME##_WRAPPER,    \
            $ARRAY(const ::framework::common::String, NAME##_PARAMETERS));     \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1)

   #define EVLAN_NATIVE_FUNCTION_2(NAME, PARAM1, PARAM2)                       \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1,                                     \
         ::evlan::vm::core::Value& PARAM2);                                    \
      void NAME##_WRAPPER()                                                    \
      {                                                                        \
         ::evlan::vm::core::Value* result = ::evlan::vm::core::gStackTop + 2;  \
         *result =                                                             \
            NAME##_IMPL(::evlan::vm::core::gStackTop[1],                       \
                        ::evlan::vm::core::gStackTop[0]);                      \
         ::evlan::vm::core::gStackTop = result;                                \
      }                                                                        \
      const ::framework::common::String NAME##_PARAMETERS[] =                  \
         {$(#PARAM1), $(#PARAM2)};                                             \
      ::evlan::vm::native::NativeFunction NAME($(#NAME), 2, NAME##_WRAPPER,    \
            $ARRAY(const ::framework::common::String, NAME##_PARAMETERS));     \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1,                                     \
         ::evlan::vm::core::Value& PARAM2)

   #define EVLAN_NATIVE_FUNCTION_3(NAME, PARAM1, PARAM2, PARAM3)               \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1,                                     \
         ::evlan::vm::core::Value& PARAM2,                                     \
         ::evlan::vm::core::Value& PARAM3);                                    \
      void NAME##_WRAPPER()                                                    \
      {                                                                        \
         ::evlan::vm::core::Value* result = ::evlan::vm::core::gStackTop + 3;  \
         *result =                                                             \
            NAME##_IMPL(::evlan::vm::core::gStackTop[2],                       \
                        ::evlan::vm::core::gStackTop[1],                       \
                        ::evlan::vm::core::gStackTop[0]);                      \
         ::evlan::vm::core::gStackTop = result;                                \
      }                                                                        \
      const ::framework::common::String NAME##_PARAMETERS[] =                  \
         {$(#PARAM1), $(#PARAM2), $(#PARAM3)};                                 \
      ::evlan::vm::native::NativeFunction NAME($(#NAME), 3, NAME##_WRAPPER,    \
            $ARRAY(const ::framework::common::String, NAME##_PARAMETERS));     \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1,                                     \
         ::evlan::vm::core::Value& PARAM2,                                     \
         ::evlan::vm::core::Value& PARAM3)

   #define EVLAN_NATIVE_FUNCTION_4(NAME, PARAM1, PARAM2, PARAM3, PARAM4)       \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1,                                     \
         ::evlan::vm::core::Value& PARAM2,                                     \
         ::evlan::vm::core::Value& PARAM3,                                     \
         ::evlan::vm::core::Value& PARAM4);                                    \
      void NAME##_WRAPPER()                                                    \
      {                                                                        \
         ::evlan::vm::core::Value* result = ::evlan::vm::core::gStackTop + 4;  \
         *result =                                                             \
            NAME##_IMPL(::evlan::vm::core::gStackTop[3],                       \
                        ::evlan::vm::core::gStackTop[2],                       \
                        ::evlan::vm::core::gStackTop[1],                       \
                        ::evlan::vm::core::gStackTop[0]);                      \
         ::evlan::vm::core::gStackTop = result;                                \
      }                                                                        \
      const ::framework::common::String NAME##_PARAMETERS[] =                  \
         {$(#PARAM1), $(#PARAM2), $(#PARAM3), $(#PARAM4)};                     \
      ::evlan::vm::native::NativeFunction NAME($(#NAME), 4, NAME##_WRAPPER,    \
            $ARRAY(const ::framework::common::String, NAME##_PARAMETERS));     \
      ::evlan::vm::core::Value NAME##_IMPL(                                    \
         ::evlan::vm::core::Value& PARAM1,                                     \
         ::evlan::vm::core::Value& PARAM2,                                     \
         ::evlan::vm::core::Value& PARAM3,                                     \
         ::evlan::vm::core::Value& PARAM4)
}

// ===================================================================

} //namespace vm
} //namespace evlan

#endif //included
