/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/04/01 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef evlan_vm_operations_h__INCLUDED__
#define evlan_vm_operations_h__INCLUDED__

#include "core.h"

namespace evlan {
namespace vm {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document evlan::vm::operations
// TODO (KV): Test evlan::vm::operations

namespace operations
{
   //must call this before using simulator
   void init();
   void init(Array<const String> identifiers);
   void shutdown();

   int getIdentifierId(String name);
   String getIdentifierName(int id);
   Array<const String> getIdentifierNames();

   /* Returns the index of the given member name in the global member
    * name table.  (If the name is not already in the table, it is
    * added.) */
   inline int getMemberNameId(String memberName) {return getIdentifierId(memberName);}
   inline String getMemberName(int id)           {return getIdentifierName(id);}

   /* Returns the index of the given atom in the global atom table.
    * (If the atom is not already in the table, it is added.) */
   inline int getAtomId(String atomName) {return getIdentifierId(atomName);}
   inline String getAtomName(int id)     {return getIdentifierName(id);}

   /* Looks up the given member within the object on the top of the
    * stack.  The object is popped and the given member is pushed in
    * its place. */
   void getMember(String memberName);
   void getMember(int id);
   int getMemberIndex(core::ObjectTag* tag, int id);

   /* These members have fixed id numbers in order to make some stuff
    * easier/faster. */
   enum BuiltinMember
   {
      BUILTIN_NOT,
      BUILTIN_NEGATE,
      BUILTIN_SIZE,
      BUILTIN_ARITY,

      BUILTIN_AND,
      BUILTIN_OR,
      BUILTIN_ADD,
      BUILTIN_SUBTRACT,
      BUILTIN_MULTIPLY,
      BUILTIN_DIVIDE,
      BUILTIN_MODULUS,
      BUILTIN_POWER,
      BUILTIN_EQUALS,
      BUILTIN_IS_LESS_THAN,
      BUILTIN_IS_GREATER_THAN,
      BUILTIN_GET_ELEMENT,

      BUILTIN_ALL    //used in catch descriptors
   };

   //These perform the given operations.
   void doNot();
   void doNegate();

   void doAnd();
   void doOr();
   void doAdd();
   void doSubtract();
   void doMultiply();
   void doDivide();
   void doModulus();
   void doPower();
   void doEquals();
   void doIsLessThan();
   void doIsGreaterThan();
}

// ===================================================================

} //namespace vm
} //namespace evlan

#endif //included
