/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/10/13 (KV)
 *  + First Version
 */

// ===================================================================

#include "shortcuts.h"
#include "arrays.h"
#include "native.h"

namespace evlan {
namespace vm {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement evlan::vm::shortcuts
// TODO (KV): Review evlan::vm::shortcuts

namespace shortcuts
{
   EVLAN_NATIVE_IDENTIFIER(gErrorId, error);

   Value simThrow()
   {
      Value* elements = memory::allocateValues(3);
      elements[0] = Value::makeAtom(gErrorId);
      elements[1] = Value::makeScalar(1);
      elements[2] = *gStackTop;
      simPop();
      return Value::makeTask(Value::THROW, 3, elements);
   }

   Value simThrow(int id)
   {
      Value* elements = memory::allocateValues(5);
      elements[0] = Value::makeAtom(id);
      elements[1] = Value::makeScalar(0);
      elements[2] = Value::makeAtom(gErrorId);
      elements[3] = Value::makeScalar(1);
      elements[4] = *gStackTop;
      simPop();
      return Value::makeTask(Value::THROW, 5, elements);
   }

   Value simThrow(int id, int generalId, Value& param)
   {
      Value* elements = memory::allocateValues(8);
      elements[0] = Value::makeAtom(id);
      elements[1] = Value::makeScalar(0);
      elements[2] = Value::makeAtom(generalId);
      elements[3] = Value::makeScalar(1);
      elements[4] = param;
      elements[5] = Value::makeAtom(gErrorId);
      elements[6] = Value::makeScalar(1);
      elements[7] = *gStackTop;
      simPop();
      return Value::makeTask(Value::THROW, 8, elements);
   }

   Value simThrow(String description)
   {
      arrays::makeString(description);
      return simThrow();
   }

   Value simThrow(int id, String description)
   {
      arrays::makeString(description);
      return simThrow(id);
   }

   Value simThrow(int id, int paramId, Value& param, String description)
   {
      arrays::makeString(description);
      return simThrow(id, paramId, param);
   }
}

// ===================================================================

} //namespace vm
} //namespace evlan
