/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/10/13 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef evlan_vm_shortcuts_h__INCLUDED__
#define evlan_vm_shortcuts_h__INCLUDED__

#include "core.h"
#include "memory.h"
#include "simulator.h"
#include "messages.h"

namespace evlan {
namespace vm {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document evlan::vm::shortcuts
// TODO (KV): Test evlan::vm::shortcuts

namespace shortcuts
{
   using core::gStackTop;
   using core::Value;

   inline void simPush(core::Value value)
   {
      --gStackTop;
      *gStackTop = value;
   }

   inline void simPop(int count = 1) {gStackTop += count;}

   inline void simApply(int arity) {simulator::apply(arity);}
   inline void simQueue() {messages::queueTask();}

   inline Value simReturn() {return Value::makeTask(Value::RETURN);}

   inline Value simReturn(Value value)
   {
      simPush(value);
      Value* parameter = memory::allocateValues(1);
      *parameter = *gStackTop;
      simPop();
      return Value::makeTask(Value::RETURN, 1, parameter);
   }

   //description on stack
   Value simThrow();
   Value simThrow(int id);
   Value simThrow(int id, int paramId, Value& param);
   //description as parameter
   Value simThrow(String description);
   Value simThrow(int id, String description);
   Value simThrow(int id, int paramId, Value& param, String description);

   inline Value simTypeError()
   {
      return Value::makeError(::evlan::vm::core::Value::TYPE_ERROR);
   }

   inline Value simTypeError(Value input1)
   {
      if(input1.isError())
         return input1;
      else
         return Value::makeError(::evlan::vm::core::Value::TYPE_ERROR);
   }

   inline Value simTypeError(Value input1, Value input2)
   {
      if(input1.isError())
         return input1;
      else if(input2.isError())
         return input2;
      else
         return Value::makeError(::evlan::vm::core::Value::TYPE_ERROR);
   }

   inline Value simTypeError(Value input1, Value input2, Value input3)
   {
      if(input1.isError())
         return input1;
      else if(input2.isError())
         return input2;
      else if(input3.isError())
         return input3;
      else
         return Value::makeError(::evlan::vm::core::Value::TYPE_ERROR);
   }

   inline Value simTypeError(Value input1, Value input2, Value input3, Value input4)
   {
      if(input1.isError())
         return input1;
      else if(input2.isError())
         return input2;
      else if(input3.isError())
         return input3;
      else if(input4.isError())
         return input4;
      else
         return Value::makeError(::evlan::vm::core::Value::TYPE_ERROR);
   }
}

// ===================================================================

} //namespace vm
} //namespace evlan

#endif //included
