/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/03/01 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef evlan_vm_simulator_h__INCLUDED__
#define evlan_vm_simulator_h__INCLUDED__

#include "core.h"

namespace evlan {
namespace vm {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document evlan::vm::simulator
// TODO (KV): Test evlan::vm::simulator

namespace simulator
{
   /* Applies a funciton to a set of parameters.  The function should
    * be pushed on the stack first, with parameters pushed in order
    * after it.  Thus, the last parameter should be at the top of the
    * stack when apply() is called.  When apply() returns, the
    * function and all parameters will have been popped, and the
    * result of the function will have been pushed on the top of the
    * stack.  If the function does not match the arity given, or if
    * the value in its location in the stack is not a function, it and
    * the parameters are popped anyway, but a runtime error is pushed
    * in their place. */
   void apply(int arity);

   /* This sets a function which will be called on a regular basis to
    * indicate progress to the user. */
   void setTickCallback(void (*tick)());

   void enableProfiling(bool enable);
}

// ===================================================================

} //namespace vm
} //namespace evlan

#endif //included
