/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/03/01 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef evlan_vm_translator_h__INCLUDED__
#define evlan_vm_translator_h__INCLUDED__

#include <evlan/compiler/Bytecode.h>
#include "core.h"

namespace evlan {
namespace vm {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document evlan::vm::translator
// TODO (KV): Test evlan::vm::translator

namespace translator
{
   core::Module* translateBytecode(compiler::BytecodePtr bytecode);

   // exceptions -----------------------------------------------------
   class InvalidBytecode: extends Exception
   {
   public:
      inline InvalidBytecode(String error, int line, int column, ExceptionPtr cause = null)
         : Exception(cause), mError(error), mLine(line), mColumn(column) {}
      virtual String getDescription();
      virtual ~InvalidBytecode() {}
      inline String getError() {return mError;}
      inline int getLine()   {return mLine;}
      inline int getColumn() {return mColumn;}

   private:
      String mError;
      int mLine;
      int mColumn;

      InvalidBytecode(const InvalidBytecode&);
      InvalidBytecode& operator=(const InvalidBytecode&);
   };
   typedef SmartPointer<InvalidBytecode> InvalidBytecodePtr;
}

// ===================================================================

} //namespace vm
} //namespace evlan

#endif //included
