/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/04/10 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef framework_Clock_h__INCLUDED__
#define framework_Clock_h__INCLUDED__

#include <framework/messages/Message.h>

namespace framework {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document framework::Clock
// TODO (KV): Test framework::Clock

interface Clock;
typedef SmartPointer<Clock> ClockPtr;

interface Clock: implements Object
{
   virtual ~Clock();

   /* getTime() returns the current posted time, which is updated
    * manually by the message queue.  If you use notifyAtTime() below,
    * when the message is later sent, getTime() will return exactly
    * the time you requested. */
   virtual double getTime() = 0;
   /* getRealTime() gets the real current time from the OS when you
    * call it. */
   virtual double getRealTime() = 0;
   /* The values returned by getTime() and getRealTime() are measured
    * in seconds relative to the beginning of the year 2000. */

   virtual void notifyAtTime(double time, messages::MessagePtr message) = 0;
};

// ===================================================================

} //namespace framework

#endif //included
