/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#ifndef framework_FrontEnd_h__INCLUDED
#define framework_FrontEnd_h__INCLUDED

#include "System.h"

namespace framework {
using namespace ::framework::common;

// class definition ==================================================
class FrontEnd;

typedef SmartPointer<FrontEnd> FrontEndPtr;

/* interface FrontEnd
 *
 * main() is just too C for me, so I wrote this to hide it.
 *
 * To create an executable, simply implement this interface with a class,
 * and then declare an executable using Kake which uses your implementation
 * as its main module.  Kake will generate a main function for you which
 * calls make[classname]() to produce the front end and then calls some
 * code which parses command-line argumetns and such.  Finally, it will
 * call your class's run(), which is where you write your program.
 */
interface FrontEnd: implements Object
{
   virtual ~FrontEnd();

   struct SwitchDescriptor
   {
      int identifier;     //integer identifier for the switch
      String name;        //long (two-dash) switch name, or null for none
      char shorthand;     //shorthand (one-dash) switch name, or '\0' for none
      int numArguments;   //number of additional arguments this switch takes
                          // (usually zero)
   };

   struct Switch
   {
      int identifier;
      Array<const String> arguments;
   };

   struct ProgramInfo
   {
      String name;
      Array<const SwitchDescriptor> switchSyntax;
      String helpMessage;
   };

   struct StartupInfo
   {
      Array<const Switch> switches;
      Array<const String> arguments;
      SystemPtr system;
   };

   virtual ProgramInfo getProgramInfo() = 0;
   virtual void run(StartupInfo startupInfo) = 0;

   class UsageError: extends FatalError
   {
   public:
      inline UsageError(String description)
         : FatalError(description) {}
      virtual ~UsageError();
   };
   typedef SmartPointer<UsageError> UsageErrorPtr;
};

//main sucks!
int internalMain(FrontEndPtr (*makeFrontEnd)(), int numParams, char* argv[]);
#define KAKE_WRITE_MAIN(SCOPE, CLASSNAME)         \
   int main(int argc, char* argv[])               \
   {                                              \
      return ::framework::internalMain(           \
         SCOPE::make##CLASSNAME, argc, argv);     \
   }

} //namespace framework

#endif //included
