/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#ifndef framework_common_Common_h__INCLUDED
#define framework_common_Common_h__INCLUDED

#include <kake/Platform.h>

#ifdef PLATFORM_WIN32
/* By default, MinGW #includes <windows.h> to get thread safety
 * functions used inline in STL implementations.  Unfortunately, this
 * pollutes the namespace with many crappy Win32 definitions that we
 * most definately do NOT want.  Fortunately, in recent versions of
 * MinGW they have added this #define which will prevent <windows.h>
 * from being included, possibly at the cost of a small amount of
 * function call overhead (non-inline versions are used). */
#define __GTHREAD_HIDE_WIN32API
#endif

namespace framework {
namespace common {

// common stuff ======================================================
//These are all the standard typedefs and such.
typedef unsigned int       uint;

#ifdef PLATFORM_32BIT
typedef   signed char       int8;
typedef   signed short      int16;
typedef   signed int        int32;
typedef   signed long long  int64;
typedef unsigned char      uint8;
typedef unsigned short     uint16;
typedef unsigned int       uint32;
typedef unsigned long long uint64;

typedef uint32 word;
#else
#error "Need typedefs for your CPU word size."
#endif

typedef char   char8;
typedef uint16 char16;
typedef uint32 char32;

typedef uint8 byte;

//Yay NullType!
class NullType
{
public:
   template <typename Type>
   inline operator Type*() const {return 0;}
};
static const NullType gNull = NullType();

//Define pseudo-keywords.
#define null (::framework::common::gNull)

#define interface struct
#define implementation struct
#define implements virtual public
#define extends public

#define restrict __restrict__

//Loops through all elements in an STL container.
#define forall(ITERATOR, CONTAINER)                                \
   for(ITERATOR = (CONTAINER).begin();                             \
       ITERATOR != (CONTAINER).end(); ++ITERATOR)

//I prefer NDEBUG to be the default.
#if !defined(DEBUG) && !defined(NDEBUG)
#define NDEBUG
#endif

} //namespace common
} //namespace framework

#endif //included
