/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/06/13 (KV)
 *  + First Version
 */

// ===================================================================

#include "Debug.h"
#include "Exception.h"
#include <unistd.h>

namespace framework {
namespace common {

// ===================================================================
// AUTHOR (KV): Created module.

namespace Debug
{
   void defaultPrintMessage(WarningLevel warningLevel, String message)
   {
      switch(warningLevel)
      {
      case MESSAGE:
         write(1, "MESSAGE: ", 9);
         break;
      case WARNING:
         write(1, "WARNING: ", 9);
         break;
      case ERROR:
         write(1, "ERROR:   ", 9);
         break;
      }

      while(true)
      {
         int pos = message.findFirstOf('\n');
         if(pos == -1)
         {
            if(message != null)
               write(1, &message.getBuffer()[0], message.getLength());
            write(1, "\n", 1);
            break;
         }
         else
         {
            String line = message.getInterval(0, pos + 1);
            message = message.getInterval(pos + 1);

            if(line != null)
               write(1, &line.getBuffer()[0], line.getLength());
            write(1, "         ", 9);
         }
      }
   }

   void (*gPrintDebugMessage)(WarningLevel warningLevel, String message) = defaultPrintMessage;

   LogEntry::~LogEntry()
   {
      int totalSize = 0;
      StringList* list = mMessage;
      while(list != null)
      {
         totalSize += list->mString.getLength();
         list = list->mNext;
      }
      Array<char> message(totalSize);
      list = mMessage;
      while(list != null)
      {
         if(list->mString.getLength() > 0)
         {
            totalSize -= list->mString.getLength();
            list->mString.copyToBuffer(&message[totalSize], 0, list->mString.getLength());
         }
         StringList* prev = list;
         list = list->mNext;
         delete prev;
      }

      gPrintDebugMessage(mLevel, String(message.toConst()));
   }

   Message& Message::operator<<(String message)
   {
      StringList* node = new StringList;
      node->mString = message;
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }

   Message& Message::operator<<(const char* message)
   {
      StringList* node = new StringList;
      node->mString = String(message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }

   Message& Message::operator<<(int message)
   {
      StringList* node = new StringList;
      node->mString = String(message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(uint message)
   {
      StringList* node = new StringList;
      node->mString = String(message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(long message)
   {
      StringList* node = new StringList;
      node->mString = String((int)message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(unsigned long message)
   {
      StringList* node = new StringList;
      node->mString = String((uint)message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(short message)
   {
      StringList* node = new StringList;
      node->mString = String((int)message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(unsigned short message)
   {
      StringList* node = new StringList;
      node->mString = String((uint)message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(double message)
   {
      StringList* node = new StringList;
      node->mString = String(message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(char message)
   {
      StringList* node = new StringList;
      node->mString = String(message);
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(bool message)
   {
      StringList* node = new StringList;
      node->mString = message?$("true"):$("false");
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }
   Message& Message::operator<<(const void* ptr)
   {
      operator<<($("0x"));
      StringList* node = new StringList;
#ifdef PLATFORM_64BIT
      node->mString = String((uint64)ptr, 8, 16);
#else
      node->mString = String((uint32)ptr, 8, 16);
#endif
      node->mNext = mLogEntry->mMessage;
      mLogEntry->mMessage = node;
      return *this;
   }

   Message log(WarningLevel level)
   {
      return Message(spnew LogEntry(level));
   }

   void setPrintFunction(void (*printDebugMessage)(WarningLevel warningLevel, String message))
   {
      if(printDebugMessage == null)
         gPrintDebugMessage = defaultPrintMessage;
      else
         gPrintDebugMessage = printDebugMessage;
   }


   void assertionFailed(String message)
   {
      FW_THROW(spnew FatalError(message));
   }
}

// ===================================================================

} //namespace common
} //namespace framework
