/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/06/13 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef framework_common_Debug_h__INCLUDED__
#define framework_common_Debug_h__INCLUDED__

#include "String.h"
#include "Object.h"

namespace framework {
namespace common {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.

/* This contains stuff useful for debugging, such as an easy way to
 * print messages to the console. */
namespace Debug
{
   enum WarningLevel
   {
      MESSAGE,
      WARNING,
      ERROR
   };

   struct StringList
   {
      String mString;
      StringList* mNext;
   };

   class LogEntry
   {
   public:
      inline LogEntry(WarningLevel level): mLevel(level), mMessage(null) {}
      ~LogEntry();

      WarningLevel mLevel;
      StringList* mMessage;
   };
   typedef SmartPointer<LogEntry> LogEntryPtr;

   class Message
   {
   public:
      inline Message(LogEntryPtr logEntry): mLogEntry(logEntry) {};

      Message& operator<<(String message);
      Message& operator<<(const char* message);
      Message& operator<<(int message);
      Message& operator<<(uint message);
      Message& operator<<(long message);
      Message& operator<<(unsigned long message);
      Message& operator<<(short message);
      Message& operator<<(unsigned short message);
      Message& operator<<(double message);
      Message& operator<<(char message);
      Message& operator<<(bool message);
      Message& operator<<(const void* ptr);

   private:
      LogEntryPtr mLogEntry;
   };

   /* Call this to write a message to the debug log.  Syntax should
    * be something like:
    *    Debug::log(Debug::ERROR) << "Problem exists between\n"
    *                             << "keyboard and chair.";
    * The output (whether to the console or a file) will look like:
    *    ERROR:   Problem exists between
    *             user and chair.
    * (see how it's indented for you?) */
   Message log(WarningLevel level = MESSAGE);

   void setPrintFunction(void (*printDebugMessage)(WarningLevel warningLevel, String message));
   void defaultPrintMessage(WarningLevel warningLevel, String message);

   void assertionFailed(String message);

#ifdef DEBUG
#define FW_ASSERT(CONDITION, MESSAGE) if(CONDITION) ::framework::common::assertionFailed($(MESSAGE))
#else
#define FW_ASSERT(CONDITION, MESSAGE)
#endif

};

// ===================================================================

} //namespace common
} //namespace framework

#endif //included
