/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#include "Exception.h"
#include <typeinfo>
#include <cstdio>

/* I wanted to #include <cerrno>, but MinGW apparently #defines errno
 * as a fucking macro.  Hello, C++ standard violation! */
#include <errno.h>

#ifdef PLATFORM_WIN32
#include <windows.h>
#endif

namespace framework {
namespace common {

// class members =====================================================
Exception::Exception(ExceptionPtr cause)
   : mCause(cause)
{
#ifdef DEBUG
   //GCC doesn't seem to like non-constant arguments to
   // __builtin_return_address...  :(
   for(int i = 0; i < STACK_TRACE_SIZE; i++)
      mStackTrace[i] = null;
   if(__builtin_frame_address(0) != 0) {
   mStackTrace[0] = __builtin_return_address(0);
   if(__builtin_frame_address(1) != 0) {
   mStackTrace[1] = __builtin_return_address(1);
   if(__builtin_frame_address(2) != 0) {
   mStackTrace[2] = __builtin_return_address(2);
   if(__builtin_frame_address(3) != 0) {
   mStackTrace[3] = __builtin_return_address(3);
   if(__builtin_frame_address(4) != 0) {
   mStackTrace[4] = __builtin_return_address(4);
   if(__builtin_frame_address(5) != 0) {
   mStackTrace[5] = __builtin_return_address(5);
   if(__builtin_frame_address(6) != 0) {
   mStackTrace[6] = __builtin_return_address(6);
   if(__builtin_frame_address(7) != 0) {
   mStackTrace[7] = __builtin_return_address(7);
   if(__builtin_frame_address(8) != 0) {
   mStackTrace[8] = __builtin_return_address(8);
   if(__builtin_frame_address(9) != 0) {
   mStackTrace[9] = __builtin_return_address(9);
   if(__builtin_frame_address(10) != 0) {
   mStackTrace[10] = __builtin_return_address(10);
   if(__builtin_frame_address(11) != 0) {
   mStackTrace[11] = __builtin_return_address(11);
   if(__builtin_frame_address(12) != 0) {
   mStackTrace[12] = __builtin_return_address(12);
   if(__builtin_frame_address(13) != 0) {
   mStackTrace[13] = __builtin_return_address(13);
   if(__builtin_frame_address(14) != 0) {
   mStackTrace[14] = __builtin_return_address(14);
   if(__builtin_frame_address(15) != 0) {
   mStackTrace[15] = __builtin_return_address(15);
   }}}}}}}}}}}}}}}}
#endif
}

Exception::~Exception()
{
}

String Exception::getDescription()
{
   //Some loser threw an exception with no description.  Grr.
   if(typeid(this) == typeid(Exception))
      return $("Generic exception.");
   else
      return $("Non-descript exception of type ") +
             String(typeid(this).name()) + $(".");
}

String Exception::mExecutablePath;

String Exception::getStackTrace()
{
   return getStackTrace(mStackTrace, STACK_TRACE_SIZE);
}

String Exception::getStackTrace(void** stack, int size)
{
#ifdef DEBUG
   if(mExecutablePath == null)
      return $("No stack trace possible as executable location is unknown.\n");

   String command = $("addr2line -e \"") + mExecutablePath + $("\"");

   for(int i = 0; i < size; i++)
   {
      if(stack[i] == null)
         break;
      command += $(" 0x") + String((word)stack[i], sizeof(word)*2, 16);
   }

   FW_MAKE_CSTRING(commandCString, command);
#ifdef PLATFORM_WIN32
   FILE* input = popen(commandCString, "rt");
#else
   FILE* input = popen(commandCString, "r");
#endif

   char buffer[512];
   int pos = 0;
   int count;
   String result;
   while((count = fread(buffer, 1, 512 - pos, input)) > 0)
   {
      pos += count;
      if(pos == 512)
      {
         result += String(buffer, 512);
         pos = 0;
      }
   }
   result += String(buffer, pos);
   fclose(input);

   return result;

#else
   return $("Optimized build - No stack traces available.\n"
            "Compile with -g -DDEBUG to enable stack traces.\n");
#endif
}

FatalError::FatalError(String description, ExceptionPtr cause)
   : Exception(cause), mDescription(description)
{
   //place breakpoint here!
}

FatalError::~FatalError()
{}

ErrnoError::ErrnoError(String functionName, ExceptionPtr cause)
   : Exception(cause), mFunctionName(functionName)
{
   mErrorNumber = errno;
}
ErrnoError::~ErrnoError()
{}
String ErrnoError::getDescription()
{
   String description = $("C library error: ");
   if(mFunctionName != null)
      description += mFunctionName + $(": ");
   description += strerror(mErrorNumber);
   return description;
}

#ifdef PLATFORM_WIN32
Win32Error::Win32Error(String functionName, ExceptionPtr cause)
   : Exception(cause), mFunctionName(functionName)
{
   mErrorNumber = GetLastError();
}
Win32Error::~Win32Error()
{}
String Win32Error::getDescription()
{
   String description = $("Win32 error: ");
   if(mFunctionName != null)
      description += mFunctionName + $(": ");

   /* Is Win32 convoluted?  You be the judge.  (Luckily for me, the following
    * code was listed in the MSDN documentation for FormatMessage.) */
   LPVOID lpMsgBuf = null;
   DWORD result = FormatMessage(
      FORMAT_MESSAGE_ALLOCATE_BUFFER |
      FORMAT_MESSAGE_FROM_SYSTEM |
      FORMAT_MESSAGE_IGNORE_INSERTS,
      NULL,
      mErrorNumber,
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), //Default language
      (LPTSTR) &lpMsgBuf,  //oh my god this is ugly
      0,
      NULL
   );

   if(result == 0)
      description += $("(could not get error description)");
   else
      description += (char*)lpMsgBuf;
   if(lpMsgBuf != null)
      LocalFree(lpMsgBuf);

   return description;
}
#endif

FW_DECLARE_EXCEPTION(NullPointerException, Exception);
FW_DEFINE_EXCEPTION(NullPointerException,
   $("Null pointer exception."));

void ThrowNullPointerException()
{
   FW_THROW(spnew NullPointerException);
}

class ArrayOutOfBounds: extends Exception
{
   int mSize;
   int mIndex;

public:
   inline ArrayOutOfBounds(int size, int index)
      : mSize(size), mIndex(index) {}
   virtual String getDescription();
   virtual ~ArrayOutOfBounds() {}
};

String ArrayOutOfBounds::getDescription()
{
   return $("Tried to access element number ") + String(mIndex) +
          $(" of an array of size ") + String(mSize) + $(".");
}

void ThrowOutOfBoundsException(int size, int index)
{
   FW_THROW(spnew ArrayOutOfBounds(size, index));
}

} //namespace common
} //namespace framework
