/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#include "MemoryManager.h"
#include <cstdlib>

#ifdef DEBUG
#include <fstream>
#endif

namespace framework {
namespace common {

namespace
{
#ifdef DEBUG
   const int STACK_SIZE = 14;
   void* gBadAllocStack[STACK_SIZE] = {0};
#endif
}

// class member functions ============================================
void*
MemoryManager::allocate(int size)
{
#ifdef DEBUG_OVERRUNS
   byte* target = (byte*)std::malloc(size + sizeof(ObjectInfo) + 128);
   ((uint*)target)[0] = size;
   for(int i = 1; i < 16; i++)
      ((uint*)target)[i] = 0xfeebdaed + i;
   ObjectInfo* result = (ObjectInfo*)(target + 64);

   target += 64 + sizeof(ObjectInfo) + size;
   for(int i = 0; i < 16; i++)
      ((uint*)target)[i] = 0xfeebdaed + i - 16;
#else
   ObjectInfo* result =
      reinterpret_cast<ObjectInfo*>(std::malloc(size + sizeof(ObjectInfo)));
#endif

   if(result == null)
   {
#ifdef DEBUG
      if(__builtin_frame_address(0) != 0) {
      gBadAllocStack[0] = __builtin_return_address(0);
      if(__builtin_frame_address(1) != 0) {
      gBadAllocStack[1] = __builtin_return_address(1);
      if(__builtin_frame_address(2) != 0) {
      gBadAllocStack[2] = __builtin_return_address(2);
      if(__builtin_frame_address(3) != 0) {
      gBadAllocStack[3] = __builtin_return_address(3);
      if(__builtin_frame_address(4) != 0) {
      gBadAllocStack[4] = __builtin_return_address(4);
      if(__builtin_frame_address(5) != 0) {
      gBadAllocStack[5] = __builtin_return_address(5);
      if(__builtin_frame_address(6) != 0) {
      gBadAllocStack[6] = __builtin_return_address(6);
      if(__builtin_frame_address(7) != 0) {
      gBadAllocStack[7] = __builtin_return_address(7);
      if(__builtin_frame_address(8) != 0) {
      gBadAllocStack[8] = __builtin_return_address(8);
      if(__builtin_frame_address(9) != 0) {
      gBadAllocStack[9] = __builtin_return_address(9);
      if(__builtin_frame_address(10) != 0) {
      gBadAllocStack[10] = __builtin_return_address(10);
      if(__builtin_frame_address(11) != 0) {
      gBadAllocStack[11] = __builtin_return_address(11);
      if(__builtin_frame_address(12) != 0) {
      gBadAllocStack[12] = __builtin_return_address(12);
      if(__builtin_frame_address(13) != 0) {
      gBadAllocStack[13] = __builtin_return_address(13);
      }}}}}}}}}}}}}}
#endif

      throw std::bad_alloc();
   }

   result->mRefCount = 1;
   result->mDestructor = null;

   return result + 1;
}

void
MemoryManager::free(void* ptr)
{
   //funny story:  I used to name member functions starting with a capital
   // letter, so this function was called "Free()".  I later renamed it to
   // "free", and spent a few hours trying fo figure out why my program was
   // going into infinite loops at seemingly random moments.  Then I added
   // global namespace qualifiers, as you can see below.  (OK, then much,
   // much later I changed the include to <cstdlib>.  Blah blah.)

   if(ptr != null)
   {
      ObjectInfo* data = reinterpret_cast<ObjectInfo*>(ptr) - 1;

#ifdef DEBUG_OVERRUNS
      byte* target = (byte*)data - 64;
      for(int i = 1; i < 16; i++)
      {
         if(((uint*)target)[i] != 0xfeebdaed + i)
            *((int*)0) = 0;   //crash!
      }

      int size = ((int*)target)[0];

      byte* target2 = target + 64 + sizeof(ObjectInfo) + size;
      for(int i = 0; i < 16; i++)
      {
         if(((uint*)target2)[i] != 0xfeebdaed + i - 16)
            *((int*)0) = 0;   //crash!
      }

      std::free(target);
#else
      std::free(data);
#endif
   }
}

void** MemoryManager::getBadAllocStackTrace(int* size)
{
#ifdef DEBUG
   *size = STACK_SIZE;
   return gBadAllocStack;
#else
   *size = 0;
   return null;
#endif
}

} //namespace common
} //namespace framework

void* operator new(size_t sz) throw(std::bad_alloc)
   { return framework::common::MemoryManager::allocate(sz); }
void operator delete(void* ptr) throw()
   { framework::common::MemoryManager::free(ptr); }

void* operator new[](size_t sz) throw(std::bad_alloc)
   { return framework::common::MemoryManager::allocate(sz); }
void operator delete[](void* ptr) throw()
   { framework::common::MemoryManager::free(ptr); }
