/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#ifndef framework_common_MemoryManager_h__INCLUDED
#define framework_common_MemoryManager_h__INCLUDED

#include "Common.h"
#include <new>

namespace framework {
namespace common {

// class definition ==================================================
/* class MemoryManager
 *
 * All memory allocation goes through here.  However, you do not need to
 * know about this class.  Just use new and delete. */
class MemoryManager
{
public:
   //Allocates [size] bytes of memory and returns a pointer to them.
   static void* allocate(int size);
   //Frees a block of memory allocated with allocate().
   static void free(void* ptr);

   //In case of bad_alloc, use this to get a stack trace.
   static void** getBadAllocStackTrace(int* size);

   //appended to the front of every allocation, used by smart pointers
   struct ObjectInfo
   {
      int mRefCount;
      void (*mDestructor)(void* object);
   };

private:
   //disable constructors
   MemoryManager();
   MemoryManager(const MemoryManager& other);
};

} //namespace common
} //namespace framework

#endif //included
