/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/04/11 (KV)
 *  + First Version
 */

// ===================================================================

#include "MessageImpl.h"
#include <queue>

namespace framework {
namespace internal {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement framework::internal::MessageQueue
// TODO (KV): Review framework::internal::MessageQueue

implementation MessageImpl;
typedef SmartPointer<MessageImpl> MessageImplPtr;

namespace  //anonymous (file-scope) namespace
{
   std::queue<MessageImplPtr> gMessageQueue;
}

implementation MessageImpl: implements messages::Message
{
   MessageImpl(messages::MessageReceiver* receiver, int id, ObjectPtr data);
   ~MessageImpl();

   // Message --------------------------------------------------------
   bool send();
   void disable();
   bool isDisabled();
   int getIdentifier();
   ObjectPtr getData();

   // internal -------------------------------------------------------
   SmartPointer<messages::MessageReceiver*> mReceiver;
   int mId;
   ObjectPtr mData;
};

MessageImpl::MessageImpl(messages::MessageReceiver* receiver, int id, ObjectPtr data)
   : mReceiver(receiver->mWeakLink),
     mId(id),
     mData(data)
{
}

MessageImpl::~MessageImpl()
{
}

// Message -----------------------------------------------------------
bool MessageImpl::send()
{
   if(mReceiver == null)
      return false;
   else if(*mReceiver == null)
   {
      disable();
      return false;
   }
   else
   {
      gMessageQueue.push(spthis);
      return true;
   }
}

void MessageImpl::disable()
{
   mReceiver = null;
   mData = null;
}

bool MessageImpl::isDisabled()
{
   if(mReceiver == null)
      return true;
   else if(*mReceiver == null)
   {
      disable();
      return true;
   }
   else
      return false;
}

int MessageImpl::getIdentifier()
{
   return mId;
}

ObjectPtr MessageImpl::getData()
{
   return mData;
}

// ===================================================================
int pumpMessageQueue(int messageCount)
{
   int dispatchCount = 0;
   if(messageCount < 0)
      messageCount = 0x7FFFFFFF;  //really big number

   while(!gMessageQueue.empty() && dispatchCount < messageCount)
   {
      MessageImplPtr message = gMessageQueue.front();
      gMessageQueue.pop();

      if(message->mReceiver == null)
         {}
      else if(*message->mReceiver == null)
         message->disable();
      else
      {
         (*message->mReceiver)->receive(message);
         ++dispatchCount;
      }
   }

   return dispatchCount;
}

// factory function ==================================================
messages::MessagePtr makeMessageImpl(messages::MessageReceiver* receiver,
                                     int id, ObjectPtr data)
{
   return spnew MessageImpl(receiver, id, data);
}

// ===================================================================

} //namespace internal
} //namespace framework
