/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/05/21 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef framework_messages_MessageList_h__INCLUDED__
#define framework_messages_MessageList_h__INCLUDED__

#include <framework/common/all.h>
#include "Message.h"

namespace framework {
namespace messages {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.

interface MessageList;
typedef SmartPointer<MessageList> MessageListPtr;

/* interface MessageList
 *
 * This is a helper class which stores a set of messages and allows
 * you to send all of them at once.  It automatically takes care of
 * removing disabled messages.  This is useful when you want to allow
 * multiple clients to wait on the same event (which is usually the
 * case).  This interface is implemented by MessageListImpl. */
interface MessageList: implements Object
{
   virtual ~MessageList();

   //Adds the given message to the set.
   virtual void add(MessagePtr message) = 0;
   //Removes all messages from the set.
   virtual void clear() = 0;

   /* Sends all of the messages in the set.  Those which are found to
    * be disabled (i.e. send() returns false) are removed.  The number
    * of messages sent is returned. */
   virtual int send() = 0;
   /* Passes through all messages and checks if they are disabled.
    * Those that are are removed from the set.  The number of messages
    * remaining in the set is returned. */
   virtual int clean() = 0;
};

// ===================================================================

} //namespace messages
} //namespace framework

#endif //included
