/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/05/21 (KV)
 *  + First Version
 */

// ===================================================================

#include "MessageListImpl.h"
#include <framework/util/Vector.h>

namespace framework {
namespace messages {

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Implement framework::messages::MessageListImpl
// TODO (KV): Review framework::messages::MessageListImpl

implementation MessageListImpl: implements MessageList
{
   MessageListImpl();
   ~MessageListImpl();

   // MessageList ----------------------------------------------------
   void add(MessagePtr message);
   void clear();
   int send();
   int clean();

   // internal -------------------------------------------------------
   util::Vector<MessagePtr> mMessages;
};

MessageListImpl::MessageListImpl()
{
}

MessageListImpl::~MessageListImpl()
{
}

// MessageList -------------------------------------------------------
void MessageListImpl::add(MessagePtr message)
{
   mMessages.pushBack(message);
}

void MessageListImpl::clear()
{
   mMessages.resize(0);
}

int MessageListImpl::send()
{
   int newIndex = 0;
   for(int i = 0; i < mMessages.getSize(); i++)
   {
      if(newIndex < i)
         mMessages[newIndex] = mMessages[i];
      if(mMessages[newIndex]->send())
         ++newIndex;
   }
   mMessages.resize(newIndex);
   return newIndex;
}

int MessageListImpl::clean()
{
   int newIndex = 0;
   for(int i = 0; i < mMessages.getSize(); i++)
   {
      if(newIndex < i)
         mMessages[newIndex] = mMessages[i];
      if(!mMessages[newIndex]->isDisabled())
         ++newIndex;
   }
   mMessages.resize(newIndex);
   return newIndex;
}

// factory function ==================================================
MessageListPtr makeMessageListImpl()
{
   return spnew MessageListImpl;
}

// ===================================================================

} //namespace messages
} //namespace framework
