/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/05/21 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef framework_messages_MessageReceiver_h__INCLUDED__
#define framework_messages_MessageReceiver_h__INCLUDED__

#include <framework/common/all.h>

namespace framework {
namespace internal {implementation MessageImpl;}
namespace messages {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.

interface Message;
typedef SmartPointer<Message> MessagePtr;
interface MessageReceiver;
typedef SmartPointer<MessageReceiver> MessageReceiverPtr;

/* interface MessageReceiver
 *
 * Abstract interface for objects which receive messages.  See
 * package.kake for information about the message handling system. */
interface MessageReceiver: implements Object
{
   MessageReceiver();
   virtual ~MessageReceiver();

   /* Called when a message owned by this receiver arrives at the top
    * of the queue. */
   virtual void receive(MessagePtr message) = 0;

private:
   /* Smart pointer to a dumb pointer to itself.  When the message
    * receiver is destroyed, the dumb pointer is set to null. */
   SmartPointer<MessageReceiver*> mWeakLink;
   friend class internal::MessageImpl;
};

// ===================================================================

} //namespace messages
} //namespace framework

#endif //included
