/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#ifndef framework_util_Atom_h__INCLUDED
#define framework_util_Atom_h__INCLUDED

#include <framework/common/all.h>

namespace framework {
namespace util {
using namespace ::framework::common;

// class definition ==================================================
class Atom;

typedef SmartPointer<Atom> AtomPtr;

//An Atom is a globally unique identifier, assigned at run-time.  No two
// calls to Atom::newAtom() will produce the same value.
//   Atom atom1 = Atom::newAtom(null);
//   Atom atom2 = Atom::newAtom(null);
//   Atom atom3 = atom1;
//   assert(atom1 == atom3);
//   assert(atom1 != atom2);
//   assert(atom3 != atom2);
class Atom
{
public:
   //newAtom() produces a new atom, which is different from any other atom
   // produced by any previous call to newAtom().  The name of the atom is
   // used for printing (for debugging purposes).  Two different atoms may
   // have the same name.
   static Atom newAtom(String name = null);

   inline Atom(): mIndex(0) {}
   inline Atom(NullType): mIndex(0) {}

   inline bool operator==(Atom other) const
      {return mIndex == other.mIndex;}
   inline bool operator!=(Atom other) const
      {return mIndex != other.mIndex;}

   //The sorting order for atoms has absolutely nothing to do with anything,
   // and is different every time the program is run.  These are only provided
   // to allow you to, say, use atoms as keys to an associative container.
   inline bool operator<=(Atom other) const
      {return mIndex <= other.mIndex;}
   inline bool operator>=(Atom other) const
      {return mIndex >= other.mIndex;}
   inline bool operator< (Atom other) const
      {return mIndex <  other.mIndex;}
   inline bool operator> (Atom other) const
      {return mIndex >  other.mIndex;}

   String getName();  //for debugging only

private:
   int mIndex;

   inline Atom(int index): mIndex(index) {}
};

} //namespace util
} //namespace framework

#endif //included
