/* Evlan - the Evlan programming language.
 * Copyright (C) 2002-2005 Kenton Varda
 * <http://www.evlan.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef framework_util_Pattern_h__INCLUDED
#define framework_util_Pattern_h__INCLUDED

// includes ==========================================================
#include <framework/common/all.h>

// begin namespace ===================================================
namespace framework {
namespace util {
using namespace common;

/* The following is another attempt to bolt features onto C++.  It isn't
 * pretty, but it'll get the job done...  at least until all of this is
 * rewritten in Evlan. */

struct PatternBase
{
   const int value;

   inline explicit PatternBase(int id)
      : value(id) {}
};

template<typename Type1_>
struct PatternValue1: public PatternBase
{
   typedef Type1_ Type1;
   const Type1 arg1;
   inline PatternValue1(int id, Type1 a1)
      : PatternBase(id), arg1(a1) {}
};

template<typename Type1_, typename Type2_>
struct PatternValue2: public PatternBase
{
   typedef Type1_ Type1;
   typedef Type2_ Type2;
   const Type1 arg1;
   const Type2 arg2;
   inline PatternValue2(int id, Type1 a1, Type2 a2)
      : PatternBase(id), arg1(a1), arg2(a2) {}
};

template<typename Type1_, typename Type2_, typename Type3_>
struct PatternValue3: public PatternBase
{
   typedef Type1_ Type1;
   typedef Type2_ Type2;
   typedef Type3_ Type3;
   const Type1 arg1;
   const Type2 arg2;
   const Type3 arg3;
   inline PatternValue3(int id, Type1 a1, Type2 a2, Type3 a3)
      : PatternBase(id), arg1(a1), arg2(a2), arg3(a3) {}
};

template<typename Type1_, typename Type2_, typename Type3_, typename Type4_>
struct PatternValue4: public PatternBase
{
   typedef Type1_ Type1;
   typedef Type2_ Type2;
   typedef Type3_ Type3;
   typedef Type4_ Type4;
   const Type1 arg1;
   const Type2 arg2;
   const Type3 arg3;
   const Type4 arg4;
   inline PatternValue4(int id, Type1 a1, Type2 a2, Type3 a3, Type4 a4)
      : PatternBase(id), arg1(a1), arg2(a2), arg3(a3), arg4(a4) {}
};

template<int i>
struct PatternValueTag
{
   static const int id = i;
};

typedef SmartPointer<PatternBase> PatternBasePtr;

#define FW_PATTERN_START(CLASSNAME)                                    \
   class CLASSNAME                                                     \
   {                                                                   \
   public:                                                             \
      inline CLASSNAME()                                               \
         : PATTERN_value(null) {}                                      \
      inline CLASSNAME(NullType)                                       \
         : PATTERN_value(null) {}                                      \
      inline CLASSNAME(::framework::util::PatternBasePtr v)            \
         : PATTERN_value(v) {}                                         \
      ::framework::util::PatternBasePtr PATTERN_value;                 \
      inline int getId() const {return PATTERN_value->value;}          \
      typedef CLASSNAME Self;                                          \
      typedef ::framework::util::PatternValueTag<0>

#define FW_PATTERN_VALUE_0(TAG)                                            \
   TAG##_PATTERN_ID;                                                       \
   static const int TAG##_ID = TAG##_PATTERN_ID::id;                       \
   typedef ::framework::util::PatternBase TAG##_PATTERN_TYPE;              \
   static inline Self TAG()                                                \
      {return spnew ::framework::util::PatternBase(TAG##_PATTERN_ID::id);} \
   typedef ::framework::util::PatternValueTag<TAG##_PATTERN_ID::id + 1>

#define FW_PATTERN_VALUE_1(TAG, TYPE1)                                     \
   TAG##_PATTERN_ID;                                                       \
   static const int TAG##_ID = TAG##_PATTERN_ID::id;                       \
   typedef ::framework::util::PatternValue1<TYPE1> TAG##_PATTERN_TYPE;     \
   static inline Self TAG(                                                 \
      TYPE1 arg1)                                                          \
   {                                                                       \
      return Self(spnew TAG##_PATTERN_TYPE(                                \
         TAG##_PATTERN_ID::id, arg1));                                     \
   }                                                                       \
   typedef ::framework::util::PatternValueTag<TAG##_PATTERN_ID::id + 1>

#define FW_PATTERN_VALUE_2(TAG, TYPE1, TYPE2)                                  \
   TAG##_PATTERN_ID;                                                           \
   static const int TAG##_ID = TAG##_PATTERN_ID::id;                           \
   typedef ::framework::util::PatternValue2<TYPE1, TYPE2> TAG##_PATTERN_TYPE;  \
   static inline Self TAG(                                                     \
      TYPE1 arg1, TYPE2 arg2)                                                  \
   {                                                                           \
      return Self(spnew TAG##_PATTERN_TYPE(                                    \
         TAG##_PATTERN_ID::id, arg1, arg2));                                   \
   }                                                                           \
   typedef ::framework::util::PatternValueTag<TAG##_PATTERN_ID::id + 1>

#define FW_PATTERN_VALUE_3(TAG, TYPE1, TYPE2, TYPE3)                                 \
   TAG##_PATTERN_ID;                                                                 \
   static const int TAG##_ID = TAG##_PATTERN_ID::id;                                 \
   typedef ::framework::util::PatternValue3<TYPE1, TYPE2, TYPE3> TAG##_PATTERN_TYPE; \
   static inline Self TAG(                                                           \
      TYPE1 arg1, TYPE2 arg2, TYPE3 arg3)                                            \
   {                                                                                 \
      return Self(spnew TAG##_PATTERN_TYPE(                                          \
         TAG##_PATTERN_ID::id, arg1, arg2, arg3));                                   \
   }                                                                                 \
   typedef ::framework::util::PatternValueTag<TAG##_PATTERN_ID::id + 1>

#define FW_PATTERN_VALUE_4(TAG, TYPE1, TYPE2, TYPE3, TYPE4)                                 \
   TAG##_PATTERN_ID;                                                                        \
   static const int TAG##_ID = TAG##_PATTERN_ID::id;                                        \
   typedef ::framework::util::PatternValue4<TYPE1, TYPE2, TYPE3, TYPE4> TAG##_PATTERN_TYPE; \
   static inline Self TAG(                                                                  \
      TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4)                                       \
   {                                                                                        \
      return Self(spnew TAG##_PATTERN_TYPE(                                                 \
         TAG##_PATTERN_ID::id, arg1, arg2, arg3, arg4));                                    \
   }                                                                                        \
   typedef ::framework::util::PatternValueTag<TAG##_PATTERN_ID::id + 1>

#define FW_PATTERN_END                                                     \
      PATTERN_LAST_ID;                                                     \
   };

#define FW_MATCH_START(VAR)                                                \
   {                                                                       \
      ::framework::util::PatternBasePtr PATTERN_value = VAR.PATTERN_value; \
      switch(PATTERN_value->value)                                         \
      {                                                                    \
      case -1:                                                             \
         {

#define FW_MATCH_CASE_0(VALUE)                                             \
      } break;                                                             \
   case VALUE##_PATTERN_ID::id:                                            \
      {

#define FW_MATCH_CASE_1(VALUE, NAME1)                                                                                                             \
      } break;                                                                                                                                    \
   case VALUE##_PATTERN_ID::id:                                                                                                                   \
      {                                                                                                                                           \
         VALUE##_PATTERN_TYPE::Type1 NAME1 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg1;

#define FW_MATCH_CASE_2(VALUE, NAME1, NAME2)                                                                                                      \
      } break;                                                                                                                                    \
   case VALUE##_PATTERN_ID::id:                                                                                                                   \
      {                                                                                                                                           \
         VALUE##_PATTERN_TYPE::Type1 NAME1 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg1; \
         VALUE##_PATTERN_TYPE::Type2 NAME2 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg2;

#define FW_MATCH_CASE_3(VALUE, NAME1, NAME2, NAME3)                                                                                               \
      } break;                                                                                                                                    \
   case VALUE##_PATTERN_ID::id:                                                                                                                   \
      {                                                                                                                                           \
         VALUE##_PATTERN_TYPE::Type1 NAME1 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg1; \
         VALUE##_PATTERN_TYPE::Type2 NAME2 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg2; \
         VALUE##_PATTERN_TYPE::Type3 NAME3 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg3;

#define FW_MATCH_CASE_4(VALUE, NAME1, NAME2, NAME3, NAME4)                                                                                        \
      } break;                                                                                                                                    \
   case VALUE##_PATTERN_ID::id:                                                                                                                   \
      {                                                                                                                                           \
         VALUE##_PATTERN_TYPE::Type1 NAME1 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg1; \
         VALUE##_PATTERN_TYPE::Type2 NAME2 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg2; \
         VALUE##_PATTERN_TYPE::Type3 NAME3 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg3; \
         VALUE##_PATTERN_TYPE::Type4 NAME4 __attribute__((unused)) = PATTERN_value.reinterpretCast<SmartPointer<VALUE##_PATTERN_TYPE > >()->arg4;

#define FW_MATCH_DEFAULT      \
      } break;                \
   default:                   \
      {

#define FW_MATCH_END          \
         } break;             \
      }                       \
   }

} //namespace util
} //namespace framework

#endif //included
