/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2004/01/27 (KV)
 *  + First Version
 */

// ===================================================================

#ifndef framework_util_TableBuilder_h__INCLUDED__
#define framework_util_TableBuilder_h__INCLUDED__

#include <framework/common/all.h>
#include <map>

namespace framework {
namespace util {
using namespace ::framework::common;

// ===================================================================
// AUTHOR (KV): Created module.
// TODO (KV): Document framework::util::TableBuilder
// TODO (KV): Test framework::util::TableBuilder

template<typename Type, typename Compare = std::less<Type> >
class TableBuilder
{
public:
   TableBuilder(): mSize(0) {}

   int lookup(Type item)
   {
      typename std::map<Type, int, Compare>::iterator iter = mMap.find(item);
      if(iter == mMap.end())
      {
         int index = mSize++;
         mMap[item] = index;
         return index;
      }
      else
         return iter->second;
   }

   Array<Type> makeTable()
   {
      Array<Type> result(mSize);
      for(typename std::map<Type, int, Compare>::iterator iter = mMap.begin(); iter != mMap.end(); ++iter)
         result[iter->second] = iter->first;
      return result;
   }

private:
   std::map<Type, int, Compare> mMap;
   int mSize;
};

// ===================================================================

} //namespace util
} //namespace framework

#endif //included
