/* Kenton Varda's C++ framework code.
 * Copyright (C) 2002-2005 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

/* Revision History:
 * 2003/04/11 (KV)
 *  + Imported code from Evlan.
 */

// ===================================================================

#ifndef framework_util_Vector_h__INCLUDED
#define framework_util_Vector_h__INCLUDED

#include <framework/common/all.h>

namespace framework {
namespace util {
using namespace ::framework::common;

// class definition ==================================================
/* This is just a vector implementation that uses smart pointers.  Can
 * be much less cumbersome than STL's implementation in some cases. */
template <typename Element>
class Vector
{
public:
   inline Vector(): mSize(0) {}
   inline Vector(int reserve): mArray(reserve), mSize(0) {}

   void pushBack(Element element)
   {
      if(mArray.getSize() == mSize)
      {
         Array<Element> newArray;
         if(mSize > 0)
            newArray = Array<Element>(mSize * 2);
         else
            newArray = Array<Element>(1);

         for(int i = 0; i < mSize; i++)
            newArray[i] = mArray[i];
         mArray = newArray;
      }

      mArray[mSize] = element;
      ++mSize;
   }

   void popBack()
   {
      --mSize;
      mArray[mSize].~Element();
      new(&mArray[mSize]) Element;
   }

   Element& getBack() const
   {
      return mArray[mSize-1];
   }

   void clear()
   {
      mArray = null;
      mSize = 0;
   }

   void resize(int size)
   {
      if(mSize < size)
      {
         if(mArray.getSize() < size)
         {
            int newSize = mSize;
            if(newSize == 0)
               newSize = 1;
            while(newSize < size)
               newSize *= 2;
            Array<Element> newArray(newSize);
            for(int i = 0; i < mSize; i++)
               newArray[i] = mArray[i];
            mArray = newArray;
         }
         mSize = size;
      }
      else while(mSize > size)
      {
         mArray[--mSize].~Element();
         new(&mArray[mSize]) Element;
      }
   }

   inline int getSize() const
      {return mSize;}

   inline Element& operator[](int index) const
   {
#ifdef DEBUG
      if(index < 0 || index >= mSize)
         ThrowOutOfBoundsException(mSize, index);
#endif
      return mArray[index];
   }

   //Once you call getArray(), you should not modify the Vector any further.
   inline Array<Element> getArray() const
   {
      return mArray.getInterval(0, mSize);
   }

private:
   Array<Element> mArray;
   int mSize;
};

} //namespace util
} //namespace framework

#endif //included
