/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_Binary_H__INCLUDED__
#define kake_Binary_H__INCLUDED__

// includes ==========================================================
#include "Common.h"
#include "Module.h"

namespace kake {

// class definition ==================================================
class Binary: public Buildable
{
public:
   enum Type
   {
      EXECUTABLE,
      LIBRARY,
      STATIC_LIBRARY
   };

   Binary(string name, Type type, string sourcefile, int linenum);
   ~Binary();

   BuildResult Build();

   bool Check();
   bool Clean();
   bool Document();

   void GetRequiredHeaders(vector<string >& list);
   void GetRequiredLibraries(vector<string >& list);

   void GetObjects(vector<string >& objects);

   string GetName();
   string GetNameWithUnderscores();

   void AddObject(Buildable* object);
   inline void SetMainModule(Module* module) {mMainObject = module;}

   void AddCFlag(string cflag);
   void AddLdFlag(string ldflag);
   void AddDefine(string name, string value);

private:
   string mName;

   Type mType;

   vector<Buildable* > mObjects;
   Module* mMainObject;

   string mSourceFile;
   int mLineNum;

   vector<string > mCFlags;
   vector<string > mLdFlags;
   vector<pair<string, string> > mDefines;

   bool mBuilt;
   bool mCleaned;
   bool mChecked;
   bool mDocumented;

   BuildResult mBuildResult;
   bool mCleanResult;
   bool mCheckResult;
   bool mDocumentResult;

   //disable copy
   Binary(const Binary& other);
   void operator=(const Binary& other);
};

} //namespace kake

#endif //included
