/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_Buildable_H__INCLUDED__
#define kake_Buildable_H__INCLUDED__

// includes ==========================================================
#include "Common.h"
#include <vector>
#include <string>

namespace kake {
using namespace std;

// class definition ==================================================

//Buildable is the base class for Package, Module, Executable, Library,
// and anything else that can be built.
class Buildable
{
public:
   virtual ~Buildable();

   enum BuildResult
   {
      UPDATED,
      FAILED,
      SKIPPED
   };

   virtual BuildResult Build() = 0;

   virtual bool Check()    = 0;
   virtual bool Clean()    = 0;
   virtual bool Document() = 0;

   virtual void GetRequiredHeaders(vector<string >& list)   = 0;
   virtual void GetRequiredLibraries(vector<string >& list) = 0;

   virtual void GetObjects(vector<string >& objects) = 0;

   virtual string GetName() = 0;
   virtual string GetNameWithUnderscores() = 0;
};

} //namespace kake

#endif //included
