/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_BuildfileElement_H__INCLUDED__
#define kake_BuildfileElement_H__INCLUDED__

// includes ==========================================================
#include "ParseElement.h"
#include "Package.h"

namespace kake {
using namespace std;

class Binary;

// class definition ==================================================
class BuildfileElement: public ParseElement
{
public:
   BuildfileElement();
   ~BuildfileElement();

   Binary* GetBinary(string name);
   void GetAllBinaries(vector<Binary* >& binaries);

   enum CleanLevel
   {
      CLEAN,
      DISTCLEAN,
      CVSCLEAN
   };
   bool Clean(CleanLevel level);

   void InPublic(bool flag) throw(ParseException);
   ParseElement* StartSubElement(string type, string name, const vector<Token >& flags,
                                 string sourceFile, int linenum) throw(ParseException);
   void EndSubElement(ParseElement* element) throw(ParseException);
   void ProcessCodeLine(const vector<Token >& tokens,
                        string sourceFile, int linenum) throw(ParseException);

   class PackageElement;
   class ModuleElement;
   class BinaryElement;
   class CleanElement;

   friend class PackageElement;
   friend class ModuleElement;
   friend class BinaryElement;
   friend class CleanElement;

private:
   static void SeparateAtSpaces(vector<string >& output, string input);

   Package mRootPackage;
   map<string, Binary* > mBinaries;

   vector<string > mCleanList;
   vector<string > mDistcleanList;
   vector<string > mCvscleanList;

   //disable copy
   BuildfileElement(const BuildfileElement& other);
   void operator=(const BuildfileElement& other);
};

} //namespace kake

#endif //included
