/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_Compiler_H__INCLUDED__
#define kake_Compiler_H__INCLUDED__

// includes ==========================================================
#include "Common.h"
#include <stack>
#include <vector>
#include <string>

namespace kake {
using namespace std;

// class definition ==================================================
class Compiler
{
public:
   //Compile() compiles a source code file into an object file.
   static bool Compile(string sourceFile, string objectFile, bool outputDeps);
   //GetDependencies() gets a list of all files that were included in the given
   // object file.
   static void GetDependencies(vector<string >& target, string objectFile);

   //LinkExecutable() links the given objects together into an executable.
   // Note that "execname" is not necessarily the output filename.  Pass execname to
   // GetExecutableFileForName() to find out what the output file will be.
   static bool LinkExecutable(const vector<string >& objects, string execname);
   //LinkLibrary() links the given objects together into a dynamic library.
   // Note that "libname" is not necessarily the output filename.  Pass libname to
   // GetLibraryFileForName() to find out what the output file will be.
   static bool LinkLibrary(const vector<string >& objects, string libname);
   static bool LinkStaticLibrary(const vector<string >& objects, string libname);

   //These functions take an executable or library name and determine what the
   // filename should be.  The input name is an arbirary name, like "kake"
   // or "gauge3d".  The returned value is system-dependent, and will be the
   // path and name of the output file, such as "bin/kake", "lib/libgauge3d.so",
   // or "bin/gauge3d.dll".  Note that GetDllFileForName returns the run-time
   // file needed for this library while GetLibraryFileForName returns the link-time
   // file.  On most systems, these two are the same.  Windows is the main exception
   // to this rule.
   static string GetExecutableFileForName(string execname);
   static string GetDllFileForName(string libname);
   static string GetLibraryFileForName(string libname);
   static string GetStaticLibraryFileForName(string libname);

   static void SetDebug(bool debug);

   static void AddDefine(string name, string value);
   static void AddDefine(string name);
   static void AddDefines(const vector<pair<string, string > >& defines);
   static void ClearDefines();

#if 0
   static void AddIncludeDir(string dir);
   static void AddIncludeDirs(const vector<string >& includeDirs);
   static void ClearIncludeDirs();

   static void AddLibraryDir(string dir);
   static void AddLibraryDirs(const vector<string >& dirs);
   static void ClearLibraryDirs();
#endif

   static void AddLibrary(string library);
   static void AddLibraries(const vector<string >& libraries);
   static void ClearLibraries();

   static void AddParameter(string param);
   static void AddParameters(const vector<string >& params);
   static void ClearParameters();

   static void AddLinkerParameter(string param);
   static void AddLinkerParameters(const vector<string >& params);
   static void ClearLinkerParameters();
};

} //namespace kake

#endif //included
