/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_Filesystem_H__INCLUDED__
#define kake_Filesystem_H__INCLUDED__

// includes ==========================================================
#include "Common.h"
#include <string>
#include <vector>

namespace kake {
using namespace std;

// class definition ==================================================
class Filesystem
{
public:
   static bool Exists(string filename);
   static void MakeDirectory(string dirname);
   static void MakeContainingDirectory(string filename);

   static void GetDirectoryTree(vector<string >& dirs, string basedir);

   static void Delete(string filename);
   static void RemoveDirectory(string dirname);

   static void Copy(string source, string destination);
   static unsigned int GetTimestamp(string filename);

   //redirects stderr to the given file.
   static void RedirectStderrToFile(string filename);
   //un-does the effects of RedirectStderrToFile().
   static void RestoreStderr();
   //returns true if something has been written to stderr since the last time
   // the function was called.  (only works if output is being redirected to
   // a file -- returns false otherwise)
   static bool CheckStderr();

   inline static bool IsStderrRedirected() {return mStderrBackup != -1;}

private:
   static int mStderrBackup;

   //disable construct
   Filesystem();
   Filesystem(const Filesystem& other);
};

} //namespace kake

#endif //included
