/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_Package_H__INCLUDED__
#define kake_Package_H__INCLUDED__

// includes ==========================================================
#include "Common.h"
#include <map>
#include "Buildable.h"

namespace kake {
using namespace std;

class Module;

// class definition ==================================================
class Package: public Buildable
{
public:
   Package(string name, bool isPublic, string sourcefile, int linenum);
   ~Package();

   BuildResult Build();

   bool Check();
   bool Clean();
   bool Document();

   void AddRequiredHeader(string header);
   void AddRequiredLibrary(string library);
   void GetRequiredHeaders(vector<string >& list);
   void GetRequiredLibraries(vector<string >& list);

   void GetObjects(vector<string >& objects);

   string GetName();
   string GetNameWithUnderscores();
   string GetFilename();

   Package* AddSubPackage(string name, bool isPublic, string sourceFile, int lineNum);
   Module* AddModule(string name, bool isPublic, string sourceFile, int lineNum);
   Buildable* GetBuildable(string name);

private:
   string mName;
   string mFilename;
   string mNameWithUnderscores;

   bool mIsPublic;

   map<string, Buildable* > mBuildableMap;
   vector<Buildable* > mBuildableList;

   vector<string > mRequiredHeaders;
   vector<string > mRequiredLibraries;

   string mSourceFile;
   int mLineNum;

   bool mBuilt;
   bool mCleaned;
   bool mChecked;
   bool mDocumented;

   BuildResult mBuildResult;
   bool mCleanResult;
   bool mCheckResult;
   bool mDocumentResult;

   //disable copy
   Package(const Package& other);
   void operator=(const Package& other);
};

} //namespace kake

#endif //included
