/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_Parser_H__INCLUDED__
#define kake_Parser_H__INCLUDED__

// includes ==========================================================
#include "Common.h"
#include <stack>
#include "ParseElement.h"

namespace kake {
using namespace std;

// class definition ==================================================
class Parser
{
public:
   Parser();
   ~Parser();

   void Start(ParseElement* rootElement);
   void Parse(string filename);
   void Parse(const char* data, int size);
   void Finish();

   inline bool HadErrors() {return mHadErrors;}

private:
   typedef ParseElement::Token Token;

   void PrintError(string description);
   void ProcessToken(Token token);

   stack<ParseElement* > mElementStack;

   string mTokenBuffer;
   vector<Token > mCodeLine;

   bool mJustParsedNewline;
   bool mJustParsedColon;
   bool mEscapedChar;
   bool mInString;
   bool mHadErrors;

   bool mInDirective;
   enum
   {
      START_DIRECTIVE,
      GOT_LINENUM,
      GOT_FILENAME
   } mDirectiveState;

   int mLineNum;
   string mFilename;

   //This is a type of parse element that just ignores everything.
   class IgnoreElement: public ParseElement
   {
   public:
      IgnoreElement();
      ~IgnoreElement();

      void InPublic(bool flag) throw(ParseException);
      ParseElement* StartSubElement(string type, string name, const vector<Token >& flags,
                                    string sourceFile, int linenum) throw(ParseException);
      void EndSubElement(ParseElement* element) throw(ParseException);
      void ProcessCodeLine(const vector<Token >& tokens,
                           string sourceFile, int linenum) throw(ParseException);
   };

   //disable copy
   Parser(const Parser& other);
   void operator=(const Parser& other);
};

} //namespace kake

#endif //included
