/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

// includes ==========================================================
#include "Preprocessor.h"
#include <iostream>

namespace kake {

static const string PREPROCESSOR_PROGRAM = "gcc";

// class member functions ============================================
Preprocessor::Preprocessor()
   : mHadErrors(false)
{
}

Preprocessor::~Preprocessor()
{
}

void Preprocessor::Preprocess(string filename)
{
   if(mProcess.IsRunning())
      mProcess.Wait();

   vector<string > params;
   params.push_back("-E");
   params.push_back("-x");
   params.push_back("c++");

   for(unsigned int i = 0; i < mDefines.size(); i++)
      params.push_back("-D" + mDefines[i].first + "=" + mDefines[i].second);
   for(unsigned int i = 0; i < mIncludes.size(); i++)
      params.push_back("-I" + mIncludes[i]);

   params.push_back("-I./src");

   params.push_back(filename);

   mProcess.Run(PREPROCESSOR_PROGRAM, params, true);
}

int Preprocessor::Read(void* buffer, int size)
{
   int numread = mProcess.Read(buffer, size);
   if(numread <= 0 && mProcess.IsRunning())
   {
      if(mProcess.Wait() != 0)
         mHadErrors = true;
   }
   return numread;
}

bool Preprocessor::Test()
{
   Process::SetEcho(true);

   Preprocessor preproc;

   preproc.AddDefine("TESTDEF", "value");
   preproc.AddIncludeDir(".");

   preproc.Preprocess("Preprocessor.cpp");

   char buf[1024];
   int size;

   while((size = preproc.Read(buf, 1024)) > 0)
   {
      cout.write(buf, size);
      cout.flush();
   }

   return true;
}

} //namespace kake
