/* kake - A Semi-Replacement for Make
 * Copyright (C) 2001 Kenton Varda
 * <temporal@gauge3d.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 */

// ===================================================================

#ifndef kake_Preprocessor_H__INCLUDED__
#define kake_Preprocessor_H__INCLUDED__

// includes ==========================================================
#include "Common.h"
#include <utility>
#include "Process.h"

namespace kake {
using namespace std;

// class definition ==================================================
class Preprocessor
{
public:
   Preprocessor();
   ~Preprocessor();

   void Preprocess(string filename);
   int Read(void* buffer, int size);
   inline bool HadErrors() {return mHadErrors;}

   inline void AddDefine(string name, string value)
      { mDefines.push_back(pair<string, string >(name, value)); }
   inline void AddDefine(string name)
      { AddDefine(name, "1"); }
   inline void AddDefines(const vector<pair<string, string > >& defines)
      { mDefines.insert(mDefines.end(), defines.begin(), defines.end()); }
   inline void ClearDefines()
      { mDefines.clear(); }

   inline void AddIncludeDir(string dir)
      { mIncludes.push_back(dir); }
   inline void AddIncludeDirs(const vector<string >& includes)
      { mIncludes.insert(mIncludes.end(), includes.begin(), includes.end()); }
   inline void ClearIncludeDirs()
      { mIncludes.clear(); }

   static bool Test();

private:
   Process mProcess;
   bool mHadErrors;

   vector<pair<string, string > > mDefines;
   vector<string > mIncludes;

   //disable copy
   Preprocessor(const Preprocessor& other);
   void operator=(const Preprocessor& other);
};

} //namespace kake

#endif //included
